@@ -3,7 +3,7 @@
 AI.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -28,14 +28,10 @@ MA 02110-1301, USA.
 #import "OOWeakReference.h"
 #import "OOCacheManager.h"
 #import "OOCollectionExtractors.h"
+#import "OOPListParsing.h"
 
 #import "ShipEntity.h"
-
-#ifdef OO_BRAIN_AI
-#import "OOInstinct.h"
-#endif
-
-#define kOOLogUnconvertedNSLog @"unclassified.AI"
+#import "ShipEntityAI.h"
 
 
 enum
@@ -59,13 +55,17 @@ static AI *sCurrentlyRunningAI = nil;
 @interface AI (OOPrivate)
 
 // Wrapper for performSelector:withObject:afterDelay: to catch/fix bugs.
-- (void)performDeferredCall:(SEL)selector withObject:(id)object afterDelay:(NSTimeInterval)delay;
-+ (void)deferredCallTrampolineWithInfo:(NSValue *)info;
+- (void) performDeferredCall:(SEL)selector withObject:(id)object afterDelay:(NSTimeInterval)delay;
++ (void) deferredCallTrampolineWithInfo:(NSValue *)info;
 
-- (void)refreshOwnerDesc;
+- (void) refreshOwnerDesc;
+
+// Set state machine and state without side effects.
+- (void) directSetStateMachine:(NSDictionary *)newSM name:(NSString *)name;
+- (void) directSetState:(NSString *)state;
 
 // Loading/whitelisting
-- (NSDictionary *) loadStateMachine:(NSString *)smName;
+- (NSDictionary *) loadStateMachine:(NSString *)smName jsName:(NSString *)script;
 - (NSDictionary *) cleanHandlers:(NSDictionary *)handlers forState:(NSString *)stateKey stateMachine:(NSString *)smName;
 - (NSArray *) cleanActions:(NSArray *)actions forHandler:(NSString *)handlerKey state:(NSString *)stateKey stateMachine:(NSString *)smName;
 
@@ -77,6 +77,31 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 #endif
 
 
+@interface OOPreservedAIStateMachine: NSObject
+{
+@private
+	NSDictionary		*_stateMachine;
+	NSString			*_name;
+	NSString			*_state;
+	NSMutableSet		*_pendingMessages;
+	NSString      *_jsScript;
+}
+
+- (id) initWithStateMachine:(NSDictionary *)stateMachine
+					   name:(NSString *)name
+					  state:(NSString *)state
+			pendingMessages:(NSSet *)pendingMessages
+									 jsScript:(NSString *)script;
+
+- (NSDictionary *) stateMachine;
+- (NSString *) name;
+- (NSString *) state;
+- (NSSet *) pendingMessages;
+- (NSString *) jsScript;
+
+@end
+
+
 @implementation AI
 
 + (AI *) currentlyRunningAI
@@ -100,26 +125,25 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 
 - (id) init
 {
-	self = [super init];
-	
-	aiStack = [[NSMutableArray alloc] init];
-	pendingMessages = [[NSMutableSet alloc] init];
-	
-	nextThinkTime = INFINITY;	// don't think for a while
-	thinkTimeInterval = AI_THINK_INTERVAL;
-	
-	stateMachineName = [[NSString stringWithString:@"<no AI>"] retain];	// no initial brain
+	if ((self = [super init]))
+	{
+		nextThinkTime = INFINITY;	// don't think for a while
+		thinkTimeInterval = AI_THINK_INTERVAL;
+		
+		stateMachineName = @"<no AI>";	// no initial brain
+	}
 	
 	return self;
 }
 
 
-- (id) initWithStateMachine:(NSString *) smName andState:(NSString *) stateName
+- (id) initWithStateMachine:(NSString *)smName andState:(NSString *)stateName
 {
-	self = [self init];
-	
-	if (smName != nil)  [self setStateMachine:smName];
-	if (stateName != nil)  currentState = [stateName retain];
+	if ((self = [self init]))
+	{
+		if (smName != nil)  [self setStateMachine:smName withJSScript:@"oolite-nullAI.js"];
+		if (stateName != nil)  currentState = [stateName retain];
+	}
 	
 	return self;
 }
@@ -127,15 +151,19 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 
 - (void) dealloc
 {
-	[_owner release];
-	[ownerDesc release];
-	[aiStack release];
-	[stateMachine release];
-	[stateMachineName release];
-	[currentState release];
-	[pendingMessages release];
+	if (sCurrentlyRunningAI == self)
+	{
+		sCurrentlyRunningAI = nil;
+	}
 	
-	if (sCurrentlyRunningAI == self)  sCurrentlyRunningAI = nil;
+	DESTROY(_owner);
+	DESTROY(ownerDesc);
+	DESTROY(aiStack);
+	DESTROY(stateMachine);
+	DESTROY(stateMachineName);
+	DESTROY(currentState);
+	DESTROY(pendingMessages);
+	DESTROY(jsScript);
 	
 	[super dealloc];
 }
@@ -149,24 +177,10 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 
 - (NSString *) shortDescriptionComponents
 {
-	return [NSString stringWithFormat:@"%@:%@", stateMachineName, currentState];
+	return [NSString stringWithFormat:@"%@:%@ / %@", stateMachineName, currentState, [stateMachine objectForKey:@"jsScript"]];
 }
 
 
-#ifdef OO_BRAIN_AI
-- (OOInstinct *) rulingInstinct
-{
-	return rulingInstinct;
-}
-
-
-- (void) setRulingInstinct:(OOInstinct*) instinct
-{
-	rulingInstinct = instinct;
-}
-#endif
-
-
 - (ShipEntity *)owner
 {
 	ShipEntity		*owner = [_owner weakRefUnderlyingObject];
@@ -201,11 +215,11 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 		{
 			OOLogIndent();
 			
-			unsigned count = [aiStack count];
+			NSUInteger count = [aiStack count];
 			while (count--)
 			{
-				NSDictionary *pickledMachine = [aiStack objectAtIndex:count];
-				OOLog(@"ai.error.stackOverflow.dump", @"%3u: %@: %@", count, [pickledMachine oo_stringForKey:@"stateMachineName"], [pickledMachine oo_stringForKey:@"currentState"]);
+				OOPreservedAIStateMachine *preservedMachine = [aiStack objectAtIndex:count];
+				OOLog(@"ai.error.stackOverflow.dump", @"%3lu: %@: %@", count, [preservedMachine name], [preservedMachine state]);
 			}
 			
 			OOLogOutdent();
@@ -216,17 +230,12 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 
 - (void) preserveCurrentStateMachine
 {
-	if (!stateMachine)
-		return;
+	if (stateMachine == nil)  return;
 	
-	NSMutableDictionary *pickledMachine = [NSMutableDictionary dictionaryWithCapacity:3];
-	
-	[pickledMachine setObject:stateMachine forKey:@"stateMachine"];
-	[pickledMachine setObject:currentState forKey:@"currentState"];
-	[pickledMachine setObject:stateMachineName forKey:@"stateMachineName"];
-	[pickledMachine setObject:[[pendingMessages copy] autorelease] forKey:@"pendingMessages"];
-	
-	if (aiStack == nil)  aiStack = [[NSMutableArray alloc] init];
+	if (aiStack == nil)
+	{
+		aiStack = [[NSMutableArray alloc] init];
+	}
 	
 	if ([aiStack count] >= kStackLimiter)
 	{
@@ -236,10 +245,20 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 					format:@"AI stack overflow for %@", _owner];
 	}
 	
+	OOPreservedAIStateMachine *preservedMachine = [[OOPreservedAIStateMachine alloc]
+												   initWithStateMachine:stateMachine
+																   name:stateMachineName
+																  state:currentState
+														pendingMessages:pendingMessages
+																									jsScript:[stateMachine objectForKey:@"jsScript"]];
+	
 #ifndef NDEBUG
 	if ([[self owner] reportAIMessages])  OOLog(@"ai.stack.push", @"Pushing state machine for %@", self);
 #endif
-	[aiStack insertObject:pickledMachine atIndex:0];	//  PUSH
+	
+	[aiStack addObject:preservedMachine];  // PUSH
+	
+	[preservedMachine release];
 }
 
 
@@ -247,25 +266,24 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 {
 	if ([aiStack count] == 0)  return;
 	
-	NSMutableDictionary *pickledMachine = [aiStack objectAtIndex:0];
+	OOPreservedAIStateMachine *preservedMachine = [aiStack lastObject];
 	
 #ifndef NDEBUG
 	if ([[self owner] reportAIMessages])  OOLog(@"ai.stack.pop", @"Popping previous state machine for %@", self);
 #endif
 	
-	[stateMachine release];
-	stateMachine = [[pickledMachine objectForKey:@"stateMachine"] retain];
+	[self directSetStateMachine:[preservedMachine stateMachine]
+						   name:[preservedMachine name]];
 	
-	[currentState release];
-	currentState = [[pickledMachine objectForKey:@"currentState"] retain];
-	
-	[stateMachineName release];
-	stateMachineName = [[pickledMachine objectForKey:@"stateMachineName"] retain];
+	[self directSetState:[preservedMachine state]];
 	
+	// restore JS script
+	[[self owner] setAIScript:[preservedMachine jsScript]];
+
 	[pendingMessages release];
-	pendingMessages = [[pickledMachine objectForKey:@"pendingMessages"] mutableCopy];  // restore a MUTABLE set
+	pendingMessages = [[preservedMachine pendingMessages] mutableCopy];  // restore a MUTABLE set
 	
-	[aiStack removeObjectAtIndex:0];   //  POP
+	[aiStack removeLastObject];  //  POP
 }
 
 
@@ -286,23 +304,17 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 }
 
 
-- (void) setStateMachine:(NSString *) smName
+- (void) setStateMachine:(NSString *)smName withJSScript:(NSString *)script
 {
-	NSDictionary *newSM = [self loadStateMachine:smName];
-	
+	NSDictionary *newSM = [self loadStateMachine:smName jsName:script];
+
 	if (newSM)
 	{
 		[self preserveCurrentStateMachine];
-		[stateMachine release];	// release old state machine
-		stateMachine = [newSM retain];
+		[self directSetStateMachine:newSM name:smName];
+		[self directSetState:@"GLOBAL"];
+		
 		nextThinkTime = 0.0;	// think at next tick
-		
-		[currentState release];
-		currentState = @"GLOBAL";
-		
-		// refresh stateMachineName
-		[stateMachineName release];
-		stateMachineName = [smName copy];
 
 		/*	CRASH in objc_msgSend, apparently on [self reactToMessage:@"ENTER"] (1.69, OS X/x86).
 			Analysis: self corrupted. We're being called by __NSFireDelayedPerform, which doesn't go
@@ -316,7 +328,6 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 		// refresh name
 		[self refreshOwnerDesc];
 	}
-
 }
 
 
@@ -332,11 +343,7 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 			 -- Ahruman, 20070706
 		*/
 		[self reactToMessage:@"EXIT" context:@"changing state"];
-		if (currentState != stateName)
-		{
-			[currentState release];
-			currentState = [stateName copy];
-		}
+		[self directSetState:stateName];
 		[self reactToMessage:@"ENTER" context:@"changing state"];
 	}
 }
@@ -360,13 +367,19 @@ extern void GenerateGraphVizForAIStateMachine(NSDictionary *stateMachine, NSStri
 }
 
 
+- (NSString *) associatedJS
+{
+	return [stateMachine objectForKey:@"jsScript"];
+}
+
+
 - (NSString *) state
 {
 	return [[currentState retain] autorelease];
 }
 
 
-- (unsigned) stackDepth
+- (NSUInteger) stackDepth
 {
 	return [aiStack count];
 }
@@ -397,14 +410,13 @@ static AIStackElement *sStack = NULL;
 	static unsigned	recursionLimiter = 0;
 	AI				*previousRunning = sCurrentlyRunningAI;
 	
-	
 	/*	CRASH in _freedHandler when called via -setState: __NSFireDelayedPerform (1.69, OS X/x86).
 		Analysis: owner invalid.
 		Fix: make owner an OOWeakReference.
 		 -- Ahruman, 20070706
 	*/
 	if (message == nil || owner == nil || [owner universalID] == NO_TARGET)  return;
-	
+
 #ifndef NDEBUG
 	// Push debug stack frame.
 	if (debugContext == nil)  debugContext = @"unspecified";
@@ -458,22 +470,21 @@ static AIStackElement *sStack = NULL;
 	
 	actions = [[[messagesForState objectForKey:message] copy] autorelease];
 	
-#ifdef OO_BRAIN_AI
-	if (rulingInstinct != nil)  [rulingInstinct freezeShipVars];	// preserve the pre-thinking state
-#endif
-	
 	sCurrentlyRunningAI = self;
 	if ([actions count] > 0)
 	{
 		++recursionLimiter;
-		NS_DURING
+		@try
+		{
 			for (i = 0; i < [actions count]; i++)
 			{
 				[self takeAction:[actions objectAtIndex:i]];
 			}
-		NS_HANDLER
-			OOLog(kOOLogException, @"Squashing exception %@:%@ in AI handler %@:%@.%@", [localException name], [localException reason], stateMachineName, currentState, message);
-		NS_ENDHANDLER
+		}
+		@catch (NSException *exception)
+		{
+			OOLog(kOOLogException, @"Squashing exception %@:%@ in AI handler %@:%@.%@", [exception name], [exception reason], stateMachineName, currentState, message);
+		}
 		
 		--recursionLimiter;
 	}
@@ -493,24 +504,12 @@ static AIStackElement *sStack = NULL;
 	// Unwind stack.
 	if (sStack != NULL)  sStack = sStack->back;
 #endif
-	
-#ifdef OO_BRAIN_AI
-	if (rulingInstinct != nil)
-	{
-		[rulingInstinct getShipVars];		// record the post-thinking state
-		[rulingInstinct unfreezeShipVars];	// restore the pre-thinking state (AI is now abstract thought = instincts motivate)
-	}
-#endif
 }
 
 
-- (void) takeAction:(NSString *) action
+- (void) takeAction:(NSString *)action
 {
-	NSArray			*tokens = ScanTokensFromString(action);
-	NSString		*dataString = nil;
-	NSString		*selectorStr;
-	SEL				selector;
-	ShipEntity		*owner = [self owner];
+	ShipEntity *owner = [self owner];
 	
 #ifndef NDEBUG
 	BOOL report = [owner reportAIMessages];
@@ -521,33 +520,35 @@ static AIStackElement *sStack = NULL;
 	}
 #endif
 	
-	if ([tokens count] != 0)
+	NSArray *tokens = ScanTokensFromString(action);
+	NSUInteger tokenCount = [tokens count];
+	
+	if (tokenCount != 0)
 	{
-		selectorStr = [tokens objectAtIndex:0];
+		NSString *selectorStr = [tokens objectAtIndex:0];
 		
 		if (owner != nil)
 		{
-			if ([tokens count] > 1)
+			NSString *dataString = nil;
+			
+			if (tokenCount == 2)
 			{
-				dataString = [[tokens subarrayWithRange:NSMakeRange(1, [tokens count] - 1)] componentsJoinedByString:@" "];
+				dataString = [tokens objectAtIndex:1];
+			}
+			else if ([tokens count] > 1)
+			{
+				dataString = [[tokens subarrayWithRange:NSMakeRange(1, tokenCount - 1)] componentsJoinedByString:@" "];
 			}
 			
-			selector = NSSelectorFromString(selectorStr);
+			SEL selector = NSSelectorFromString(selectorStr);
 			if ([owner respondsToSelector:selector])
 			{
-				if (dataString)  [owner performSelector:selector withObject:dataString];
+				if (dataString != nil)  [owner performSelector:selector withObject:dataString];
 				else  [owner performSelector:selector];
 			}
 			else
 			{
-				if ([selectorStr isEqual:@"debugMessage:"])
-				{
-					OOLog(@"ai.takeAction.debugMessage", @"DEBUG: AI MESSAGE from %@: %@", ownerDesc, dataString);
-				}
-				else
-				{
-					OOLogERR(@"ai.takeAction.badSelector", @"in AI %@ in state %@: %@ does not respond to %@", stateMachineName, currentState, ownerDesc, selectorStr);
-				}
+				OOLogERR(@"ai.takeAction.badSelector", @"in AI %@ in state %@: %@ does not respond to %@", stateMachineName, currentState, ownerDesc, selectorStr);
 			}
 		}
 		else
@@ -580,8 +581,11 @@ static AIStackElement *sStack = NULL;
 	
 	[self reactToMessage:@"UPDATE" context:@"periodic update"];
 
-	if ([pendingMessages count] > 0)  ms_list = [pendingMessages allObjects];
-	[pendingMessages removeAllObjects];
+	if ([pendingMessages count] > 0)
+	{
+		ms_list = [pendingMessages allObjects];
+		[pendingMessages removeAllObjects];
+	}
 	
 	if (ms_list != nil)
 	{
@@ -593,22 +597,27 @@ static AIStackElement *sStack = NULL;
 }
 
 
-- (void) message:(NSString *) ms
+- (void) message:(NSString *)ms
 {
 	if ([[self owner] universalID] == NO_TARGET)  return;  // don't think until launched
 
-	if ([pendingMessages count] > 32)
+	if (EXPECT_NOT([pendingMessages count] > 32))
 	{
-		OOLogERR(@"ai.message.failed.overflow", @"AI pending messages overflow for '%@'; pending messages:\n%@", ownerDesc, pendingMessages);
-		[NSException raise:@"OoliteException"
-					format:@"AI pendingMessages overflow for %@", ownerDesc];
+		// Generate the error, but don't crash Oolite! Fixes bug #18055 - Pending message overflow for thargoids, -> crash !
+		OOLogERR(@"ai.message.failed.overflow", @"AI message \"%@\" received by '%@' AI while pending messages stack full; message discarded. Pending messages:\n%@", ms, ownerDesc, pendingMessages);
+	}
+	else
+	{
+		if (pendingMessages == nil)
+		{
+			pendingMessages = [[NSMutableSet alloc] init];
+		}
+		[pendingMessages addObject:ms];
 	}
-	
-	[pendingMessages addObject:ms];
 }
 
 
-- (void) dropMessage:(NSString *) ms
+- (void) dropMessage:(NSString *)ms
 {
 	[pendingMessages removeObject:ms];
 }
@@ -616,7 +625,14 @@ static AIStackElement *sStack = NULL;
 
 - (NSSet *) pendingMessages
 {
-	return [[pendingMessages copy] autorelease];
+	if (pendingMessages != nil)
+	{
+		return [[pendingMessages copy] autorelease];
+	}
+	else
+	{
+		return [NSSet set];
+	}
 }
 
 
@@ -685,19 +701,6 @@ static AIStackElement *sStack = NULL;
 {
 	OOLog(@"dumpState.ai", @"State machine name: %@", stateMachineName);
 	OOLog(@"dumpState.ai", @"Current state: %@", currentState);
-#ifdef OO_BRAIN_AI
-	if (rulingInstinct!= nil && OOLogWillDisplayMessagesInClass(@"dumpState.ai.instinct"))
-	{
-		OOLog(@"dumpState.ai.instinct", @"Ruling instinct:");
-		OOLogPushIndent();
-		OOLogIndent();
-		NS_DURING
-			[rulingInstinct dumpState];
-		NS_HANDLER
-		NS_ENDHANDLER
-		OOLogPopIndent();
-	}
-#endif
 	OOLog(@"dumpState.ai", @"Next think time: %g", nextThinkTime);
 	OOLog(@"dumpState.ai", @"Next think interval: %g", thinkTimeInterval);
 }
@@ -770,7 +773,32 @@ static AIStackElement *sStack = NULL;
 }
 
 
-- (NSDictionary *) loadStateMachine:(NSString *)smName
+- (void) directSetStateMachine:(NSDictionary *)newSM name:(NSString *)name
+{
+	if (stateMachine != newSM)
+	{
+		[stateMachine release];
+		stateMachine = [newSM copy];
+	}
+	if (stateMachineName != name)
+	{
+		[stateMachineName release];
+		stateMachineName = [name copy];
+	}
+}
+
+
+- (void) directSetState:(NSString *)state
+{
+	if (currentState != state)
+	{
+		[currentState release];
+		currentState = [state copy];
+	}
+}
+
+
+- (NSDictionary *) loadStateMachine:(NSString *)smName jsName:(NSString *)script
 {
 	NSDictionary			*newSM = nil;
 	NSMutableDictionary		*cleanSM = nil;
@@ -780,8 +808,12 @@ static AIStackElement *sStack = NULL;
 	NSDictionary			*stateHandlers = nil;
 	NSAutoreleasePool		*pool = nil;
 	
-	newSM = [cacheMgr objectForKey:smName inCache:@"AIs"];
-	if (newSM != nil && ![newSM isKindOfClass:[NSDictionary class]])  return nil;	// catches use of @"nil" to indicate no AI found.
+	if (![smName isEqualToString:@"nullAI.plist"])
+	{
+		// don't cache nullAI since they're different depending on associated JS AI
+		newSM = [cacheMgr objectForKey:smName inCache:@"AIs"];
+		if (newSM != nil && ![newSM isKindOfClass:[NSDictionary class]])  return nil;	// catches use of @"nil" to indicate no AI found.
+	}
 	
 	if (newSM == nil)
 	{
@@ -790,9 +822,14 @@ static AIStackElement *sStack = NULL;
 		OOLogPushIndent();
 		OOLogIndentIf(@"ai.load");
 		
-		NS_DURING
+		@try
+		{
 			// Load state machine and validate against whitelist.
-			newSM = [ResourceManager dictionaryFromFilesNamed:smName inFolder:@"AIs" mergeMode:MERGE_NONE cache:NO];
+			NSString *aiPath = [ResourceManager pathForFileNamed:smName inFolder:@"AIs"];
+			if (aiPath != nil)
+			{
+				newSM = OODictionaryFromFile(aiPath);
+			}
 			if (newSM == nil)
 			{
 				[cacheMgr setObject:@"nil" forKey:smName inCache:@"AIs"];
@@ -802,7 +839,7 @@ static AIStackElement *sStack = NULL;
 					fromString = [NSString stringWithFormat:@" from %@:%@", [self name], [self state]];
 				}
 				OOLog(@"ai.load.failed.unknownAI", @"Can't switch AI for %@%@ to \"%@\" - could not load file.", [[self owner] shortDescription], fromString, smName);
-				NS_VALUERETURN(nil, NSDictionary *);
+				return nil;
 			}
 			
 			cleanSM = [NSMutableDictionary dictionaryWithCapacity:[newSM count]];
@@ -819,7 +856,8 @@ static AIStackElement *sStack = NULL;
 				stateHandlers = [self cleanHandlers:stateHandlers forState:stateKey stateMachine:smName];
 				[cleanSM setObject:stateHandlers forKey:stateKey];
 			}
-			
+			[cleanSM setObject:script forKey:@"jsScript"];
+
 			// Make immutable.
 			newSM = [[cleanSM copy] autorelease];
 			
@@ -832,12 +870,12 @@ static AIStackElement *sStack = NULL;
 			
 			// Cache.
 			[cacheMgr setObject:newSM forKey:smName inCache:@"AIs"];
-		NS_HANDLER
+		}
+		@finally
+		{
 			OOLogPopIndent();
-			[localException raise];
-		NS_ENDHANDLER
+		}
 		
-		OOLogPopIndent();
 		[newSM retain];
 		[pool release];
 		[newSM autorelease];
@@ -948,3 +986,67 @@ static AIStackElement *sStack = NULL;
 }
 
 @end
+
+
+@implementation OOPreservedAIStateMachine
+
+- (id) initWithStateMachine:(NSDictionary *)stateMachine
+					   name:(NSString *)name
+					  state:(NSString *)state
+			pendingMessages:(NSSet *)pendingMessages
+									 jsScript:(NSString *)script
+{
+	if ((self = [super init]))
+	{
+		_stateMachine = [stateMachine copy];
+		_name = [name copy];
+		_state = [state copy];
+		_pendingMessages = [pendingMessages copy];
+		_jsScript = [script copy];
+	}
+	
+	return self;
+}
+
+
+- (void) dealloc
+{
+	[_stateMachine autorelease];
+	[_name autorelease];
+	[_state autorelease];
+	[_pendingMessages autorelease];
+	[_jsScript autorelease];
+	
+	[super dealloc];
+}
+
+
+- (NSDictionary *) stateMachine
+{
+	return _stateMachine;
+}
+
+
+- (NSString *) name
+{
+	return _name;
+}
+
+
+- (NSString *) state
+{
+	return _state;
+}
+
+
+- (NSSet *) pendingMessages
+{
+	return _pendingMessages;
+}
+
+- (NSString *) jsScript
+{
+	return _jsScript;
+}
+
+@end
