@@ -5,7 +5,7 @@ OODebugMonitor.m
 
 Oolite debug support
 
-Copyright (C) 2007-2011 Jens Ayton
+Copyright (C) 2007-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -72,14 +72,14 @@ static OODebugMonitor *sSingleton = nil;
 
 
 @implementation OODebugMonitor
+#if OOLITE_GNUSTEP
+	NSString					*NSApplicationWillTerminateNotification = @"ApplicationWillTerminate";
+#endif
 
 - (id)init
 {
 	NSUserDefaults				*defaults = nil;
-	NSDictionary				*config = nil;
-#if OOLITE_GNUSTEP
-	NSString					*NSApplicationWillTerminateNotification = @"ApplicationWillTerminate";
-#endif
+	NSMutableDictionary			*config = nil;
 	
 	self = [super init];
 	if (self != nil)
@@ -90,11 +90,12 @@ static OODebugMonitor *sSingleton = nil;
 		_configFromOXPs = [[self normalizeConfigDictionary:config] copy];
 		
 		defaults = [NSUserDefaults standardUserDefaults];
-		config = [defaults dictionaryForKey:@"debug-settings-override"];
-		config = [self normalizeConfigDictionary:config];
+		config = [self normalizeConfigDictionary:[defaults dictionaryForKey:@"debug-settings-override"]];
 		if (config == nil)  config = [NSMutableDictionary dictionary];
 		_configOverrides = [config retain];
 		
+		_TCPIgnoresDroppedPackets = NO;
+		
 		OOJavaScriptEngine *jsEng = [OOJavaScriptEngine sharedEngine];
 #if OOJSENGINE_MONITOR_SUPPORT
 		[jsEng setMonitor:self];
@@ -142,7 +143,7 @@ static OODebugMonitor *sSingleton = nil;
 }
 
 
-+ (id)sharedDebugMonitor
++ (OODebugMonitor *) sharedDebugMonitor
 {
 	// NOTE: assumes single-threaded access. The debug monitor is not, on the whole, thread safe.
 	if (sSingleton == nil)
@@ -173,7 +174,8 @@ static OODebugMonitor *sSingleton = nil;
 		// If a new debugger was specified, try to connect it.
 		if (newDebugger != nil)
 		{
-			NS_DURING
+			@try
+			{
 				if ([newDebugger connectDebugMonitor:self errorMessage:&error])
 				{
 					[newDebugger debugMonitor:self
@@ -184,9 +186,11 @@ static OODebugMonitor *sSingleton = nil;
 				{
 					OOLog(@"debugMonitor.setDebugger.failed", @"Could not connect to debugger %@, because an error occurred: %@", newDebugger, error);
 				}
-			NS_HANDLER
-				OOLog(@"debugMonitor.setDebugger.failed", @"Could not connect to debugger %@, because an exception occurred: %@ -- %@", newDebugger, [localException name], [localException reason]);
-			NS_ENDHANDLER
+			}
+			@catch (NSException *exception)
+			{
+				OOLog(@"debugMonitor.setDebugger.failed", @"Could not connect to debugger %@, because an exception occurred: %@ -- %@", newDebugger, [exception name], [exception reason]);
+			}
 		}
 	}
 	
@@ -211,14 +215,17 @@ static OODebugMonitor *sSingleton = nil;
 {
 	if (string == nil)  return;
 	OOJSPauseTimeLimiter();
-	NS_DURING
+	@try
+	{
 		[_debugger debugMonitor:self
 				jsConsoleOutput:string
 					   colorKey:colorKey
 				  emphasisRange:emphasisRange];
-	NS_HANDLER
-		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to send JavaScript console text to debugger: %@ -- %@", [localException name], [localException reason]);
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to send JavaScript console text to debugger: %@ -- %@", [exception name], [exception reason]);
+	}
 	OOJSResumeTimeLimiter();
 }
 
@@ -235,11 +242,14 @@ static OODebugMonitor *sSingleton = nil;
 - (void)clearJSConsole
 {
 	OOJSPauseTimeLimiter();
-	NS_DURING
+	@try
+	{
 		[_debugger debugMonitorClearConsole:self];
-	NS_HANDLER
-		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to clear JavaScript console: %@ -- %@", [localException name], [localException reason]);
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to clear JavaScript console: %@ -- %@", [exception name], [exception reason]);
+	}
 	OOJSResumeTimeLimiter();
 }
 
@@ -247,11 +257,14 @@ static OODebugMonitor *sSingleton = nil;
 - (void)showJSConsole
 {
 	OOJSPauseTimeLimiter();
-	NS_DURING
+	@try
+	{
 		[_debugger debugMonitorShowConsole:self];
-	NS_HANDLER
-		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to show JavaScript console: %@ -- %@", [localException name], [localException reason]);
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to show JavaScript console: %@ -- %@", [exception name], [exception reason]);
+	}
 	OOJSResumeTimeLimiter();
 }
 
@@ -313,13 +326,16 @@ static OODebugMonitor *sSingleton = nil;
 		// Setting a nil value removes an override, and may reveal an underlying OXP-defined value
 		value = [self configurationValueForKey:key];
 	}
-	NS_DURING
+	@try
+	{
 		[_debugger debugMonitor:self
    noteChangedConfigrationValue:value
 						 forKey:key];
-	NS_HANDLER
-		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to send configuration update to debugger: %@ -- %@", [localException name], [localException reason]);
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to send configuration update to debugger: %@ -- %@", [exception name], [exception reason]);
+	}
 }
 
 
@@ -374,7 +390,7 @@ static NSString *SizeString(size_t size)
 		magnitude = 1;
 		suffix = @"KiB";
 	}
-	else if (size < (size_t)(kThreshold << 30))
+	else if (size < ((size_t)kThreshold << 30))
 	{
 		magnitude = 2;
 		suffix = @"MiB";
@@ -387,7 +403,7 @@ static NSString *SizeString(size_t size)
 	
 	float unit = 1 << (magnitude * 10);
 	float sizef = (float)size / unit;
-	sizef = roundf(sizef * 100.0f) / 100.f;
+	sizef = round(sizef * 100.0f) / 100.f;
 	
 	return [NSString stringWithFormat:@"%.2f %@", sizef, suffix];
 }
@@ -456,7 +472,7 @@ typedef struct
 		
 		if ([entity isPlayer])
 		{
-			unsigned i, count = [entity dialMaxMissiles];
+			NSUInteger i, count = [entity dialMaxMissiles];
 			for (i = 0; i < count; i++)
 			{
 				subentity = [entity missileForPylon:i];
@@ -466,11 +482,15 @@ typedef struct
 	}
 	if ([entity isPlanet])
 	{
+#if NEW_PLANETS
+		// FIXME: dump atmosphere texture.
+#else
 		PlanetEntity *atmosphere = [entity atmosphere];
 		if (atmosphere != nil)
 		{
 			[self dumpEntity:atmosphere withState:state parentVisible:visible];
 		}
+#endif
 	}
 	if ([entity isWormhole])
 	{
@@ -500,7 +520,7 @@ typedef struct
 	for (texEnum = [allTextures objectEnumerator]; (tex = [texEnum nextObject]); )
 	{
 		// We subtract one because allTextures retains the textures.
-		[textureRefCounts setObject:[NSNumber numberWithUnsignedInt:[tex retainCount] - 1] forKey:[NSValue valueWithNonretainedObject:tex]];
+		[textureRefCounts setObject:[NSNumber numberWithUnsignedInteger:[tex retainCount] - 1] forKey:[NSValue valueWithNonretainedObject:tex]];
 	}
 	
 	size_t totalSize = 0;
@@ -602,6 +622,16 @@ typedef struct
 	 SizeString(totalTextureDataSize),
 	 SizeString(visibleTextureDataSize)];
 	
+	totalSize += [self dumpJSMemoryStatistics];
+	
+	[self writeMemStat:@"Total: %@", SizeString(totalSize)];
+	
+	OOLogOutdent();
+}
+
+
+- (size_t) dumpJSMemoryStatistics
+{
 	JSContext *context = OOJSAcquireContext();
 	
 	JSRuntime *runtime = JS_GetRuntime(context);
@@ -612,11 +642,36 @@ typedef struct
 	OOJSRelinquishContext(context);
 	
 	[self writeMemStat:@"JavaScript heap: %@ (limit %@, %u collections to date)", SizeString(jsSize), SizeString(jsMax), jsGCCount];
-	totalSize += jsSize;
-	
-	[self writeMemStat:@"Total: %@", SizeString(totalSize)];
-	
-	OOLogOutdent();
+	return jsSize;
+}
+
+
+- (void) setTCPIgnoresDroppedPackets:(BOOL)flag
+{
+	if (_TCPIgnoresDroppedPackets != flag)
+	{
+		OOLog(@"debugMonitor.TCPSettings", @"The TCP console will %@ TCP packets.",
+				(flag ? @"try to stay connected, ignoring dropped" : @"disconnect if an error affects"));
+	}
+	_TCPIgnoresDroppedPackets = flag;
+}
+
+
+- (BOOL) TCPIgnoresDroppedPackets
+{
+	return _TCPIgnoresDroppedPackets;
+}
+
+
+- (void) setUsingPlugInController:(BOOL)flag
+{
+	_usingPlugInController = flag;
+}
+
+
+- (BOOL) usingPlugInController
+{
+	return _usingPlugInController;
 }
 
 
@@ -657,6 +712,14 @@ typedef struct
 }
 
 
+#if OOLITE_GNUSTEP
+- (void) applicationWillTerminate
+{
+	[[NSNotificationCenter defaultCenter] postNotificationName:NSApplicationWillTerminateNotification object:nil];
+}
+#endif
+
+
 - (void)applicationWillTerminate:(NSNotification *)notification
 {
 	if (_configOverrides != nil)
@@ -717,11 +780,14 @@ typedef struct
 
 - (void)disconnectDebuggerWithMessage:(NSString *)message
 {
-	NS_DURING
+	@try
+	{
 		[_debugger disconnectDebugMonitor:self message:message];
-	NS_HANDLER
-		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to disconnect debugger: %@ -- %@", [localException name], [localException reason]);
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		OOLog(@"debugMonitor.debuggerConnection.exception", @"Exception while attempting to disconnect debugger: %@ -- %@", [exception name], [exception reason]);
+	}
 	
 	id debugger = _debugger;
 	_debugger = nil;
@@ -838,7 +904,7 @@ FIXME: this works with CRLF and LF, but not CR.
 	
 	if (errorReport->flags & JSREPORT_STRICT)
 	{
-		prefix = [prefix stringByAppendingString:@" (pedantic mode)"];
+		prefix = [prefix stringByAppendingString:@" (strict mode)"];
 	}
 	
 	// Prefix and subsequent colon should be bold:
@@ -881,7 +947,7 @@ FIXME: this works with CRLF and LF, but not CR.
 				emphasisRange:emphasisRange];
 	
 	if (errorReport->flags & JSREPORT_WARNING)  showKey = @"show-console-on-warning";
-	else  showKey = @"show-console-on-warning";
+	else  showKey = @"show-console-on-error";	// if not a warning, it's a proper error.
 	if (OOBooleanFromObject([self configurationValueForKey:showKey], NO))
 	{
 		[self showJSConsole];
@@ -955,7 +1021,7 @@ NOTE: assumes single-threaded access.
 }
 
 
-- (OOUInteger)retainCount
+- (NSUInteger)retainCount
 {
 	return UINT_MAX;
 }
