@@ -3,9 +3,9 @@
 OODebugTCPConsoleClient.m
 
 
-Oolite Debug OXP
+Oolite Debug Support
 
-Copyright (C) 2009-2011 Jens Ayton and contributors
+Copyright (C) 2009-2013 Jens Ayton and contributors
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -36,6 +36,7 @@ SOFTWARE.
 #import "OOFunctionAttributes.h"
 #import "OOLogging.h"
 #include <stdint.h>
+#import "NSDictionaryOOExtensions.h"
 
 #if OOLITE_WINDOWS
 #include <winsock2.h>
@@ -241,8 +242,8 @@ OOINLINE BOOL StatusIsSendable(OOTCPClientConnectionStatus status)
 	if (emphasisRange.length != 0)
 	{
 		range = [NSArray arrayWithObjects:
-						[NSNumber numberWithUnsignedInt:emphasisRange.location],
-						[NSNumber numberWithUnsignedInt:emphasisRange.length],
+						[NSNumber numberWithUnsignedInteger:emphasisRange.location],
+						[NSNumber numberWithUnsignedInteger:emphasisRange.length],
 						nil];
 		[parameters setObject:range forKey:kOOTCPEmphasisRanges];
 	}
@@ -337,14 +338,12 @@ noteChangedConfigrationValue:(in id)newValue
 
 - (BOOL) sendBytes:(const void *)bytes count:(size_t)count
 {
-	int						written;
-	
 	if (bytes == NULL || count == 0)  return YES;
 	if (!StatusIsSendable(_status) || _outStream == nil)  return NO;
 	
 	do
 	{
-		written = [_outStream write:bytes maxLength:count];
+		NSInteger written = [_outStream write:bytes maxLength:count];
 		if (written < 1)  return NO;
 		
 		count -= written;
@@ -363,6 +362,7 @@ noteChangedConfigrationValue:(in id)newValue
 	size_t					count;
 	const uint8_t			*bytes = NULL;
 	uint32_t				header;
+	bool 					sentOK = YES;
 	
 	if (dictionary == nil || !StatusIsSendable(_status))  return;
 	
@@ -386,9 +386,46 @@ noteChangedConfigrationValue:(in id)newValue
 	header = htonl(count);
 	
 	bytes = [data bytes];
-	if (![self sendBytes:&header count:sizeof header] || ![self sendBytes:bytes count:count])
+	
+	/*	In testing, all bad stream errors were caused by the python console
+		rejecting headers. Made the protocol a bit more fault tolerant.
+		-- Kaks 2012.03.24
+	*/
+	if (![self sendBytes:&header count:sizeof header])
 	{
-		[self breakConnectionWithBadStream:_outStream];
+		OOLog(@"debugTCP.send.warning", @"Error sending packet header, retrying.");
+		// wait 8 milliseconds, resend the header
+		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:.008]];
+		if (![self sendBytes:&header count:sizeof header])
+		{
+			//OOLog(@"debugTCP.send.warning", @"Error sending packet header, retrying one more time.");
+			// wait 16 milliseconds, try to resend the header one last time!
+			[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:.016]];
+			if (![self sendBytes:&header count:sizeof header])
+			{
+				sentOK = NO;
+			}
+		}
+	}
+	
+	if(sentOK && ![self sendBytes:bytes count:count])
+	{
+		OOLog(@"debugTCP.send.warning", @"Error sending packet body, retrying.");
+		// wait 8 milliseconds, try again.
+		[[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:.008]];
+		if(![self sendBytes:bytes count:count])
+		{
+			sentOK = NO;
+		}
+	}
+	
+	if (!sentOK)
+	{
+		OOLog(@"debugTCP.send.error", @"The following packet could not be sent: %@", dictionary);
+		if(![[OODebugMonitor sharedDebugMonitor] TCPIgnoresDroppedPackets])
+		{
+			[self breakConnectionWithBadStream:_outStream];
+		}
 	}
 }
 
@@ -396,22 +433,20 @@ noteChangedConfigrationValue:(in id)newValue
 - (void) sendPacket:(NSString *)packetType
 	 withParameters:(NSDictionary *)parameters
 {
-	NSMutableDictionary		*dict = nil;
+	NSDictionary		*dict = nil;
 	
 	if (packetType == nil)  return;
 	
 	if (parameters != nil)
 	{
-		dict = [parameters mutableCopy];
-		[dict setObject:packetType forKey:kOOTCPPacketType];
+		dict = [parameters dictionaryByAddingObject:packetType forKey:kOOTCPPacketType];
 	}
 	else
 	{
-		dict = [[NSDictionary alloc] initWithObjectsAndKeys:packetType, kOOTCPPacketType, nil];
+		dict = [NSDictionary dictionaryWithObjectsAndKeys:packetType, kOOTCPPacketType, nil];
 	}
 	
 	[self sendDictionary:dict];
-	[dict release];
 }
 
 
@@ -434,24 +469,12 @@ noteChangedConfigrationValue:(in id)newValue
 	enum { kBufferSize = 16 << 10 };
 	
 	uint8_t							buffer[kBufferSize];
-	int								length;
+	NSInteger						length;
 	NSData							*data;
 	
 	length = [_inStream read:buffer maxLength:kBufferSize];
-	while( length > 0 )
+	while (length > 0)
 	{
-		/* This test is superfluous after the rewrite to fix Bug#014643
-		 * TODO: Put the BadStream test back into the code
-		if (length < 1)
-		{
-			// Under GNUstep, but not OS X (currently), -hasBytesAvailable will return YES when the buffer is in fact empty.
-			if ([_inStream streamStatus] == NSStreamStatusReading) break;
-			
-			[self breakConnectionWithBadStream:_inStream];
-			return;
-		}
-		*/
-		
 		data = [NSData dataWithBytesNoCopy:buffer length:length freeWhenDone:NO];
 		OOTCPStreamDecoderReceiveData(_decoder, data);
 		length = [_inStream read:buffer maxLength:kBufferSize];
@@ -663,9 +686,9 @@ noteChangedConfigrationValue:(in id)newValue
 	error = [stream streamError];
 	errorDesc = [error localizedDescription];
 	if (errorDesc == nil)  errorDesc = [error description];
-	if (errorDesc == nil)  errorDesc = @"unknown error.";
+	if (errorDesc == nil)  errorDesc = @"bad stream.";
 	[self breakConnectionWithMessage:[NSString stringWithFormat:
-	   @"Connection to debug console failed: '%@' (outStream status: %i, inStream status: %i).",
+	   @"Connection to debug console failed: '%@' (outStream status: %li, inStream status: %li).",
 		errorDesc, [_outStream streamStatus], [_inStream streamStatus]]];
 }
 
