@@ -4,7 +4,7 @@ OOJSConsole.m
 
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -79,6 +79,7 @@ static JSBool ConsoleDisplayMessagesInClass(JSContext *context, uintN argc, jsva
 static JSBool ConsoleSetDisplayMessagesInClass(JSContext *context, uintN argc, jsval *vp);
 static JSBool ConsoleWriteLogMarker(JSContext *context, uintN argc, jsval *vp);
 static JSBool ConsoleWriteMemoryStats(JSContext *context, uintN argc, jsval *vp);
+static JSBool ConsoleWriteJSMemoryStats(JSContext *context, uintN argc, jsval *vp);
 static JSBool ConsoleGarbageCollect(JSContext *context, uintN argc, jsval *vp);
 #if DEBUG
 static JSBool ConsoleDumpNamedRoots(JSContext *context, uintN argc, jsval *vp);
@@ -120,11 +121,11 @@ enum
 {
 	// Property IDs
 	kConsole_debugFlags,						// debug flags, integer, read/write
-	kConsole_shaderMode,						// shader mode, symbolic string, read/write
-	kConsole_maximumShaderMode,					// highest supported shader mode, symbolic string, read-only
-	kConsole_reducedDetailMode,					// reduced detail mode, boolean, read/write
+	kConsole_detailLevel,						// graphics detail level, symbolic string, read/write
+	kConsole_maximumDetailLevel,				// maximum graphics detail level, symbolic string, read-only
 	kConsole_displayFPS,						// display FPS (and related info), boolean, read/write
 	kConsole_platformDescription,				// Information about system we're running on in unspecified format, string, read-only
+	kConsole_ignoreDroppedPackets,				// boolean (default false), read/write
 	kConsole_pedanticMode,						// JS pedantic mode (JS_STRICT flag, not the same as "use strict"), boolean (default true), read/write
 	kConsole_showErrorLocations,				// Show error/warning source locations, boolean (default true), read/write
 	kConsole_dumpStackForErrors,				// Write stack dump when reporting error/exception, boolean (default false), read/write
@@ -155,12 +156,12 @@ static JSPropertySpec sConsoleProperties[] =
 {
 	// JS name								ID											flags
 	{ "debugFlags",							kConsole_debugFlags,						OOJS_PROP_READWRITE_CB },
-	{ "shaderMode",							kConsole_shaderMode,						OOJS_PROP_READWRITE_CB },
-	{ "maximumShaderMode",					kConsole_maximumShaderMode,					OOJS_PROP_READONLY_CB },
-	{ "reducedDetailMode",					kConsole_reducedDetailMode,					OOJS_PROP_READWRITE_CB },
+	{ "detailLevel",						kConsole_detailLevel,						OOJS_PROP_READWRITE_CB },
+	{ "maximumDetailLevel",					kConsole_maximumDetailLevel,				OOJS_PROP_READONLY_CB },
 	{ "displayFPS",							kConsole_displayFPS,						OOJS_PROP_READWRITE_CB },
 	{ "platformDescription",				kConsole_platformDescription,				OOJS_PROP_READONLY_CB },
 	{ "pedanticMode",						kConsole_pedanticMode,						OOJS_PROP_READWRITE_CB },
+	{ "ignoreDroppedPackets",				kConsole_ignoreDroppedPackets,				OOJS_PROP_READWRITE_CB },
 	{ "__showErrorLocations",				kConsole_showErrorLocations,				OOJS_PROP_HIDDEN_READWRITE_CB },
 	{ "__dumpStackForErrors",				kConsole_dumpStackForErrors,				OOJS_PROP_HIDDEN_READWRITE_CB },
 	{ "__dumpStackForWarnings",				kConsole_dumpStackForWarnings,				OOJS_PROP_HIDDEN_READWRITE_CB },
@@ -203,6 +204,7 @@ static JSFunctionSpec sConsoleMethods[] =
 	{ "setDisplayMessagesInClass",		ConsoleSetDisplayMessagesInClass,	2 },
 	{ "writeLogMarker",					ConsoleWriteLogMarker,				0 },
 	{ "writeMemoryStats",				ConsoleWriteMemoryStats,			0 },
+	{ "writeJSMemoryStats",				ConsoleWriteJSMemoryStats,			0 },
 	{ "garbageCollect",					ConsoleGarbageCollect,				0 },
 #if DEBUG
 	{ "dumpNamedRoots",					ConsoleDumpNamedRoots,				0 },
@@ -313,21 +315,16 @@ static JSBool ConsoleGetProperty(JSContext *context, JSObject *this, jsid propID
 	{
 #ifndef NDEBUG
 		case kConsole_debugFlags:
-			*value = INT_TO_JSVAL(gDebugFlags);
+			*value = INT_TO_JSVAL((uint32_t)gDebugFlags);
 			break;
 #endif		
 			
-		case kConsole_shaderMode:
-			// EMMSTRAN: if still relevant, OOConstToJSString-ify.
-			*value = OOJSValueFromNativeObject(context, OOStringFromShaderSetting([UNIVERSE shaderEffectsLevel]));
+		case kConsole_detailLevel:
+			*value = [OOStringFromGraphicsDetail([UNIVERSE detailLevel]) oo_jsValueInContext:context];
 			break;
 			
-		case kConsole_maximumShaderMode:
-			*value = OOJSValueFromNativeObject(context, OOStringFromShaderSetting([[OOOpenGLExtensionManager sharedManager] maximumShaderSetting]));
-			break;
-			
-		case kConsole_reducedDetailMode:
-			*value = OOJSValueFromBOOL([UNIVERSE reducedDetail]);
+		case kConsole_maximumDetailLevel:
+			*value = [OOStringFromGraphicsDetail([[OOOpenGLExtensionManager sharedManager] maximumDetailLevel]) oo_jsValueInContext:context];
 			break;
 			
 		case kConsole_displayFPS:
@@ -345,6 +342,10 @@ static JSBool ConsoleGetProperty(JSContext *context, JSObject *this, jsid propID
 			}
 			break;
 			
+		case kConsole_ignoreDroppedPackets:
+			*value = OOJSValueFromBOOL([[OODebugMonitor sharedDebugMonitor] TCPIgnoresDroppedPackets]);
+			break;
+			
 		case kConsole_showErrorLocations:
 			*value = OOJSValueFromBOOL([[OOJavaScriptEngine sharedEngine] showErrorLocations]);
 			break;
@@ -406,8 +407,8 @@ static JSBool ConsoleSetProperty(JSContext *context, JSObject *this, jsid propID
 	OOJS_NATIVE_ENTER(context)
 	
 	int32						iValue;
-	NSString					*sValue = nil;
 	JSBool						bValue = NO;
+	NSString					*sValue;
 	
 	switch (JSID_TO_INT(propID))
 	{
@@ -419,24 +420,11 @@ static JSBool ConsoleSetProperty(JSContext *context, JSObject *this, jsid propID
 			}
 			break;
 #endif		
-		case kConsole_shaderMode:
+		case kConsole_detailLevel:
 			sValue = OOStringFromJSValue(context, *value);
-			if (sValue != nil)
-			{
-				OOJS_BEGIN_FULL_NATIVE(context)
-				OOShaderSetting setting = OOShaderSettingFromString(sValue);
-				[UNIVERSE setShaderEffectsLevel:setting transiently:YES];
-				OOJS_END_FULL_NATIVE
-			}
-			break;
-			
-		case kConsole_reducedDetailMode:
-			if (JS_ValueToBoolean(context, *value, &bValue))
-			{
-				OOJS_BEGIN_FULL_NATIVE(context)
-				[UNIVERSE setReducedDetail:bValue transiently:YES];
-				OOJS_END_FULL_NATIVE
-			}
+			OOJS_BEGIN_FULL_NATIVE(context)
+			[UNIVERSE setDetailLevel:OOGraphicsDetailFromString(sValue)];
+			OOJS_END_FULL_NATIVE
 			break;
 			
 		case kConsole_displayFPS:
@@ -457,6 +445,13 @@ static JSBool ConsoleSetProperty(JSContext *context, JSObject *this, jsid propID
 			}
 			break;
 			
+		case kConsole_ignoreDroppedPackets:
+			if (JS_ValueToBoolean(context, *value, &bValue))
+			{
+				[[OODebugMonitor sharedDebugMonitor] setTCPIgnoresDroppedPackets:bValue];
+			}
+			break;
+			
 		case kConsole_showErrorLocations:
 			if (JS_ValueToBoolean(context, *value, &bValue))
 			{
@@ -619,7 +614,7 @@ static JSBool ConsoleSettingsSetProperty(JSContext *context, JSObject *this, jsi
 		}
 		else
 		{
-			OOJSReportWarning(context, @"debugConsole.settings: could not convert %@ to native object.", [NSString stringWithJavaScriptValue:*value inContext:context]);
+			OOJSReportWarning(context, @"debugConsole.settings: could not convert %@ to native object.", OOStringFromJSValue(context, *value));
 		}
 	}
 	OOJSResumeTimeLimiter();
@@ -654,10 +649,10 @@ static JSBool ConsoleConsoleMessage(JSContext *context, uintN argc, jsval *vp)
 		return NO;
 	}
 	
-	colorKey = OOStringFromJSValue(context,OOJS_ARGV[0]);
-	message = OOStringFromJSValue(context,OOJS_ARGV[1]);
+	if (argc > 0) colorKey = OOStringFromJSValue(context,OOJS_ARGV[0]);
+	if (argc > 1) message = OOStringFromJSValue(context,OOJS_ARGV[1]);
 	
-	if (4 <= argc)
+	if (argc > 3)
 	{
 		// Attempt to get two numbers, specifying an emphasis range.
 		if (JS_ValueToNumber(context, OOJS_ARGV[2], &location) &&
@@ -762,7 +757,7 @@ static JSBool ConsoleCallObjCMethod(JSContext *context, uintN argc, jsval *vp)
 	object = OOJSNativeObjectFromJSObject(context, OOJS_THIS);
 	if (object == nil)
 	{
-		OOJSReportError(context, @"Attempt to call __callObjCMethod() for non-Objective-C object %@.", [NSString stringWithJavaScriptValue:JS_THIS(context, vp) inContext:context]);
+		OOJSReportError(context, @"Attempt to call __callObjCMethod() for non-Objective-C object %@.", OOStringFromJSValueEvenIfNull(context, JS_THIS(context, vp)));
 		return NO;
 	}
 	
@@ -821,7 +816,7 @@ static JSBool ConsoleIsExecutableJavaScript(JSContext *context, uintN argc, jsva
 	
 	OOJSResumeTimeLimiter();
 	
-	OOJS_RETURN_BOOL(YES);
+	OOJS_RETURN_BOOL(result);
 	
 	OOJS_NATIVE_EXIT
 }
@@ -834,8 +829,8 @@ static JSBool ConsoleDisplayMessagesInClass(JSContext *context, uintN argc, jsva
 	
 	NSString				*messageClass = nil;
 	
-	messageClass = [NSString stringWithJavaScriptValue:OOJS_ARGV[0] inContext:context];
-	OOJS_RETURN_BOOL(OOLogWillDisplayMessagesInClass(messageClass));
+	messageClass = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	OOJS_RETURN_BOOL(messageClass != nil && OOLogWillDisplayMessagesInClass(messageClass));
 	
 	OOJS_NATIVE_EXIT
 }
@@ -887,6 +882,21 @@ static JSBool ConsoleWriteMemoryStats(JSContext *context, uintN argc, jsval *vp)
 }
 
 
+// function writeJSMemoryStats() : void
+static JSBool ConsoleWriteJSMemoryStats(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	OOJS_BEGIN_FULL_NATIVE(context)
+	[[OODebugMonitor sharedDebugMonitor] dumpJSMemoryStatistics];
+	OOJS_END_FULL_NATIVE
+	
+	OOJS_RETURN_VOID;
+	
+	OOJS_NATIVE_EXIT
+}
+
+
 // function garbageCollect() : string
 static JSBool ConsoleGarbageCollect(JSContext *context, uintN argc, jsval *vp)
 {
@@ -911,10 +921,12 @@ typedef struct
 
 static void DumpCallback(const char *name, void *rp, JSGCRootType type, void *datap)
 {
+	assert(type == JS_GC_ROOT_VALUE_PTR || type == JS_GC_ROOT_GCTHING_PTR);
+	
 	DumpCallbackData *data = datap;
 	
 	const char *typeString = "unknown type";
-	jsval value = JSVAL_VOID;
+	jsval value;
 	switch (type)
 	{
 		case JS_GC_ROOT_VALUE_PTR:
