@@ -3,7 +3,7 @@
 Entity.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -28,7 +28,6 @@ MA 02110-1301, USA.
 #import "OOPlanetEntity.h"
 
 #import "OOMaths.h"
-#import "Geometry.h"
 #import "Universe.h"
 #import "GameController.h"
 #import "ResourceManager.h"
@@ -40,20 +39,21 @@ MA 02110-1301, USA.
 #import "OODebugFlags.h"
 #import "NSObjectOOExtensions.h"
 
-#define kOOLogUnconvertedNSLog @"unclassified.Entity"
-
 #ifndef NDEBUG
 uint32_t gLiveEntityCount = 0;
 size_t gTotalEntityMemory = 0;
 #endif
 
 
+#ifndef NDEBUG
 static NSString * const kOOLogEntityAddToList				= @"entity.linkedList.add";
 static NSString * const kOOLogEntityAddToListError			= @"entity.linkedList.add.error";
 static NSString * const kOOLogEntityRemoveFromList			= @"entity.linkedList.remove";
 static NSString * const kOOLogEntityRemoveFromListError		= @"entity.linkedList.remove.error";
-	   NSString * const kOOLogEntityVerificationError		= @"entity.linkedList.verify.error";
 static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.error";
+#endif
+static NSString * const kOOLogEntityVerificationError		= @"entity.linkedList.verify.error";
+
 
 
 @interface Entity (OOPrivate)
@@ -74,7 +74,7 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 	
 	orientation = kIdentityQuaternion;
 	rotMatrix = kIdentityMatrix;
-	position = kZeroVector;
+	position = kZeroHPVector;
 	
 	no_draw_distance = 100000.0;  //  10 km
 	
@@ -106,7 +106,7 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 	
 #ifndef NDEBUG
 	gLiveEntityCount--;
-	gTotalEntityMemory += [self oo_objectSize];
+	gTotalEntityMemory -= [self oo_objectSize];
 #endif
 	
 	[super dealloc];
@@ -115,11 +115,11 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 
 - (NSString *)descriptionComponents
 {
-	return [NSString stringWithFormat:@"position: %@ scanClass: %@ status: %@", VectorDescription([self position]), OOStringFromScanClass([self scanClass]), OOStringFromEntityStatus([self status])];
+	return [NSString stringWithFormat:@"position: %@ scanClass: %@ status: %@", HPVectorDescription([self position]), OOStringFromScanClass([self scanClass]), OOStringFromEntityStatus([self status])];
 }
 
 
-- (OOUInteger) sessionID
+- (NSUInteger) sessionID
 {
 	return _sessionID;
 }
@@ -131,6 +131,12 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
+- (BOOL)isDock
+{
+	return NO;
+}
+
+
 - (BOOL)isStation
 {
 	return isStation;
@@ -184,10 +190,22 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
+- (BOOL) isVisualEffect
+{
+	return NO;
+}
+
+
+- (BOOL) isWaypoint
+{
+	return NO;
+}
+
+
 - (BOOL) validForAddToUniverse
 {
-	OOUInteger mySessionID = [self sessionID];
-	OOUInteger currentSessionID = [UNIVERSE sessionID];
+	NSUInteger mySessionID = [self sessionID];
+	NSUInteger currentSessionID = [UNIVERSE sessionID];
 	if (EXPECT_NOT(mySessionID != currentSessionID))
 	{
 		OOLogERR(@"entity.invalidSession", @"Entity %@ from session %lu cannot be added to universe in session %lu. This is an internal error, please report it.", [self shortDescription], mySessionID, currentSessionID);
@@ -250,8 +268,6 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 		{
 			OOLog(kOOLogEntityAddToListError, @"DEBUG LINKED LISTS - problem encountered while adding %@ to linked lists", self);
 			[UNIVERSE debugDumpEntities];
-		
-			exit(-1);
 		}
 	}
 #endif
@@ -299,8 +315,6 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 		{
 			OOLog(kOOLogEntityRemoveFromListError, @"DEBUG LINKED LISTS - problem encountered while removing %@ from linked lists", self);
 			[UNIVERSE debugDumpEntities];
-		
-			exit(-1);
 		}
 	}
 #endif
@@ -401,8 +415,6 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 		{
 			OOLog(kOOLogEntityVerificationError, @"DEBUG LINKED LISTS problem encountered before updating linked lists for %@", self);
 			[UNIVERSE debugDumpEntities];
-		
-			exit(-1);
 		}
 	}
 #endif
@@ -484,8 +496,6 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 		{
 			OOLog(kOOLogEntityUpdateError, @"DEBUG LINKED LISTS problem encountered after updating linked lists for %@", self);
 			[UNIVERSE debugDumpEntities];
-		
-			exit(-1);
 		}
 	}
 #endif
@@ -590,37 +600,78 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
-- (void) setPosition:(Vector) posn
+- (HPVector) position
 {
-	position = posn;
+	return position;
+}
+
+- (Vector) cameraRelativePosition
+{
+	return cameraRelativePosition;
+}
+
+- (GLfloat) cameraRangeFront
+{
+	return magnitude(cameraRelativePosition) - [self frustumRadius];
+}
+
+- (GLfloat) cameraRangeBack
+{
+	return magnitude(cameraRelativePosition) + [self frustumRadius];
 }
 
 
-- (void) setPositionX:(GLfloat)x y:(GLfloat)y z:(GLfloat)z
+
+// Exposed to uniform bindings.
+// so needs to remain at OpenGL precision levels
+- (Vector) relativePosition
+{
+	return HPVectorToVector(HPvector_subtract([self position], [PLAYER position]));
+}
+
+- (Vector) vectorTo:(Entity *)entity
+{
+	return HPVectorToVector(HPvector_subtract([entity position], [self position]));
+}
+
+
+- (void) setPosition:(HPVector) posn
+{
+	position = posn;
+	[self updateCameraRelativePosition];
+}
+
+
+- (void) setPositionX:(OOHPScalar)x y:(OOHPScalar)y z:(OOHPScalar)z
 {
 	position.x = x;
 	position.y = y;
 	position.z = z;
+	[self updateCameraRelativePosition];
 }
 
 
-- (Vector) absolutePositionForSubentity
+- (void) updateCameraRelativePosition
 {
-	return [self absolutePositionForSubentityOffset:kZeroVector];
+	cameraRelativePosition = HPVectorToVector(HPvector_subtract([self absolutePositionForSubentity],[PLAYER viewpointPosition]));
 }
 
 
-- (Vector) absolutePositionForSubentityOffset:(Vector) offset
+- (HPVector) absolutePositionForSubentity
 {
-	Vector		abspos = vector_add(position, OOVectorMultiplyMatrix(offset, rotMatrix));
+	return [self absolutePositionForSubentityOffset:kZeroHPVector];
+}
+
+
+- (HPVector) absolutePositionForSubentityOffset:(HPVector) offset
+{
+	HPVector		abspos = HPvector_add(position, OOHPVectorMultiplyMatrix(offset, rotMatrix));
 	Entity		*last = nil;
 	Entity		*father = [self parentEntity];
-	OOMatrix	r_mat;
 	
-	while ((father)&&(father != last)  && (father != NO_TARGET))
+	while (father != nil && father != last)
 	{
-		r_mat = [father drawRotationMatrix];
-		abspos = vector_add(OOVectorMultiplyMatrix(abspos, r_mat), [father position]);
+		abspos = HPvector_add(OOHPVectorMultiplyMatrix(abspos, [father drawRotationMatrix]), [father position]);
 		last = father;
 		if (![last isSubEntity]) break;
 		father = [father owner];
@@ -635,6 +686,12 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
+- (double) camZeroDistance
+{
+	return cam_zero_distance;
+}
+
+
 - (NSComparisonResult) compareZeroDistance:(Entity *)otherEntity
 {
 	if ((otherEntity)&&(zero_distance > otherEntity->zero_distance))
@@ -796,13 +853,11 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
-- (void) moveForward:(double) amount
+- (void) moveForward:(double)amount
 {
-	Vector		forward = vector_forward_from_quaternion(orientation);
+	HPVector forward = HPvector_multiply_scalar(HPvector_forward_from_quaternion(orientation), amount);
+	position = HPvector_add(position, forward);
 	distanceTravelled += amount;
-	position.x += amount * forward.x;
-	position.y += amount * forward.y;
-	position.z += amount * forward.z;
 }
 
 
@@ -821,20 +876,14 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 - (OOMatrix) transformationMatrix
 {
 	OOMatrix result = rotMatrix;
-	return OOMatrixTranslate(result, position);
+	return OOMatrixHPTranslate(result, position);
 }
 
 
 - (OOMatrix) drawTransformationMatrix
 {
 	OOMatrix result = rotMatrix;
-	return OOMatrixTranslate(result, position);
-}
-
-
-- (Vector) position
-{
-	return position;
+	return OOMatrixHPTranslate(result, position);
 }
 
 
@@ -850,6 +899,12 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
+- (GLfloat) frustumRadius
+{
+	return collision_radius;
+}
+
+
 - (void) setCollisionRadius:(GLfloat) amount
 {
 	collision_radius = amount;
@@ -866,23 +921,41 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 {
 	if (_status != STATUS_COCKPIT_DISPLAY)
 	{
-		zero_distance = distance2(PLAYER->position, position);
+		if ([self isSubEntity])
+		{
+			zero_distance = [[self owner] zeroDistance];
+			cam_zero_distance = [[self owner] camZeroDistance];
+			[self updateCameraRelativePosition];
+		}
+		else
+		{
+			zero_distance = HPdistance2(PLAYER->position, position);
+			cam_zero_distance = HPdistance2([PLAYER viewpointPosition], position);
+			[self updateCameraRelativePosition];
+		}
 	}
 	else
 	{
-		zero_distance = magnitude2(position);
+		zero_distance = HPmagnitude2(position);
+		cam_zero_distance = zero_distance;
+		cameraRelativePosition = HPVectorToVector(position);
 	}
 	
-	hasMoved = !vector_equal(position, lastPosition);
+	if ([self status] != STATUS_COCKPIT_DISPLAY)
+	{
+		[self applyVelocity:delta_t];
+	}
+
+	hasMoved = !HPvector_equal(position, lastPosition);
 	hasRotated = !quaternion_equal(orientation, lastOrientation);
 	lastPosition = position;
 	lastOrientation = orientation;
 }
 
 
-- (void) applyVelocityWithTimeDelta:(OOTimeDelta)delta_t
+- (void) applyVelocity:(OOTimeDelta)delta_t
 {
-	position = vector_add(position, vector_multiply_scalar(velocity, delta_t));
+	position = HPvector_add(position, HPvector_multiply_scalar(vectorToHPVector(velocity), delta_t));
 }
 
 
@@ -899,14 +972,7 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
-- (Geometry *)geometry
-{
-	OOLogGenericSubclassResponsibility();
-	return nil;
-}
-
-
-- (void) drawEntity:(BOOL)immediate :(BOOL)translucent
+- (void) drawImmediate:(bool)immediate translucent:(bool)translucent
 {
 	OOLogGenericSubclassResponsibility();
 }
@@ -925,10 +991,11 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 		OOLog(@"dumpState", @"State for %@:", self);
 		OOLogPushIndent();
 		OOLogIndent();
-		NS_DURING
+		@try
+		{
 			[self dumpSelfState];
-		NS_HANDLER
-		NS_ENDHANDLER
+		}
+		@catch (id exception) {}
 		OOLogPopIndent();
 	}
 }
@@ -946,7 +1013,7 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 	OOLog(@"dumpState.entity", @"Universal ID: %u", universalID);
 	OOLog(@"dumpState.entity", @"Scan class: %@", OOStringFromScanClass(scanClass));
 	OOLog(@"dumpState.entity", @"Status: %@", OOStringFromEntityStatus([self status]));
-	OOLog(@"dumpState.entity", @"Position: %@", VectorDescription(position));
+	OOLog(@"dumpState.entity", @"Position: %@", HPVectorDescription(position));
 	OOLog(@"dumpState.entity", @"Orientation: %@", QuaternionDescription(orientation));
 	OOLog(@"dumpState.entity", @"Distance travelled: %g", distanceTravelled);
 	OOLog(@"dumpState.entity", @"Energy: %g of %g", energy, maxEnergy);
@@ -963,10 +1030,11 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 	ADD_FLAG_IF_SET(hasMoved);
 	ADD_FLAG_IF_SET(hasRotated);
 	ADD_FLAG_IF_SET(isSunlit);
-	ADD_FLAG_IF_SET(collisionTestFilter);
 	ADD_FLAG_IF_SET(throw_sparks);
 	flagsString = [flags count] ? [flags componentsJoinedByString:@", "] : (NSString *)@"none";
 	OOLog(@"dumpState.entity", @"Flags: %@", flagsString);
+	OOLog(@"dumpState.entity", @"Collision Test Filter: %u", collisionTestFilter);
+
 }
 
 
@@ -1010,7 +1078,7 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 {
 	NSString *result = [self descriptionForObjDumpBasic];
 	
-	result = [result stringByAppendingFormat:@" range: %g (visible: %@)", distance([self position], [PLAYER position]), [self isVisible] ? @"yes" : @"no"];
+	result = [result stringByAppendingFormat:@" range: %g (visible: %@)", HPdistance([self position], [PLAYER position]), [self isVisible] ? @"yes" : @"no"];
 	
 	return result;
 }
@@ -1025,7 +1093,32 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 
 - (BOOL) isVisible
 {
-	return zero_distance <= ABSOLUTE_NO_DRAW_DISTANCE2;
+	return cam_zero_distance <= ABSOLUTE_NO_DRAW_DISTANCE2;
+}
+
+
+- (BOOL) isInSpace
+{
+	switch ([self status])
+	{
+	case STATUS_IN_FLIGHT:
+	case STATUS_DOCKING:
+	case STATUS_LAUNCHING:
+	case STATUS_AUTOPILOT_ENGAGED:
+	case STATUS_WITCHSPACE_COUNTDOWN:
+	case STATUS_BEING_SCOOPED:
+	case STATUS_EFFECT:
+	case STATUS_ACTIVE:
+		return YES;
+	default:
+		return NO;
+	}
+}
+
+
+- (BOOL) isImmuneToBreakPatternHide
+{
+	return isImmuneToBreakPatternHide;
 }
 
 @end
