@@ -6,7 +6,7 @@ Flashes during explosions and laser hits - not to be confused with flashers.
 
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -33,10 +33,11 @@ MA 02110-1301, USA.
 @private
 	float				_duration;
 	float				_growthRate;
+	float				_alpha;
 }
 
-+ (id) explosionFlashFromEntity:(Entity *)entity;
-+ (id) laserFlashWithPosition:(Vector)position velocity:(Vector)vel color:(OOColor *)color;
++ (instancetype) explosionFlashFromEntity:(Entity *)entity;
++ (instancetype) laserFlashWithPosition:(HPVector)position velocity:(Vector)vel color:(OOColor *)color;
 
 + (void) setUpTexture;
 
