@@ -3,7 +3,7 @@
  PlayerEntityLoadSave.m
  
  Oolite
- Copyright (C) 2004-2011 Giles C Williams and contributors
+ Copyright (C) 2004-2013 Giles C Williams and contributors
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
@@ -29,7 +29,11 @@
 
 #import "NSFileManagerOOExtensions.h"
 #import "GameController.h"
+#import "ResourceManager.h"
+#import "OOStringExpander.h"
 #import "PlayerEntityControls.h"
+#import "ProxyPlayerEntity.h"
+#import "ShipEntityAI.h"
 #import "OOXMLExtensions.h"
 #import "OOSound.h"
 #import "OOColor.h"
@@ -45,18 +49,8 @@
 #import "OOJavaScriptEngine.h"
 
 
-// Set to 1 to use custom load/save dialogs in windowed mode on Macs in debug builds. No effect on other platforms.
-#define USE_CUSTOM_LOAD_SAVE_ON_MAC_DEBUG		0
-
-#if USE_CUSTOM_LOAD_SAVE_ON_MAC_DEBUG && OO_DEBUG && defined(OOLITE_USE_APPKIT_LOAD_SAVE)
-#undef OOLITE_USE_APPKIT_LOAD_SAVE
-#endif
-
-
 // Name of modifier key used to issue commands. See also -isCommandModifierKeyDown.
-#if OOLITE_MAC_OS_X
-#define COMMAND_MODIFIER_KEY		"Ctrl" // was "Command"
-#else
+#if OO_USE_CUSTOM_LOAD_SAVE
 #define COMMAND_MODIFIER_KEY		"Ctrl"
 #endif
 
@@ -64,12 +58,16 @@
 static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 
 
+#if OO_USE_CUSTOM_LOAD_SAVE
+
 @interface MyOpenGLView (OOLoadSaveExtensions)
 
 - (BOOL)isCommandModifierKeyDown;
 
 @end
 
+#endif
+
 
 @interface PlayerEntity (OOLoadSavePrivate)
 
@@ -80,15 +78,20 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 
 #endif
 
+#if OO_USE_CUSTOM_LOAD_SAVE
+
 - (void) setGuiToLoadCommanderScreen;
 - (void) setGuiToSaveCommanderScreen: (NSString *)cdrName;
 - (void) setGuiToOverwriteScreen: (NSString *)cdrName;
 - (void) lsCommanders: (GuiDisplayGen *)gui directory: (NSString*)directory pageNumber: (int)page highlightName: (NSString *)highlightName;
-- (void) writePlayerToPath:(NSString *)path;
-- (void) nativeSavePlayer: (NSString *)cdrName;
-- (BOOL) existingNativeSave: (NSString *)cdrName;
 - (void) showCommanderShip: (int)cdrArrayIndex;
 - (int) findIndexOfCommander: (NSString *)cdrName;
+- (void) nativeSavePlayer: (NSString *)cdrName;
+- (BOOL) existingNativeSave: (NSString *)cdrName;
+
+#endif
+
+- (void) writePlayerToPath:(NSString *)path;
 
 @end
 
@@ -99,7 +102,9 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 {
 	BOOL				OK = YES;
 	
-#if OOLITE_USE_APPKIT_LOAD_SAVE
+#if OO_USE_APPKIT_LOAD_SAVE_ALWAYS
+	OK = [self loadPlayerWithPanel];
+#elif OOLITE_USE_APPKIT_LOAD_SAVE
 	// OS X: use system open/save dialogs in windowed mode, custom interface in full-screen.
 	if ([[UNIVERSE gameController] inFullScreenMode])
 	{
@@ -119,11 +124,13 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 
 - (void)savePlayer
 {
-#if OOLITE_USE_APPKIT_LOAD_SAVE
+#if OO_USE_APPKIT_LOAD_SAVE_ALWAYS
+	[self savePlayerWithPanel];
+#elif OOLITE_USE_APPKIT_LOAD_SAVE
 	// OS X: use system open/save dialogs in windowed mode, custom interface in full-screen.
 	if ([[UNIVERSE gameController] inFullScreenMode])
 	{
-		[self setGuiToSaveCommanderScreen:player_name];
+		[self setGuiToSaveCommanderScreen:self.lastsaveName];
 	}
 	else
 	{
@@ -131,7 +138,7 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	}
 #else
 	// Other platforms: use custom interface all the time.
-	[self setGuiToSaveCommanderScreen:player_name];
+	[self setGuiToSaveCommanderScreen:[self lastsaveName]];
 #endif
 }
 
@@ -141,32 +148,37 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	NSString		*tmp_name = nil;
 	NSString		*dir = [[UNIVERSE gameController] playerFileDirectory];
 	
-	tmp_name = player_name;
+	tmp_name = [self lastsaveName];
 	tmp_path = save_path;
 	
 	ShipScriptEventNoCx(self, "playerWillSaveGame", OOJSSTR("AUTO_SAVE"));
 	
-	NSString *saveName = player_name;
-	if (![player_name hasSuffix:DESC(@"autosave-commander-suffix")])
-				saveName = [player_name stringByAppendingString:DESC(@"autosave-commander-suffix")];
+	NSString *saveName = [self lastsaveName];
+	NSString *autosaveSuffix = DESC(@"autosave-commander-suffix");
+	
+	if (![saveName hasSuffix:autosaveSuffix])
+	{
+		saveName = [saveName stringByAppendingString:autosaveSuffix];
+	}
 	NSString *savePath = [dir stringByAppendingPathComponent:[saveName stringByAppendingString:@".oolite-save"]];
 	
-	[player_name autorelease];
-	player_name = [saveName copy];
+	[self setLastsaveName:saveName];
 	
-	NS_DURING
+	@try
+	{
 		[self writePlayerToPath:savePath];
-	NS_HANDLER
+	}
+	@catch (id exception)
+	{
 		// Suppress exceptions silently. Warning the user about failed autosaves would be pretty unhelpful.
-	NS_ENDHANDLER
+	}
 	
 	if (tmp_path != nil)
 	{
 		[save_path autorelease];
 		save_path = [tmp_path copy];
 	}
-	[player_name autorelease];
-	player_name = [tmp_name copy];
+	[self setLastsaveName:tmp_name];
 }
 
 
@@ -192,6 +204,161 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 }
 
 
+- (void) setGuiToScenarioScreen:(int)page
+{
+	NSArray *scenarios = [UNIVERSE scenarios];
+	[UNIVERSE removeDemoShips];
+	// GUI stuff
+	{
+		GuiDisplayGen	*gui = [UNIVERSE gui];
+		OOGUIRow		start_row = GUI_ROW_SCENARIOS_START;
+		OOGUIRow		row = start_row;
+		BOOL			guiChanged = (gui_screen != GUI_SCREEN_NEWGAME);
+
+		[gui clearAndKeepBackground:!guiChanged];
+		[gui setTitle:DESC(@"oolite-newgame-title")];
+
+		OOGUITabSettings tab_stops;
+		tab_stops[0] = 0;
+		tab_stops[1] = -480;
+		[gui setTabStops:tab_stops];
+
+		unsigned n_rows = GUI_MAX_ROWS_SCENARIOS;
+		NSUInteger i, count = [scenarios count];
+
+		NSDictionary *scenario = nil;
+
+		[gui setArray:[NSArray arrayWithObjects:DESC(@"oolite-scenario-exit"), @" <----- ", nil] forRow:start_row - 2];
+		[gui setColor:[OOColor redColor] forRow:start_row - 2];
+		[gui setKey:@"exit" forRow:start_row - 2];
+		
+
+		if (page > 0)
+		{
+			[gui setArray:[NSArray arrayWithObjects:DESC(@"gui-back"), @" <-- ", nil] forRow:start_row - 1];
+			[gui setColor:[OOColor greenColor] forRow:start_row - 1];
+			[gui setKey:[NSString stringWithFormat:@"__page:%i",page-1] forRow:start_row - 1];
+		}
+
+		[self setShowDemoShips:NO];
+
+		for (i = page*n_rows ; i < count && row < start_row + n_rows ; i++)
+		{
+			scenario = [[UNIVERSE scenarios] objectAtIndex:i];
+			NSString *scenarioName = [NSString stringWithFormat:@" %@ ",[scenario oo_stringForKey:@"name"]];
+			[gui setText:OOExpand(scenarioName) forRow:row];
+			[gui setKey:[NSString stringWithFormat:@"Scenario:%lu", (unsigned long)i] forRow:row];
+			++row;
+		}
+
+		if ((page+1) * n_rows < count)
+		{
+			[gui setArray:[NSArray arrayWithObjects:DESC(@"gui-more"), @" --> ", nil] forRow:row];
+			[gui setColor:[OOColor greenColor] forRow:row];
+			[gui setKey:[NSString stringWithFormat:@"__page:%i",page+1] forRow:row];
+			++row;
+		}
+		
+		[gui setSelectableRange:NSMakeRange(start_row - 2,3 + row - start_row)];
+		[gui setSelectedRow:start_row];
+		[self showScenarioDetails];
+
+		gui_screen = GUI_SCREEN_NEWGAME;
+	
+		if (guiChanged)
+		{
+			[gui setBackgroundTextureKey:@"newgame"];
+			[gui setForegroundTextureKey:@"newgame_overlay"];
+		}
+	}
+	
+	[UNIVERSE enterGUIViewModeWithMouseInteraction:YES];
+}
+
+- (void) addScenarioModel:(NSString *)shipKey
+{
+	[self showShipModelWithKey:shipKey shipData:nil personality:0 factorX:1.2 factorY:0.8 factorZ:6.4 inContext:@"scenario"];
+}
+
+
+- (void) showScenarioDetails
+{
+	GuiDisplayGen* gui = [UNIVERSE gui];
+	NSString* key = [gui selectedRowKey];
+	[UNIVERSE removeDemoShips];
+
+	if ([key hasPrefix:@"Scenario"])
+	{
+		int item = [[key componentsSeparatedByString:@":"] oo_intAtIndex:1];
+		NSDictionary *scenario = [[UNIVERSE scenarios] objectAtIndex:item];
+		[self setShowDemoShips:NO];
+		for (NSUInteger i=GUI_ROW_SCENARIOS_DETAIL;i<=27;i++)
+		{
+			[gui setText:@"" forRow:i];
+		}
+		if (scenario)
+		{
+			[gui addLongText:OOExpand([scenario oo_stringForKey:@"description"]) startingAtRow:GUI_ROW_SCENARIOS_DETAIL align:GUI_ALIGN_LEFT];
+			NSString *shipKey = [scenario oo_stringForKey:@"model"];
+			if (shipKey != nil)
+			{
+				[self addScenarioModel:shipKey];
+				[self setShowDemoShips:YES];
+			}
+		}
+
+	}
+}
+
+
+- (BOOL) startScenario
+{
+	GuiDisplayGen* gui = [UNIVERSE gui];
+	NSString* key = [gui selectedRowKey];
+
+	if ([key isEqualToString:@"exit"])
+	{
+		// intended to return to main menu
+		return NO; 
+	}
+	if ([key hasPrefix:@"__page"])
+	{
+		int page = [[key componentsSeparatedByString:@":"] oo_intAtIndex:1];
+		[self setGuiToScenarioScreen:page];
+		return YES;
+	}
+	int selection = [[key componentsSeparatedByString:@":"] oo_intAtIndex:1];
+
+	NSDictionary *scenario = [[UNIVERSE scenarios] objectAtIndex:selection];
+	NSString *file = [scenario oo_stringForKey:@"file" defaultValue:nil];
+	if (file == nil) 
+	{
+		OOLog(@"scenario.init.error",@"No file entry found for scenario");
+		return NO;
+	}
+	NSString *path = [ResourceManager pathForFileNamed:file inFolder:@"Scenarios"];
+	if (path == nil)
+	{
+		OOLog(@"scenario.init.error",@"Game file not found for scenario %@",file);
+		return NO;
+	}
+	BOOL result = [self loadPlayerFromFile:path asNew:YES];
+	if (!result)
+	{
+		return NO;
+	}
+	[scenarioKey release];
+	scenarioKey = [[scenario oo_stringForKey:@"scenario" defaultValue:nil] retain];
+
+	// don't drop the save game directory in
+	return YES;
+}
+
+
+
+
+#if OO_USE_CUSTOM_LOAD_SAVE
+
 - (NSString *)commanderSelector
 {
 	MyOpenGLView	*gameView = [UNIVERSE gameView];
@@ -203,10 +370,18 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	{
 		int guiSelectedRow=[gui selectedRow];
 		idx=(guiSelectedRow - STARTROW) + (currentPage * NUMROWS);
-		if (guiSelectedRow != MOREROW && guiSelectedRow != BACKROW)
+		if (guiSelectedRow != MOREROW && guiSelectedRow != BACKROW && guiSelectedRow != EXITROW)
 		{
 			[self showCommanderShip: idx];
 		}
+		else
+		{
+			[UNIVERSE removeDemoShips];
+			[gui setText:@"" forRow:CDRDESCROW align:GUI_ALIGN_LEFT];
+			[gui setText:@"" forRow:CDRDESCROW + 1 align:GUI_ALIGN_LEFT];
+			[gui setText:@"" forRow:CDRDESCROW + 2 align:GUI_ALIGN_LEFT];
+		}
+
 	}
 	else
 	{
@@ -236,6 +411,13 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 		NSDictionary *cdr;
 		switch ([gui selectedRow])
 		{
+			case EXITROW:
+				if ([self status] == STATUS_START_GAME)
+				{
+					[self setGuiToIntroFirstGo:YES];
+					return nil;
+				}
+				break;
 			case BACKROW:
 				currentPage--;
 				[self lsCommanders: gui	directory: dir	pageNumber: currentPage  highlightName: nil];
@@ -288,13 +470,23 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 		{
 			[self showCommanderShip: idx];
 			if ([(NSDictionary *)[cdrDetailArray objectAtIndex:idx] oo_boolForKey:@"isSavedGame"])	// don't show things that aren't saved games
-				commanderNameString = [[cdrDetailArray oo_dictionaryAtIndex:idx] oo_stringForKey:@"player_name"];
+				commanderNameString = [[cdrDetailArray oo_dictionaryAtIndex:idx] oo_stringForKey:@"player_save_name" defaultValue:[[cdrDetailArray oo_dictionaryAtIndex:idx] oo_stringForKey:@"player_name"]];
 			else
 				commanderNameString = [gameView typedString];
 		}
+		else
+		{
+			[UNIVERSE removeDemoShips];
+			[gui setText:@"" forRow:CDRDESCROW align:GUI_ALIGN_LEFT];
+			[gui setText:@"" forRow:CDRDESCROW + 1 align:GUI_ALIGN_LEFT];
+			[gui setText:@"" forRow:CDRDESCROW + 2 align:GUI_ALIGN_LEFT];
+		}
 	}
 	else
+	{
 		commanderNameString = [gameView typedString];
+	}
+	
 	[gameView setTypedString: commanderNameString];
 	
 	[gui setText:
@@ -303,14 +495,16 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	[gui setColor:[OOColor cyanColor] forRow:INPUTROW];
 	
 	// handle page <-- and page --> keys, and on-screen buttons
-	if ((([gameView isDown:gvMouseDoubleClick] && [gui selectedRow] == BACKROW) || [gameView isDown:gvArrowKeyLeft]) && [[gui keyForRow:BACKROW] isEqual: GUI_KEY_OK])
+	if (((([gameView isDown:gvMouseDoubleClick] || [gameView isDown: 13]) && [gui selectedRow] == BACKROW) || [gameView isDown:gvArrowKeyLeft])
+					&& [[gui keyForRow:BACKROW] isEqual: GUI_KEY_OK])
 	{
 		currentPage--;
 		[self lsCommanders: gui	directory: dir	pageNumber: currentPage  highlightName: nil];
 		[gameView supressKeysUntilKeyUp];
 	}
 	//
-	if ((([gameView isDown:gvMouseDoubleClick] && [gui selectedRow] == MOREROW) || [gameView isDown:gvArrowKeyRight]) && [[gui keyForRow:MOREROW] isEqual: GUI_KEY_OK])
+	if (((([gameView isDown:gvMouseDoubleClick] || [gameView isDown: 13]) && [gui selectedRow] == MOREROW) || [gameView isDown:gvArrowKeyRight])
+					&& [[gui keyForRow:MOREROW] isEqual: GUI_KEY_OK])
 	{
 		currentPage++;
 		[self lsCommanders: gui	directory: dir	pageNumber: currentPage  highlightName: nil];
@@ -324,6 +518,7 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 			int guiSelectedRow=[gui selectedRow];
 			int	idx = (guiSelectedRow - STARTROW) + (currentPage * NUMROWS);
 			NSDictionary* cdr = [cdrDetailArray objectAtIndex:idx];
+			
 			if (![cdr oo_boolForKey:@"isSavedGame"])	// don't open saved games
 			{
 				// change directory to the selected path
@@ -395,8 +590,10 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	}
 }
 
+#endif
 
-- (BOOL) loadPlayerFromFile:(NSString *)fileToOpen
+
+- (BOOL) loadPlayerFromFile:(NSString *)fileToOpen asNew:(BOOL)asNew
 {
 	/*	TODO: it would probably be better to load by creating a new
 		PlayerEntity, verifying that's OK, then replacing the global player.
@@ -419,6 +616,7 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	
 	if (loadedOK)
 	{
+		OOLog(@"load.progress",@"Reading file");
 		fileDic = OODictionaryFromFile(fileToOpen);
 		if (fileDic == nil)
 		{
@@ -429,22 +627,33 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 
 	if (loadedOK)
 	{
-		NSString		*shipKey = nil;
-		NSDictionary	*shipDict = nil;
-		
-		shipKey = [fileDic oo_stringForKey:@"ship_desc"];
-		shipDict = [[OOShipRegistry sharedRegistry] shipInfoForKey:shipKey];
-		
-		if (shipDict == nil && [UNIVERSE strict] && shipKey != nil)
+		OOLog(@"load.progress",@"Restricting scenario");
+		NSString *scenarioRestrict = [fileDic oo_stringForKey:@"scenario_restriction" defaultValue:nil];
+		if (scenarioRestrict == nil)
 		{
-			fail_reason = [NSString stringWithFormat:DESC(@"loadfailed-could-not-use-ship-type-@-please-switch-to-unrestricted"), shipKey];
-			loadedOK = NO;
+			// older save game - use the 'strict' key instead
+			BOOL strict = [fileDic oo_boolForKey:@"strict" defaultValue:NO];
+			if (strict)
+			{
+				scenarioRestrict = SCENARIO_OXP_DEFINITION_NONE;
+			}
+			else
+			{
+				scenarioRestrict = SCENARIO_OXP_DEFINITION_ALL;
+			}
 		}
-	}	
-		
+
+		if (![UNIVERSE setUseAddOns:scenarioRestrict fromSaveGame:YES forceReinit:YES]) 
+		{
+			fail_reason = DESC(@"loadfailed-saved-game-failed-to-load");
+			loadedOK = NO;
+		} 
+	}
+	
 
 	if (loadedOK)
 	{
+		OOLog(@"load.progress",@"Creating player ship");
 		// Check that player ship exists
 		NSString		*shipKey = nil;
 		NSDictionary	*shipDict = nil;
@@ -462,16 +671,21 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	
 	if (loadedOK)
 	{
-		[self setUp];
-		if ([self setCommanderDataFromDictionary:fileDic])
+		OOLog(@"load.progress",@"Initialising player entity");
+		if (![self setUpAndConfirmOK:YES saveGame:YES])
 		{
-			// Remember the savegame target.
-			Random_Seed target = target_system_seed;
-			[self completeSetUp];
-			target_system_seed = target;
+			fail_reason = DESC(@"loadfailed-could-not-reset-javascript");
+			loadedOK = NO;
 		}
-		else
+	}
+	
+	if (loadedOK)
+	{
+		OOLog(@"load.progress",@"Loading commander data");
+		if (![self setCommanderDataFromDictionary:fileDic])
 		{
+			// this could still be a reset js issue, if switching from strict / unrestricted
+			// TODO: use "could not reset js message" if that's the case.
 			fail_reason = DESC(@"loadfailed-could-not-set-up-player-ship");
 			loadedOK = NO;
 		}
@@ -479,11 +693,15 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	
 	if (loadedOK)
 	{
-		[save_path autorelease];
-		save_path = [fileToOpen retain];
+		OOLog(@"load.progress",@"Recording save path");
+		if (!asNew)
+		{
+			[save_path autorelease];
+			save_path = [fileToOpen retain];
 		
-		[[[UNIVERSE gameView] gameController] setPlayerFileToLoad:fileToOpen];
-		[[[UNIVERSE gameView] gameController] setPlayerFileDirectory:fileToOpen];
+			[[[UNIVERSE gameView] gameController] setPlayerFileToLoad:fileToOpen];
+			[[[UNIVERSE gameView] gameController] setPlayerFileDirectory:fileToOpen];
+		}
 	}
 	else
 	{
@@ -496,16 +714,20 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 		return NO;
 	}
 	
+	OOLog(@"load.progress",@"Creating system");
 	[UNIVERSE setTimeAccelerationFactor:TIME_ACCELERATION_FACTOR_DEFAULT];
-	[UNIVERSE setSystemTo:system_seed];
+	[UNIVERSE setSystemTo:system_id];
 	[UNIVERSE removeAllEntitiesExceptPlayer];
+	[UNIVERSE setGalaxyTo: galaxy_number andReinit:YES]; // set overridden planet names on long range map
 	[UNIVERSE setUpSpace];
 	[UNIVERSE setAutoSaveNow:NO];
 	
-	[self setStatus:STATUS_DOCKED];
-	[UNIVERSE setViewDirection:VIEW_GUI_DISPLAY];
+	OOLog(@"load.progress",@"Resetting player flight variables");
+	[self setDockedAtMainStation];
+	StationEntity *dockedStation = [self dockedStation];
+	
+	[UNIVERSE enterGUIViewModeWithMouseInteraction:NO];
 	
-	dockedStation = [UNIVERSE station];
 	if (dockedStation)
 	{
 		position = [dockedStation position];
@@ -522,17 +744,62 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	
 	[self setEntityPersonalityInt:PersonalityForCommanderDict(fileDic)];
 	
-	if (![dockedStation localMarket])
+	OOLog(@"load.progress",@"Loading system market");
+	// dockedStation is always the main station at this point;
+	// "localMarket" save key always refers to the main station (system) market
+	NSArray *market = [fileDic oo_arrayForKey:@"localMarket"];
+	if (market != nil) 
 	{
-		NSArray *market = [fileDic oo_arrayForKey:@"localMarket"];
-		if (market != nil)  [dockedStation setLocalMarket:market];
-		else  [dockedStation initialiseLocalMarketWithRandomFactor:market_rnd];
+		[dockedStation setLocalMarket:market];
 	}
+	else
+	{
+		[dockedStation initialiseLocalMarket];
+	}
+
 	[self calculateCurrentCargo];
-	[UNIVERSE setGalaxySeed: galaxy_seed andReinit:YES]; // set overridden planet names on long range map
+	
+	OOLog(@"load.progress",@"Setting scenario key");
+	// set scenario key if the scenario allows saving and has one
+	NSString *scenario = [fileDic oo_stringForKey:@"scenario_key" defaultValue:nil];
+	DESTROY(scenarioKey);
+	if (scenario != nil)
+	{
+		scenarioKey = [scenario retain];
+	}
+
+	OOLog(@"load.progress",@"Starting JS engine");
+	// Remember the savegame target, run js startUp.
+	[self completeSetUpAndSetTarget:NO];
+	// run initial system population
+	OOLog(@"load.progress",@"Populating initial system");
+	[UNIVERSE populateNormalSpace];
+
+	// might as well start off with a collected JS environment
+	[[OOJavaScriptEngine sharedEngine] garbageCollectionOpportunity:YES];
+	
+	// read saved position vector and primary role, check for an
+	// appropriate station at those coordinates, if found, switch
+	// docked station to that one.
+	HPVector dockedPos = [fileDic oo_hpvectorForKey:@"docked_station_position"];
+	NSString *dockedRole = [fileDic oo_stringForKey:@"docked_station_role" defaultValue:@""];
+	StationEntity *saveStation = [UNIVERSE stationWithRole:dockedRole andPosition:dockedPos];
+	if (saveStation != nil && [saveStation allowsSaving])
+	{
+		[self setDockedStation:saveStation];
+		position = [saveStation position];
+	}
+	// and initialise markets for the secondary stations
+	[UNIVERSE loadStationMarkets:[fileDic oo_arrayForKey:@"station_markets"]];
+
+	OOLog(@"load.progress",@"Completing JS startup");
+	[self startUpComplete];
+
 	[[UNIVERSE gameView] supressKeysUntilKeyUp];
+	gui_screen = GUI_SCREEN_LOAD; // force evaluation of new gui screen on startup
 	[self setGuiToStatusScreen];
 	if (loadedOK) [self doWorldEventUntilMissionScreen:OOJSID("missionScreenOpportunity")];  // trigger missionScreenOpportunity immediately after loading
+	OOLog(@"load.progress",@"Loading complete");
 	return loadedOK;
 }
 
@@ -545,50 +812,44 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 
 - (BOOL)loadPlayerWithPanel
 {
-	int				result;
-	NSArray			*fileTypes = nil;
-	NSOpenPanel		*oPanel = nil;
+	NSOpenPanel *oPanel = [NSOpenPanel openPanel];
 	
-	fileTypes = [NSArray arrayWithObject:@"oolite-save"];
-	oPanel = [NSOpenPanel openPanel];
+	oPanel.allowsMultipleSelection = NO;
+	oPanel.allowedFileTypes = [NSArray arrayWithObject:@"oolite-save"];
 	
-	[oPanel setAllowsMultipleSelection:NO];
-	result = [oPanel runModalForDirectory:nil file:nil types:fileTypes];
-	if (result == NSOKButton)
+	if ([oPanel runModal] == NSOKButton)
 	{
-		return [self loadPlayerFromFile:[oPanel filename]];
-	}
-	else
-	{
-		return NO;
+		NSURL *url = oPanel.URL;
+		if (url.isFileURL)
+		{
+			return [self loadPlayerFromFile:url.path asNew:NO];
+		}
 	}
+	
+	return NO;
 }
 
 
 - (void) savePlayerWithPanel
 {
-	NSSavePanel		*sp;
-	int				runResult;
+	NSSavePanel *sPanel = [NSSavePanel savePanel];
 	
-	sp = [NSSavePanel savePanel];
-	[sp setRequiredFileType:@"oolite-save"];
-	[sp setCanSelectHiddenExtension:YES];
+	sPanel.allowedFileTypes = [NSArray arrayWithObject:@"oolite-save"];
+	sPanel.canSelectHiddenExtension = YES;
+	sPanel.nameFieldStringValue = self.lastsaveName;
 	
-	// display the NSSavePanel
-	runResult = [sp runModalForDirectory:nil file:player_name];
-	
-	// if successful, save file under designated name
-	if (runResult == NSOKButton)
+	if ([sPanel runModal] == NSOKButton)
 	{
-		NSArray*	path_components = [[sp filename] pathComponents];
-		NSString*   new_name = [[path_components objectAtIndex:[path_components count]-1] stringByDeletingPathExtension];
+		NSURL *url = sPanel.URL;
+		NSAssert(url.isFileURL, @"Save panel with default configuration should not provide non-file URLs.");
+		
+		NSString *path = url.path;
+		NSString *newName = [path.lastPathComponent stringByDeletingPathExtension];
 		
 		ShipScriptEventNoCx(self, "playerWillSaveGame", OOJSSTR("STANDARD_SAVE"));
 		
-		[player_name release];
-		player_name = [new_name copy];
-		
-		[self writePlayerToPath:[sp filename]];
+		self.lastsaveName = newName;
+		[self writePlayerToPath:path];
 	}
 	[self setGuiToStatusScreen];
 }
@@ -614,13 +875,14 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	else  didSave = [dict writeOOXMLToFile:path atomically:YES errorDescription:&errDesc];
 	if (didSave)
 	{
-		
 		[UNIVERSE clearPreviousMessage];	// allow this to be given time and again
 		[UNIVERSE addMessage:DESC(@"game-saved") forCount:2];
 		[save_path autorelease];
 		save_path = [path copy];
 		[[UNIVERSE gameController] setPlayerFileToLoad:save_path];
 		[[UNIVERSE gameController] setPlayerFileDirectory:save_path];
+		// no duplicated autosave immediately after a save.
+		[UNIVERSE setAutoSaveNow:NO];
 	}
 	else
 	{
@@ -640,13 +902,13 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	
 	ShipScriptEventNoCx(self, "playerWillSaveGame", OOJSSTR("STANDARD_SAVE"));
 	
-	[player_name release];
-	player_name = [cdrName copy];
+	[self setLastsaveName:cdrName];
 	
 	[self writePlayerToPath:savePath];
 }
 
 
+#if OO_USE_CUSTOM_LOAD_SAVE
 
 - (void) setGuiToLoadCommanderScreen
 {
@@ -666,23 +928,22 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	
 	[[UNIVERSE gameView] supressKeysUntilKeyUp];
 	
-	[self setShowDemoShips: YES];
-	[UNIVERSE setDisplayCursor: YES];
-	[UNIVERSE setViewDirection: VIEW_GUI_DISPLAY];
+	[self setShowDemoShips:YES];
+	[UNIVERSE enterGUIViewModeWithMouseInteraction:YES];
 }
 
 
-- (void) setGuiToSaveCommanderScreen: (NSString *)cdrName
+- (void) setGuiToSaveCommanderScreen:(NSString *)cdrName
 {
 	GuiDisplayGen *gui=[UNIVERSE gui];
-	MyOpenGLView*	gameView = [UNIVERSE gameView];
-	NSString*	dir = [[UNIVERSE gameController] playerFileDirectory];
+	MyOpenGLView *gameView = [UNIVERSE gameView];
+	NSString *dir = [[UNIVERSE gameController] playerFileDirectory];
 	
 	pollControls = NO;
 	gui_screen = GUI_SCREEN_SAVE;
 	
 	[gui clear];
-	[gui setTitle:[NSString stringWithFormat:DESC(@"savescreen-title")]];
+	[gui setTitle:DESC(@"savescreen-title")];
 	
 	currentPage = 0;
 	[self lsCommanders:gui directory:dir pageNumber: currentPage highlightName:nil];
@@ -695,16 +956,15 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	[gui setForegroundTextureKey:@"docked_overlay"];
 	[gui setBackgroundTextureKey:@"load_save"];
 	
-	[gameView setTypedString: cdrName];
+	[gameView setTypedString:cdrName];
 	[gameView supressKeysUntilKeyUp];
 	
-	[self setShowDemoShips: YES];
-	[UNIVERSE setDisplayCursor: YES];
-	[UNIVERSE setViewDirection: VIEW_GUI_DISPLAY];
+	[self setShowDemoShips:YES];
+	[UNIVERSE enterGUIViewModeWithMouseInteraction:YES];
 }
 
 
-- (void) setGuiToOverwriteScreen: (NSString *)cdrName
+- (void) setGuiToOverwriteScreen:(NSString *)cdrName
 {
 	GuiDisplayGen *gui=[UNIVERSE gui];
 	MyOpenGLView*	gameView = [UNIVERSE gameView];
@@ -734,12 +994,15 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	[gui setForegroundTextureKey:@"overwrite_overlay"];
 	[gui setBackgroundTextureKey:@"load_save"];
 	
-	[self setShowDemoShips: NO];
-	[UNIVERSE setDisplayCursor: NO];
-	[gameView setStringInput: gvStringInputNo];
-	[UNIVERSE setViewDirection: VIEW_GUI_DISPLAY];
+	[self setShowDemoShips:NO];
+	[gameView setStringInput:gvStringInputNo];
+	[UNIVERSE enterGUIViewModeWithMouseInteraction:NO];	// FIXME: should be YES, but was NO before introducing new mouse mode stuff. If set to YES, choices can be selected but not activated.
 }
 
+NSComparisonResult sortCommanders(id cdr1, id cdr2, void *context)
+{
+	return [[cdr1 objectForKey:@"saved_game_path"] localizedCompare:[cdr2 objectForKey:@"saved_game_path"]];
+}
 
 - (void) lsCommanders: (GuiDisplayGen *)gui
 			directory: (NSString*) directory
@@ -800,6 +1063,8 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 		[gui setText:DESC(@"loadsavescreen-no-commanders-found") forRow:STARTROW align:GUI_ALIGN_CENTER];
 		return;
 	}
+
+	[cdrDetailArray sortUsingFunction:sortCommanders context:NULL];
 	
 	// Do we need to highlight a name?
 	int highlightRowOnPage=STARTROW;
@@ -828,18 +1093,19 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	tabStop[1]=160;
 	tabStop[2]=270;
 	[gui setTabStops: tabStop];
-	[gui setColor: [OOColor greenColor] forRow: LABELROW];
-	[gui setArray: [NSArray arrayWithObjects: DESC(@"loadsavescreen-commander-name"), DESC(@"loadsavescreen-rating"), nil]
-		   forRow:LABELROW];
 	
 	// clear text lines here
-	for (i = STARTROW - 1; i < ENDROW + 1; i++)
+	for (i = EXITROW ; i < ENDROW + 1; i++)
 	{
 		[gui setText:@"" forRow:i align:GUI_ALIGN_LEFT];
 		[gui setColor: [OOColor yellowColor] forRow: i];
 		[gui setKey:GUI_KEY_SKIP forRow:i];
 	}
-	
+
+	[gui setColor: [OOColor greenColor] forRow: LABELROW];
+	[gui setArray: [NSArray arrayWithObjects: DESC(@"loadsavescreen-commander-name"), DESC(@"loadsavescreen-rating"), nil]
+		   forRow:LABELROW];
+
 	if (page)
 	{
 		[gui setColor:[OOColor greenColor] forRow:STARTROW-1];
@@ -848,11 +1114,20 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 		[gui setKey:GUI_KEY_OK forRow:STARTROW-1];
 		rangeStart=STARTROW-1;
 	}
+
+	if ([self status] == STATUS_START_GAME)
+	{
+		[gui setArray:[NSArray arrayWithObjects:DESC(@"oolite-loadsave-exit"), @" <----- ", nil] forRow:EXITROW];
+		[gui setColor:[OOColor redColor] forRow:EXITROW];
+		[gui setKey:GUI_KEY_OK forRow:EXITROW];
+		rangeStart = EXITROW;
+	}
+
 	
 	if (firstIndex + NUMROWS >= [cdrDetailArray count])
 	{
 		lastIndex=[cdrDetailArray count];
-		[gui setSelectableRange: NSMakeRange(rangeStart, rangeStart + NUMROWS)];
+		[gui setSelectableRange: NSMakeRange(rangeStart, rangeStart + NUMROWS + 2)];
 	}
 	else
 	{
@@ -871,12 +1146,14 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 		{
 			NSString *ratingDesc = OODisplayRatingStringFromKillCount([cdr oo_unsignedIntForKey:@"ship_kills"]);
 			[gui setArray:[NSArray arrayWithObjects:
-				[NSString stringWithFormat:@" %@ ",[cdr oo_stringForKey:@"player_name"]],
+				[NSString stringWithFormat:@" %@ ",[cdr oo_stringForKey:@"player_save_name" defaultValue:[cdr oo_stringForKey:@"player_name"]]],
 				[NSString stringWithFormat:@" %@ ",ratingDesc],
 				nil]
 				   forRow:row];
-			if ([player_name isEqualToString:[cdr oo_stringForKey:@"player_name"]])
+			if ([[self lastsaveName] isEqualToString:[cdr oo_stringForKey:@"player_save_name" defaultValue:[cdr oo_stringForKey:@"player_name"]]])
+			{
 				highlightRowOnPage = row;
+			}
 			
 			[gui setKey:GUI_KEY_OK forRow:row];
 			row++;
@@ -947,12 +1224,11 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 		[gui addLongText: folderDesc startingAtRow: CDRDESCROW align: GUI_ALIGN_LEFT];
 		return;
 	}
-	[gui setColor: [OOColor yellowColor] forRow: CDRDESCROW];
+	[gui setColor:[gui colorFromSetting:nil defaultValue:nil] forRow: CDRDESCROW];
 
-	if (![cdr oo_boolForKey:@"isSavedGame"])	// don't show things that aren't saved games
-		return;
+	if (![cdr oo_boolForKey:@"isSavedGame"])  return;	// don't show things that aren't saved games
 	
-	if(!dockedStation)  dockedStation = [UNIVERSE station];
+	if ([self dockedStation] == nil)  [self setDockedAtMainStation];
 	
 	// Display the commander's ship.
 	NSString			*shipDesc = [cdr oo_stringForKey:@"ship_desc"];
@@ -978,7 +1254,14 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	{
 		[self showShipyardModel:@"oolite-unknown-ship" shipData:nil personality:personality];
 		shipName = [cdr oo_stringForKey:@"ship_name" defaultValue:@"unknown"];
-		shipName = [shipName stringByAppendingString:@" - OXP not installed"];
+		if (![[UNIVERSE useAddOns] isEqualToString:SCENARIO_OXP_DEFINITION_ALL])
+		{
+			shipName = [shipName stringByAppendingString:@" - OXPs disabled or not installed"];
+		}
+		else
+		{
+			shipName = [shipName stringByAppendingString:@" - OXP not installed"];
+		}
 	}
 	
 	// Make a short description of the commander
@@ -990,26 +1273,18 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	// Nikos - Add some more information in the load game screen (current location, galaxy number and timestamp).
 	//-------------------------------------------------------------------------------------------------------------------------
 	
-	// Store the current galaxy seed because findSystemNumberAtCoords may alter it in a while.
-	PlayerEntity		*player = PLAYER;
-	Random_Seed		player_galaxy_seed = [player galaxy_seed];	
-	
 	int			galNumber;
 	NSString		*timeStamp  = nil;
 	NSString 		*locationName = [cdr oo_stringForKey:@"current_system_name"];
 	
-	// If there is no key containing the name of the current system in the savefile, fall back to
-	// extracting the name from the galaxy seed and coordinates information.
+	// If there is no key containing the name of the current system in
+	// the savefile, calculating what it should have been is going to
+	// be tricky now that system generation isn't seed based - but
+	// this implies a save game well over 5 years old.
 	if (locationName == nil)
 	{	
-		Random_Seed		gal_seed;
-		NSPoint			gal_coords;
-		int			locationNumber;
-		
-		gal_coords = PointFromString([cdr oo_stringForKey:@"galaxy_coordinates"]);
-		gal_seed = RandomSeedFromString([cdr oo_stringForKey:@"galaxy_seed"]);
-		locationNumber = [UNIVERSE findSystemNumberAtCoords:gal_coords withGalaxySeed:gal_seed];
-		locationName = [UNIVERSE systemNameIndex:locationNumber];
+		// Leaving the location blank in this case is probably okay
+		locationName = @"";
 	}
 	
 	galNumber = [cdr oo_intForKey:@"galaxy_number"] + 1;	// Galaxy numbering starts at 0.
@@ -1032,8 +1307,6 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	
 	[gui addLongText:cdrDesc startingAtRow:CDRDESCROW align:GUI_ALIGN_LEFT];
 	
-	// Restore the seed of the galaxy the player is currently in.
-	[UNIVERSE setGalaxySeed: player_galaxy_seed];
 }
 
 
@@ -1042,7 +1315,7 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	unsigned i;
 	for (i=0; i < [cdrDetailArray count]; i++)
 	{
-		NSString *currentName = [[cdrDetailArray oo_dictionaryAtIndex: i] oo_stringForKey:@"player_name"];
+		NSString *currentName = [[cdrDetailArray oo_dictionaryAtIndex: i] oo_stringForKey:@"player_save_name" defaultValue:[[cdrDetailArray oo_dictionaryAtIndex: i] oo_stringForKey:@"player_name"]];
 		if([cdrName compare: currentName] == NSOrderedSame)
 		{
 			return i;
@@ -1053,23 +1326,24 @@ static uint16_t PersonalityForCommanderDict(NSDictionary *dict);
 	return -1;
 }
 
+#endif
+
 @end
 
 
+#if OO_USE_CUSTOM_LOAD_SAVE
+
 @implementation MyOpenGLView (OOLoadSaveExtensions)
 
 - (BOOL)isCommandModifierKeyDown
 {
-#if OOLITE_MAC_OS_X
-	//return [self isCommandDown];
 	return [self isCtrlDown];
-#else
-	return [self isCtrlDown];
-#endif
 }
 
 @end
 
+#endif
+
 
 static uint16_t PersonalityForCommanderDict(NSDictionary *dict)
 {
