@@ -6,7 +6,7 @@ Additional methods relating to behaviour/artificial intelligence.
 
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,12 +34,44 @@ MA 02110-1301, USA.
 //	AI methods also used in other code.
 
 - (void) setAITo:(NSString *)aiString;
+- (void) setAIScript:(NSString *)aiString;
 - (void) switchAITo:(NSString *)aiString;
 
 - (void) scanForHostiles;
-- (void) performTumble;
-- (void) performStop;
 - (BOOL) performHyperSpaceToSpecificSystem:(OOSystemID)systemID;
 - (void) scanForNearestIncomingMissile;
 
+- (void) enterTargetWormhole;
+- (void) enterPlayerWormhole;
+
+- (void) wormholeEscorts;
+- (void) wormholeEntireGroup;
+
+- (BOOL) suggestEscortTo:(ShipEntity *)mother;
+
+- (void) groupAttackTarget;
+
+- (void) performAttack;
+- (void) performCollect;
+- (void) performEscort;
+- (void) performFaceDestination;
+- (void) performFlee;
+- (void) performFlyToRangeFromDestination;
+- (void) performHold;
+- (void) performIdle;
+- (void) performIntercept;
+- (void) performLandOnPlanet;
+- (void) performMining;
+- (void) performScriptedAI;
+- (void) performScriptedAttackAI;
+- (void) performStop;
+- (void) performTumble;
+
+- (void) broadcastDistressMessage;
+- (void) broadcastDistressMessageWithDumping:(BOOL)dumpCargo;
+
+- (void) requestDockingCoordinates;
+- (void) recallDockingInstructions;
+
+
 @end
