@@ -3,7 +3,7 @@
 OOMaterialConvenienceCreators.m
 
 
-Copyright (C) 2007-2011 Jens Ayton
+Copyright (C) 2007-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -25,9 +25,19 @@ SOFTWARE.
 
 */
 
+#ifndef USE_NEW_SHADER_SYNTHESIZER
+#define USE_NEW_SHADER_SYNTHESIZER	0
+#endif
+
+
 #import "OOMaterialConvenienceCreators.h"
 #import "OOMaterialSpecifier.h"
 
+#if USE_NEW_SHADER_SYNTHESIZER
+#import "OODefaultShaderSynthesizer.h"
+#import "ResourceManager.h"
+#endif
+
 #import "OOOpenGLExtensionManager.h"
 #import "OOShaderMaterial.h"
 #import "OOSingleTextureMaterial.h"
@@ -39,12 +49,13 @@ SOFTWARE.
 #import "OODebugFlags.h"
 
 
+#if !USE_NEW_SHADER_SYNTHESIZER
 typedef struct
 {
 	NSDictionary			*inConfig;
 	NSMutableDictionary		*outConfig;
-	OOUInteger				texturesUsed;
-	OOUInteger				maxTextures;
+	NSUInteger				texturesUsed;
+	NSUInteger				maxTextures;
 	
 	NSMutableDictionary		*macros;
 	NSMutableArray			*textures;
@@ -71,9 +82,13 @@ static void SynthEmissionAndIllumination(OOMaterialSynthContext *context);
 static void SynthNormalMap(OOMaterialSynthContext *context);
 static void SynthSpecular(OOMaterialSynthContext *context);
 
+#endif
+
 
 @implementation OOMaterial (OOConvenienceCreators)
 
+#if !USE_NEW_SHADER_SYNTHESIZER
+
 + (NSDictionary *)synthesizeMaterialDictionaryWithName:(NSString *)name
 										 configuration:(NSDictionary *)configuration
 												macros:(NSDictionary *)macros
@@ -107,7 +122,7 @@ static void SynthSpecular(OOMaterialSynthContext *context);
 	SynthNormalMap(&context);
 	SynthSpecular(&context);
 	
-	if ([UNIVERSE shaderEffectsLevel] == SHADERS_FULL)
+	if ([UNIVERSE detailLevel] >= DETAIL_LEVEL_SHADERS)
 	{
 		//	Add uniforms required for hull heat glow.
 		[context.uniforms setObject:@"hullHeatLevel" forKey:@"uHullHeatLevel"];
@@ -147,7 +162,9 @@ static void SynthSpecular(OOMaterialSynthContext *context);
 	if (cacheKey != nil)
 	{
 		cache = [OOCacheManager sharedCache];
-		cacheKey = [NSString stringWithFormat:@"%@/%@", cacheKey, name];
+		// configuration must be in cache key, as otherwise changes in
+		// non-diffuse map can end up miscached
+		cacheKey = [NSString stringWithFormat:@"%@/%@/%@", cacheKey, name, configuration];
 		synthesizedConfig = [cache objectForKey:cacheKey inCache:@"synthesized shader materials"];
 	}
 	
@@ -177,13 +194,81 @@ static void SynthSpecular(OOMaterialSynthContext *context);
 	return result;
 }
 
+#else
 
-+ (id)materialWithName:(NSString *)name
-			  cacheKey:(NSString *)cacheKey
-		 configuration:(NSDictionary *)configuration
-				macros:(NSDictionary *)macros
-		 bindingTarget:(id<OOWeakReferenceSupport>)object
-	   forSmoothedMesh:(BOOL)smooth
+#ifndef NDEBUG
+static BOOL sDumpShaderSource = NO;
+
++ (void) initialize
+{
+	sDumpShaderSource = [[NSUserDefaults standardUserDefaults] boolForKey:@"dump-synthesized-shaders"];
+}
+#endif
+
+
++ (OOMaterial *) defaultShaderMaterialWithName:(NSString *)name
+									  cacheKey:(NSString *)cacheKey
+								 configuration:(NSDictionary *)configuration
+										macros:(NSDictionary *)macros
+								 bindingTarget:(id<OOWeakReferenceSupport>)target
+{
+	NSString		*vertexShader = nil;
+	NSString		*fragmentShader = nil;
+	NSArray			*textureSpecs = nil;
+	NSDictionary	*uniformSpecs = nil;
+	
+	if (!OOSynthesizeMaterialShader(configuration, name, cacheKey /* FIXME: entity name for error reporting */, &vertexShader, &fragmentShader, &textureSpecs, &uniformSpecs))
+	{
+		return nil;
+	}
+	
+	NSDictionary	*synthesizedConfig = [NSDictionary dictionaryWithObjectsAndKeys:
+										  [NSNumber numberWithBool:YES], kOOIsSynthesizedMaterialConfigurationKey,
+										  textureSpecs, kOOTexturesKey,
+										  uniformSpecs, kOOUniformsKey,
+										  vertexShader, kOOVertexShaderSourceKey,
+										  fragmentShader, kOOFragmentShaderSourceKey,
+										  nil];
+	
+#ifndef NDEBUG
+	if (sDumpShaderSource)
+	{
+		NSString *dumpPath = [NSString stringWithFormat:@"Synthesized Materials/%@/%@", cacheKey, name];
+		
+		[ResourceManager writeDiagnosticString:vertexShader toFileNamed:[dumpPath stringByAppendingPathExtension:@"vertex"]];
+		[ResourceManager writeDiagnosticString:fragmentShader toFileNamed:[dumpPath stringByAppendingPathExtension:@"fragment"]];
+		
+		// Hide internal keys in the synthesized config before writing it.
+		NSMutableDictionary *humanFriendlyConfig = [[synthesizedConfig mutableCopy] autorelease];
+		[humanFriendlyConfig removeObjectForKey:kOOVertexShaderSourceKey];
+		[humanFriendlyConfig removeObjectForKey:kOOFragmentShaderSourceKey];
+		[humanFriendlyConfig removeObjectForKey:kOOIsSynthesizedMaterialConfigurationKey];
+		[humanFriendlyConfig setObject:[NSString stringWithFormat:@"%@.vertex", name] forKey:kOOVertexShaderNameKey];
+		[humanFriendlyConfig setObject:[NSString stringWithFormat:@"%@.fragment", name] forKey:kOOFragmentShaderNameKey];
+		
+		[ResourceManager writeDiagnosticPList:humanFriendlyConfig toFileNamed:[dumpPath stringByAppendingPathExtension:@"plist"]];
+		
+		[ResourceManager writeDiagnosticPList:configuration toFileNamed:[[dumpPath stringByAppendingString:@"-original"] stringByAppendingPathExtension:@"plist"]];
+	}
+#endif
+	
+	return [self materialWithName:name
+						 cacheKey:cacheKey
+					configuration:synthesizedConfig
+						   macros:nil
+					bindingTarget:target
+				  forSmoothedMesh:YES];
+}
+
+#endif
+
+
++ (OOMaterial *) materialWithName:(NSString *)name
+						 cacheKey:(NSString *)cacheKey
+					configuration:(NSDictionary *)configuration
+						   macros:(NSDictionary *)macros
+					bindingTarget:(id<OOWeakReferenceSupport>)object
+				  forSmoothedMesh:(BOOL)smooth	// Internally, this flg really means "force use of shaders".
 {
 	id result = nil;
 	
@@ -203,9 +288,10 @@ static void SynthSpecular(OOMaterialSynthContext *context);
 		if (result == nil &&
 				(smooth ||
 				 gDebugFlags & DEBUG_NO_SHADER_FALLBACK ||
-				 [UNIVERSE shaderEffectsLevel] == SHADERS_FULL ||
-				 [configuration oo_specularMapSpecifier] != nil ||
+				 [UNIVERSE detailLevel] >= DETAIL_LEVEL_SHADERS ||
+				 [configuration oo_combinedSpecularMapSpecifier] != nil ||
 				 [configuration oo_normalMapSpecifier] != nil ||
+				 [configuration oo_parallaxMapSpecifier] != nil ||
 				 [configuration oo_normalAndParallaxMapSpecifier] != nil ||
 				 [configuration oo_emissionMapSpecifier] != nil ||
 				 [configuration oo_illuminationMapSpecifier] != nil ||
@@ -222,7 +308,7 @@ static void SynthSpecular(OOMaterialSynthContext *context);
 #endif
 	
 #if OO_MULTITEXTURE
-	if (result == nil && ![UNIVERSE reducedDetail])
+	if (result == nil /*&& ![UNIVERSE reducedDetail]*/)
 	{
 		if ([configuration oo_emissionMapSpecifier] != nil ||
 			[configuration oo_illuminationMapSpecifier] ||
@@ -254,13 +340,13 @@ static void SynthSpecular(OOMaterialSynthContext *context);
 }
 
 
-+ (id)materialWithName:(NSString *)name
-			  cacheKey:(NSString *)cacheKey
-	materialDictionary:(NSDictionary *)materialDict
-	 shadersDictionary:(NSDictionary *)shadersDict
-				macros:(NSDictionary *)macros
-		 bindingTarget:(id<OOWeakReferenceSupport>)object
-	   forSmoothedMesh:(BOOL)smooth
++ (OOMaterial *) materialWithName:(NSString *)name
+						 cacheKey:(NSString *)cacheKey
+			   materialDictionary:(NSDictionary *)materialDict
+				shadersDictionary:(NSDictionary *)shadersDict
+						   macros:(NSDictionary *)macros
+					bindingTarget:(id<OOWeakReferenceSupport>)object
+				  forSmoothedMesh:(BOOL)smooth
 {
 	NSDictionary			*configuration = nil;
 	
@@ -298,6 +384,8 @@ static void SynthSpecular(OOMaterialSynthContext *context);
 @end
 
 
+#if !USE_NEW_SHADER_SYNTHESIZER
+
 static void SetUniform(NSMutableDictionary *uniforms, NSString *key, NSString *type, id value)
 {
 	[uniforms setObject:[NSDictionary dictionaryWithObjectsAndKeys:type, @"type", value, @"value", nil] forKey:key];
@@ -315,7 +403,7 @@ static void AddTexture(OOMaterialSynthContext *context, NSString *uniformName, N
 	NSCParameterAssert(context->texturesUsed < context->maxTextures);
 	
 	context->texturesUsed++;
-	SetUniform(context->uniforms, uniformName, @"texture", [NSNumber numberWithInt:[context->textures count]]);
+	SetUniform(context->uniforms, uniformName, @"texture", [NSNumber numberWithUnsignedInteger:[context->textures count]]);
 	[context->textures addObject:specifier];
 	if (nonShaderKey != nil)
 	{
@@ -461,7 +549,7 @@ static void SynthNormalMap(OOMaterialSynthContext *context)
 
 static void SynthSpecular(OOMaterialSynthContext *context)
 {
-	GLint shininess = [context->inConfig oo_shininess];
+	GLint shininess = [context->inConfig oo_specularExponent];
 	if (shininess <= 0)  return;
 	
 	NSDictionary *specularMapSpec = nil;
@@ -469,14 +557,14 @@ static void SynthSpecular(OOMaterialSynthContext *context)
 	
 	if (context->texturesUsed < context->maxTextures)
 	{
-		specularMapSpec = [context->inConfig oo_specularMapSpecifier];
+		specularMapSpec = [context->inConfig oo_combinedSpecularMapSpecifier];
 	}
 	
 	if (specularMapSpec != nil)  specularColor = [context->inConfig oo_specularModulateColor];
 	else  specularColor = [context->inConfig oo_specularColor];
 	if ([specularColor isBlack])  return;
 	
-	[context->outConfig setObject:[NSNumber numberWithUnsignedInt:shininess] forKey:kOOMaterialShininess];
+	[context->outConfig setObject:[NSNumber numberWithUnsignedInt:shininess] forKey:kOOMaterialSpecularExponentLegacyName];
 	
 	if (specularMapSpec != nil)
 	{
@@ -493,3 +581,5 @@ static void SynthSpecular(OOMaterialSynthContext *context)
 	}
 	[context->macros setObject:@"1" forKey:@"OOSTD_SPECULAR"];
 }
+
+#endif
