@@ -5,7 +5,7 @@
 	
 	
 	Oolite
-	Copyright (C) 2004-2011 Giles C Williams and contributors
+	Copyright (C) 2004-2013 Giles C Williams and contributors
 	
 	This program is free software; you can redistribute it and/or
 	modify it under the terms of the GNU General Public License
@@ -28,7 +28,7 @@
 #if NEW_PLANETS
 
 
-#define DEBUG_DUMP			(	1	&& !defined(NDEBUG))
+#define DEBUG_DUMP			(	0	&& !defined(NDEBUG))
 #define DEBUG_DUMP_RAW		(	1	&& DEBUG_DUMP)
 
 #define POLAR_CAPS			1
@@ -37,9 +37,9 @@
 
 #import "OOPlanetTextureGenerator.h"
 #import "OOCollectionExtractors.h"
+#import "OOColor.h"
 
 #ifndef TEXGEN_TEST_RIG
-#import "OOColor.h"
 #import "OOTexture.h"
 #import "Universe.h"
 #endif
@@ -99,9 +99,10 @@ enum
 @private
 	NSString				*_cacheKey;
 	RANROTSeed				_seed;
+	OOPlanetTextureGenerator	*_parent;
 }
 
-- (id) initWithCacheKey:(NSString *)cacheKey seed:(RANROTSeed)seed;
+- (id) initWithCacheKey:(NSString *)cacheKey seed:(RANROTSeed)seed andParent:(OOPlanetTextureGenerator *)parent;
 
 - (void) completeWithData:(void *)data width:(unsigned)width height:(unsigned)height;
 
@@ -113,9 +114,10 @@ static FloatRGB FloatRGBFromDictColor(NSDictionary *dictionary, NSString *key);
 static BOOL FillFBMBuffer(OOPlanetTextureGeneratorInfo *info);
 
 static float QFactor(float *accbuffer, int x, int y, unsigned width, float polar_y_value, float bias, float polar_y);
-static float GetQ(float *qbuffer, unsigned x, unsigned y, unsigned width, unsigned height, unsigned widthMask, unsigned heightMask);
+static float GetQ(float *qbuffer, int x, int y, unsigned width, unsigned height, unsigned widthMask, unsigned heightMask);
 
 static FloatRGB Blend(float fraction, FloatRGB a, FloatRGB b);
+static float BlendAlpha(float fraction, float a, float b);
 static void SetMixConstants(OOPlanetTextureGeneratorInfo *info, float temperatureFraction);
 static FloatRGBA CloudMix(OOPlanetTextureGeneratorInfo *info, float q, float nearPole);
 static FloatRGBA PlanetMix(OOPlanetTextureGeneratorInfo *info, float q, float nearPole);
@@ -137,7 +139,8 @@ enum
 	kPlanetScale4096x4096,
 	
 	kPlanetScaleReducedDetail	= kPlanetScale512x512,
-	kPlanetScaleFullDetail		= kPlanetScale1024x1024
+	kPlanetScaleFullDetail		= kPlanetScale1024x1024,
+	kPlanetScaleExtraDetail		= kPlanetScale2048x2048
 };
 
 
@@ -145,35 +148,43 @@ enum
 
 - (id) initWithPlanetInfo:(NSDictionary *)planetInfo
 {
-	if ((self = [super init]))
+	OOLog(@"texture.planet.generate",@"Initialising planetary generator");
+
+	// AllowCubeMap not used yet but might be in future
+	if ((self = [super initWithPath:[NSString stringWithFormat:@"OOPlanetTexture@%p", self] options:kOOTextureAllowCubeMap]))
 	{
+		OOLog(@"texture.planet.generate",@"Extracting parameters for generator %@",self);
+
 		_info.landFraction = OOClamp_0_1_f([planetInfo oo_floatForKey:@"land_fraction" defaultValue:0.3]);
+		_info.polarFraction = OOClamp_0_1_f([planetInfo oo_floatForKey:@"polar_fraction" defaultValue:0.05]);
 		_info.landColor = FloatRGBFromDictColor(planetInfo, @"land_color");
 		_info.seaColor = FloatRGBFromDictColor(planetInfo, @"sea_color");
 		_info.paleLandColor = FloatRGBFromDictColor(planetInfo, @"polar_land_color");
 		_info.polarSeaColor = FloatRGBFromDictColor(planetInfo, @"polar_sea_color");
 		[[planetInfo objectForKey:@"noise_map_seed"] getValue:&_info.seed];
-		
 		if ([planetInfo objectForKey:@"cloud_alpha"])
 		{
+			OOLog(@"texture.planet.generate",@"Extracting atmosphere parameters");
 			// we have an atmosphere:
 			_info.cloudAlpha = [planetInfo oo_floatForKey:@"cloud_alpha" defaultValue:1.0f];
 			_info.cloudFraction = OOClamp_0_1_f([planetInfo oo_floatForKey:@"cloud_fraction" defaultValue:0.3]);
-			_info.airColor = FloatRGBFromDictColor(planetInfo, @"air_color");
 			_info.cloudColor = FloatRGBFromDictColor(planetInfo, @"cloud_color");
-			_info.paleAirColor = FloatRGBFromDictColor(planetInfo, @"polar_air_color");
 			_info.paleCloudColor = FloatRGBFromDictColor(planetInfo, @"polar_cloud_color");
 		}
 		
 #ifndef TEXGEN_TEST_RIG
-		if ([UNIVERSE reducedDetail])
+		if ([UNIVERSE detailLevel] < DETAIL_LEVEL_SHADERS || [planetInfo oo_boolForKey:@"isMiniature" defaultValue:NO])
 		{
 			_planetScale = kPlanetScaleReducedDetail;
 		}
-		else
+		else if ([UNIVERSE detailLevel] == DETAIL_LEVEL_SHADERS)
 		{
 			_planetScale = kPlanetScaleFullDetail;
 		}
+		else
+		{
+			_planetScale = kPlanetScaleExtraDetail;
+		}
 #else
 		_planetScale = kPlanetScale4096x4096;
 #endif
@@ -265,9 +276,10 @@ enum
 		}
 		return NO;
 	}
+
+	OOLog(@"texture.planet.generate",@"Generator %@ has atmosphere %@",diffuseGen,*atmosphere);
 	
 	*texture = [OOTexture textureWithGenerator:diffuseGen];
-	
 	return *texture != nil;
 }
 
@@ -327,13 +339,13 @@ enum
 {
 	if (_atmoGenerator == nil)
 	{
-		_atmoGenerator = [[OOPlanetAtmosphereGenerator alloc] initWithCacheKey:[self cacheKeyForType:@"atmo"] seed:_info.seed];
+		_atmoGenerator = [[OOPlanetAtmosphereGenerator alloc] initWithCacheKey:[self cacheKeyForType:@"atmo"] seed:_info.seed andParent:self];
 	}
 	return _atmoGenerator;
 }
 
 
-- (BOOL)getResult:(void **)outData
+- (BOOL)getResult:(OOPixMap *)outData
 		   format:(OOTextureDataFormat *)outFormat
 			width:(uint32_t *)outWidth
 		   height:(uint32_t *)outHeight
@@ -342,18 +354,18 @@ enum
 	if (![self isReady])
 	{
 		waiting = true;
-		OOLog(@"planetTex.temp", @"%s generator %@", "Waiting for", self);
+		OOLog(@"texture.planet.generate.wait", @"%s generator %@", "Waiting for", self);
 	}
 	
-	BOOL result = [super getResult:outData format:outFormat width:outWidth height:outHeight];
+	BOOL result = [super getResult:outData format:outFormat originalWidth:outWidth originalHeight:outHeight];
 	
 	if (waiting)
 	{
-		OOLog(@"planetTex.temp", @"%s generator %@", result ? "Dequeued" : "Failed to dequeue", self);
+		OOLog(@"texture.planet.generate.dequeue", @"%s generator %@", result ? "Dequeued" : "Failed to dequeue", self);
 	}
 	else
 	{
-		OOLog(@"planetTex.temp", @"%s generator %@ without waiting.", result ? "Dequeued" : "Failed to dequeue", self);
+		OOLog(@"texture.planet.generate.dequeue", @"%s generator %@ without waiting.", result ? "Dequeued" : "Failed to dequeue", self);
 	}
 	
 	return result;
@@ -362,7 +374,7 @@ enum
 
 - (void) loadTexture
 {
-	OOLog(@"planetTex.temp", @"Started generator %@", self);
+	OOLog(@"texture.planet.generate.begin", @"Started generator %@", self);
 	
 	BOOL success = NO;
 	BOOL generateNormalMap = (_nMapGenerator != nil);
@@ -373,26 +385,26 @@ enum
 	uint8_t		*aBuffer = NULL, *apx = NULL;
 	float		*randomBuffer = NULL;
 	
-	height = _info.height = 1 << (_planetScale + kPlanetScaleOffset);
-	width = _info.width = height * kPlanetAspectRatio;
+	_height = _info.height = 1 << (_planetScale + kPlanetScaleOffset);
+	_width = _info.width = _height * kPlanetAspectRatio;
 	
 #define FAIL_IF(cond)  do { if (EXPECT_NOT(cond))  goto END; } while (0)
 #define FAIL_IF_NULL(x)  FAIL_IF((x) == NULL)
 	
-	buffer = malloc(4 * width * height);
+	buffer = malloc(4 * _width * _height);
 	FAIL_IF_NULL(buffer);
 	px = buffer;
 	
 	if (generateNormalMap)
 	{
-		nBuffer = malloc(4 * width * height);
+		nBuffer = malloc(4 * _width * _height);
 		FAIL_IF_NULL(nBuffer);
 		npx = nBuffer;
 	}
 	
 	if (generateAtmosphere)
 	{
-		aBuffer = malloc(4 * width * height);
+		aBuffer = malloc(4 * _width * _height);
 		FAIL_IF_NULL(aBuffer);
 		apx = aBuffer;
 	}
@@ -402,78 +414,68 @@ enum
 	[self dumpNoiseBuffer:_info.fbmBuffer];
 #endif
 	
-	//TODO: sort out CloudMix
 	float paleClouds = (_info.cloudFraction * _info.fbmBuffer[0] < 1.0f - _info.cloudFraction) ? 0.0f : 1.0f;
 	float poleValue = (_info.landFraction > 0.5f) ? 0.5f * _info.landFraction : 0.0f;
 	float seaBias = _info.landFraction - 1.0f;
 	
-	/*	The system key 'polar_sea_colour' was used as 'paleSeaColour'.
-		The generated texture had presumably iceberg covered shallows.
-		paleSeaColour is now a pale blend of sea and land colours, giving
-		a softer transition colour for the shallows - those have also been
-		widened from 1.73 to smooth out the coast / deep sea boundary.
-		-- Kaks
-	*/
-	_info.paleSeaColor = Blend(0.45f, _info.polarSeaColor, Blend(0.7f, _info.seaColor, _info.landColor));
+	_info.paleSeaColor = Blend(0.35f, _info.polarSeaColor, Blend(0.7f, _info.seaColor, _info.landColor));
 	float normalScale = 1 << _planetScale;
 	if (!generateNormalMap)  normalScale *= 3.0f;
 	
-	// Deep sea colour: slightly darkened so the sea isn't just a uniform colour.
-	_info.deepSeaColor = Blend(0.80f, _info.seaColor, (FloatRGB){ 0, 0, 0 });
+	// Deep sea colour: sea darker past the continental shelf.
+	_info.deepSeaColor = Blend(0.85f, _info.seaColor, (FloatRGB){ 0, 0, 0 });
 	
-	unsigned x, y;
+	int x, y;
 	FloatRGBA color;
-	FloatRGBA cloudColor = (FloatRGBA){_info.cloudColor.r, _info.cloudColor.g, _info.cloudColor.b, 1.0f};
 	Vector norm;
 	float q, yN, yS, yW, yE, nearPole;
 	GLfloat shade;
-	float rHeight = 1.0f / height;
-	float fy, fHeight = height;
+	float rHeight = 1.0f / _height;
+	float fy, fHeight = _height;
 	// The second parameter is the temperature fraction. Most favourable: 1.0f,  little ice. Most unfavourable: 0.0f, frozen planet. TODO: make it dependent on ranrot / planetinfo key...
-	SetMixConstants(&_info, 0.95f);	// no need to recalculate them inside each loop!
+	SetMixConstants(&_info, 1.0f-_info.polarFraction);	// no need to recalculate them inside each loop!
 	
 	// first pass, calculate q.
-	_info.qBuffer = malloc(width * height * sizeof (float));
+	_info.qBuffer = malloc(_width * _height * sizeof (float));
 	FAIL_IF_NULL(_info.qBuffer);
 	
-	for (y = 0, fy = 0.0f; y < height; y++, fy++)
+	for (y = (int)_height - 1, fy = (float)y; y >=0; y--, fy--)
 	{
 		nearPole = (2.0f * fy - fHeight) * rHeight;
 		nearPole *= nearPole;
 		
-		for (x = 0; x < width; x++)
+		for (x = (int)_width - 1; x >=0; x--)
 		{
-			_info.qBuffer[y * width + x] = QFactor(_info.fbmBuffer, x, y, width, poleValue, seaBias, nearPole);
+			_info.qBuffer[y * _width + x] = QFactor(_info.fbmBuffer, x, y, _width, poleValue, seaBias, nearPole);
 		}
 	}
 	
 	// second pass, use q.
-	float cloudAlpha = _info.cloudAlpha;
 	float cloudFraction = _info.cloudFraction;
-	unsigned widthMask = width - 1;
-	unsigned heightMask = height - 1;
+	unsigned widthMask = _width - 1;
+	unsigned heightMask = _height - 1;
 	
-	for (y = 0, fy = 0.0f; y < height; y++, fy++)
+	for (y = (int)_height - 1, fy = (float)y; y >= 0; y--, fy--)
 	{
 		nearPole = (2.0f * fy - fHeight) * rHeight;
 		nearPole *= nearPole;
 		
-		for (x = 0; x < width; x++)
+		for (x = (int)_width - 1; x >= 0; x--)
 		{
-			q = _info.qBuffer[y * width + x];	// no need to use GetQ, x and y are always within bounds.
-			yN = GetQ(_info.qBuffer, x, y - 1, width, height, widthMask, heightMask);	// recalculates x & y if they go out of bounds.
-			yS = GetQ(_info.qBuffer, x, y + 1, width, height, widthMask, heightMask);
-			yW = GetQ(_info.qBuffer, x - 1, y, width, height, widthMask, heightMask);
-			yE = GetQ(_info.qBuffer, x + 1, y, width, height, widthMask, heightMask);
+			q = _info.qBuffer[y * _width + x];	// no need to use GetQ, x and y are always within bounds.
+			yN = GetQ(_info.qBuffer, x, y - 1, _width, _height, widthMask, heightMask);	// recalculates x & y if they go out of bounds.
+			yS = GetQ(_info.qBuffer, x, y + 1, _width, _height, widthMask, heightMask);
+			yW = GetQ(_info.qBuffer, x - 1, y, _width, _height, widthMask, heightMask);
+			yE = GetQ(_info.qBuffer, x + 1, y, _width, _height, widthMask, heightMask);
 			
 			color = PlanetMix(&_info, q, nearPole);
 			
-			norm = vector_normal(make_vector(normalScale * (yW - yE), normalScale * (yS - yN), 1.0f));
+			norm = vector_normal(make_vector(normalScale * (yE - yW), normalScale * (yN - yS), 1.0f));
 			if (generateNormalMap)
 			{
 				shade = 1.0f;
 				
-				// Flatten in the sea.
+				// Flatten the sea.
 				norm = OOVectorInterpolate(norm, kBasisZVector, color.a);
 				
 				// Put norm in normal map, scaled from [-1..1] to [0..255].
@@ -504,28 +506,18 @@ enum
 			
 			if (generateAtmosphere)
 			{
-				//TODO: sort out CloudMix
-				if (NO) 
-				{
-					q = QFactor(_info.fbmBuffer, x, y, width, paleClouds, cloudFraction, nearPole);
-					color = CloudMix(&_info, q, nearPole);
-				}
-				else
-				{
-					q = _info.fbmBuffer[y * width + x];
-					q *= q;
-					color = cloudColor;
-				}
+				q = QFactor(_info.fbmBuffer, x, y, _width, paleClouds, cloudFraction, nearPole);
+				color = CloudMix(&_info, q, nearPole);
 				*apx++ = 255.0f * color.r;
 				*apx++ = 255.0f * color.g;
 				*apx++ = 255.0f * color.b;
-				*apx++ = 255.0f * cloudAlpha * q;
+				*apx++ = 255.0f * color.a * _info.cloudAlpha;
 			}
 		}
 	}
 	
 	success = YES;
-	format = kOOTextureDataRGBA;
+	_format = kOOTextureDataRGBA;
 	
 END:
 	FREE(_info.fbmBuffer);
@@ -533,9 +525,9 @@ END:
 	FREE(randomBuffer);
 	if (success)
 	{
-		data = buffer;
-		if (generateNormalMap) [_nMapGenerator completeWithData:nBuffer width:width height:height];
-		if (generateAtmosphere) [_atmoGenerator completeWithData:aBuffer width:width height:height];
+		_data = buffer;
+		if (generateNormalMap) [_nMapGenerator completeWithData:nBuffer width:_width height:_height];
+		if (generateAtmosphere) [_atmoGenerator completeWithData:aBuffer width:_width height:_height];
 	}
 	else
 	{
@@ -546,7 +538,7 @@ END:
 	DESTROY(_nMapGenerator);
 	DESTROY(_atmoGenerator);
 	
-	OOLog(@"planetTex.temp", @"Completed generator %@ %@successfully", self, success ? @"" : @"un");
+	OOLog(@"texture.planet.generate.complete", @"Completed generator %@ %@successfully", self, success ? @"" : @"un");
 	
 #if DEBUG_DUMP
 	if (success)
@@ -557,9 +549,9 @@ END:
 		[[UNIVERSE gameView] dumpRGBAToRGBFileNamed:diffuseName
 								   andGrayFileNamed:lightsName
 											  bytes:buffer
-											  width:width
-											 height:height
-										   rowBytes:width * 4];
+											  width:_width
+											 height:_height
+										   rowBytes:_width * 4];
 	}
 #endif
 }
@@ -571,21 +563,21 @@ END:
 {
 	NSString *noiseName = [NSString stringWithFormat:@"planet-%u-%u-noise-new", _info.seed.high, _info.seed.low];
 	
-	uint8_t *noisePx = malloc(width * height);
+	uint8_t *noisePx = malloc(_width * _height);
 	unsigned x, y;
-	for (y = 0; y < height; y++)
+	for (y = 0; y < _height; y++)
 	{
-		for (x = 0; x < width; x++)
+		for (x = 0; x < _width; x++)
 		{
-			noisePx[y * width + x] = 255.0f * noise[y * width + x];
+			noisePx[y * _width + x] = 255.0f * noise[y * _width + x];
 		}
 	}
 	
 	[[UNIVERSE gameView] dumpGrayToFileNamed:noiseName
 									   bytes:noisePx
-									   width:width
-									  height:height
-									rowBytes:width];
+									   width:_width
+									  height:_height
+									rowBytes:_width];
 	FREE(noisePx);
 }
 
@@ -612,81 +604,67 @@ static FloatRGB Blend(float fraction, FloatRGB a, FloatRGB b)
 }
 
 
+static float BlendAlpha(float fraction, float a, float b)
+{
+	return Lerp(b, a, fraction);
+}
+
+
 static FloatRGBA CloudMix(OOPlanetTextureGeneratorInfo *info, float q, float nearPole)
 {
+//#define AIR_ALPHA				(0.15f)
+//#define CLOUD_ALPHA				(1.0f)
+// CIM: make distinction between cloud and not-cloud bigger
 #define AIR_ALPHA				(0.05f)
-#define CLOUD_ALPHA				(0.50f)
-#define POLAR_AIR_ALPHA			(0.34f)
-#define POLAR_CLOUD_ALPHA		(0.75f)
+#define CLOUD_ALPHA				(2.0f)
 
-#define POLAR_BOUNDARY			(0.66f)
-#define RECIP_CLOUD_BOUNDARY	(200.0f)
-#define CLOUD_BOUNDARY			(1.0f / RECIP_CLOUD_BOUNDARY)
+#define POLAR_BOUNDARY			(0.33f)
+#define CLOUD_BOUNDARY			(0.5f)
+#define RECIP_CLOUD_BOUNDARY	(1.0f / CLOUD_BOUNDARY)
 
-	FloatRGB result = info->cloudColor;
-	float alpha = info->cloudAlpha;
-	float portion = 0.0f;
+	FloatRGB cloudColor = info->cloudColor;
+	float alpha = info->cloudAlpha, portion = 0.0f;
 	
-	q -= CLOUD_BOUNDARY;
+	q -= CLOUD_BOUNDARY * 0.5f;
 	
 	if (nearPole > POLAR_BOUNDARY)
 	{
-		portion = nearPole > POLAR_BOUNDARY + 0.06f ? 1.0f : 0.4f + (nearPole - POLAR_BOUNDARY) * 10.0f;	// x * 10 == ((x / 0.06) * 0.6
-		
-		if (q <= 0.0f)
+		portion = nearPole > POLAR_BOUNDARY + 0.2f ? 1.0f : (nearPole - POLAR_BOUNDARY) * 5.0f;
+		cloudColor = Blend(portion, info->paleCloudColor, cloudColor);
+		 
+		portion = nearPole > POLAR_BOUNDARY + 0.625f ? 1.0f : (nearPole - POLAR_BOUNDARY) * 1.6f;
+	}
+	
+	if (q <= 0.0f)
+	{
+		if (q >= -CLOUD_BOUNDARY)
 		{
-			alpha *= Lerp(POLAR_CLOUD_ALPHA, CLOUD_ALPHA, portion);
-			if (q >= -CLOUD_BOUNDARY)
-			{
-				portion = -q * 0.5f * RECIP_CLOUD_BOUNDARY + 0.5f;
-				result = Blend(portion, info->paleCloudColor, info->paleAirColor);
-			}
-			else
-			{
-				result = info->paleCloudColor;
-			}
+			alpha *= BlendAlpha(-q * 0.5f * RECIP_CLOUD_BOUNDARY + 0.5f, CLOUD_ALPHA, AIR_ALPHA);
 		}
-		else 
+		else
 		{
-			alpha *= portion * POLAR_AIR_ALPHA;
-			if (q < CLOUD_BOUNDARY)
-			{
-				result = Blend(q * RECIP_CLOUD_BOUNDARY, info->paleAirColor, info->paleCloudColor);
-			}
-			else
-			{
-				result = info->paleAirColor;
-			}
+			alpha *= CLOUD_ALPHA;
 		}
 	}
 	else
 	{
-		if (q <= 0.0f)
+		if (q < CLOUD_BOUNDARY)
 		{
-			if (q >= -CLOUD_BOUNDARY){
-				portion = -q * 0.5f * RECIP_CLOUD_BOUNDARY + 0.5f;
-				alpha *=  portion * CLOUD_ALPHA;
-				result = Blend(portion, info->cloudColor, info->airColor);
-			}
-			else
-			{
-				result = info->cloudColor;
-				alpha *= CLOUD_ALPHA;
-			}
+			alpha *= BlendAlpha( q * 0.5f * RECIP_CLOUD_BOUNDARY + 0.5f,  AIR_ALPHA,CLOUD_ALPHA);
 		}
-		else if (q < CLOUD_BOUNDARY)
+		else
 		{
 			alpha *= AIR_ALPHA;
-			result = Blend(q * RECIP_CLOUD_BOUNDARY, info->airColor, info->cloudColor);
-		}
-		else if (q >= CLOUD_BOUNDARY)
-		{
-			alpha *= AIR_ALPHA;
-			result = info->airColor;
 		}
 	}
+	// magic numbers! at the poles we have fairly thin air.
+	alpha *= BlendAlpha(portion, 0.6f, 1.0f);
+	if (alpha > 1.0)
+	{
+		alpha = 1.0;
+	}
 
-	return (FloatRGBA){ result.r, result.g, result.b, alpha };
+	return (FloatRGBA){ cloudColor.r, cloudColor.g, cloudColor.b, alpha };
 }
 
 
@@ -696,10 +674,13 @@ static FloatRGBA PlanetMix(OOPlanetTextureGeneratorInfo *info, float q, float ne
 #define COASTLINE_PORTION			(1.0f / RECIP_COASTLINE_PORTION)
 #define SHALLOWS					(2.0f * COASTLINE_PORTION)	// increased shallows area.
 #define RECIP_SHALLOWS				(1.0f / SHALLOWS)
+// N.B.: DEEPS can't be more than RECIP_COASTLINE_PORTION * COASTLINE_PORTION!
+#define DEEPS						(40.0f * COASTLINE_PORTION) 
+#define RECIP_DEEPS					(1.0f / DEEPS)
 	
 	const FloatRGB white = { 1.0f, 1.0f, 1.0f };
 	FloatRGB diffuse;
-	// windows specular 'fix': 0 was showing pitch black continents when on the dark side, 0.01 shows the same shading as on macs.
+	// windows specular 'fix': 0 was showing pitch black continents when on the dark side, 0.01 shows the same shading as on Macs.
 	// TODO: a less hack-like fix.
 	float specular = 0.01f;
 	
@@ -715,7 +696,8 @@ static FloatRGBA PlanetMix(OOPlanetTextureGeneratorInfo *info, float q, float ne
 		else
 		{
 			// Open sea.
-			diffuse = Blend(-q, info->deepSeaColor, info->seaColor);
+			if (q > -DEEPS)  diffuse = Blend(-q * RECIP_DEEPS, info->deepSeaColor, info->seaColor);
+			else  diffuse = info->deepSeaColor;
 			specular = Lerp(1.0f, 0.85f, -q);
 		}
 	}
@@ -728,7 +710,7 @@ static FloatRGBA PlanetMix(OOPlanetTextureGeneratorInfo *info, float q, float ne
 	}
 	else if (q > 1.0f)
 	{
-		// High up - snow-capped peaks. -- Question: does q ever get higher than 1.0 ? --Kaks 20091228
+		// High up - snow-capped peaks. With overrides q can range between -2 to +2.
 		diffuse = white;
 	}
 	else if (q > info->mix_hi)
@@ -747,7 +729,8 @@ static FloatRGBA PlanetMix(OOPlanetTextureGeneratorInfo *info, float q, float ne
 	if (q > phi)	 // (nearPole + q) / 2 > pole
 	{
 		// thinner to thicker ice.
-		specular = q > phi + 0.02f ? 1.0f : 0.4f + (q - phi) * 30.0f;	// (q - phi) * 30 == ((q-phi) / 0.02) * 0.6
+		specular = q > phi + 0.02f ? 1.0f : 0.2f + (q - phi) * 40.0f;	// (q - phi) * 40 == ((q-phi) / 0.02) * 0.8
+		//diffuse = info->polarSeaColor;
 		diffuse = Blend(specular, info->polarSeaColor, diffuse);
 		specular = specular * 0.5f; // softer contours under ice, but still contours.
 	}
@@ -772,7 +755,7 @@ OOINLINE float Hermite(float q)
 }
 
 
-#if __BIG_ENDIAN_
+#if __BIG_ENDIAN__
 #define iman_ 1
 #else
 #define iman_ 0
@@ -780,10 +763,10 @@ OOINLINE float Hermite(float q)
 
  // (same behaviour as, but faster than, FLOAT->INT)
  //Works OK for -32728 to 32727.99999236688
-OOINLINE long fast_floor(double val)
+OOINLINE int32_t fast_floor(double val)
 {
    val += 68719476736.0 * 1.5;
-   return (((long*)&val)[iman_] >> 16);
+   return (((int32_t*)&val)[iman_] >> 16);
 }
 
 
@@ -881,9 +864,9 @@ static float SampleNoise3D(OOPlanetTextureGeneratorInfo *info, Vector p)
 	uint16_t	*permutations = info->permutations;
 	
 	// Split coordinates into integer and fractional parts.
-	float		fx = floorf(p.x);
-	float		fy = floorf(p.y);
-	float		fz = floorf(p.z);
+	float		fx = floor(p.x);
+	float		fy = floor(p.y);
+	float		fz = floor(p.z);
 	int			X = fx;
 	int			Y = fy;
 	int			Z = fz;
@@ -1000,15 +983,15 @@ static BOOL GenerateFBMNoise(OOPlanetTextureGeneratorInfo *info)
 	
 	for (y = 0, lat = -M_PI_2; y < height; y++, lat += dlat)
 	{
-		float las = sinf(lat);
-		float lac = cosf(lat);
+		float las = sin(lat);
+		float lac = cos(lat);
 		
 		for (x = 0, lon = -M_PI; x < width; x++, lon += dlon)
 		{
 			// FIXME: in real life, we really don't want sin and cos per pixel.
 			// Convert spherical coordinates to vector.
-			float los = sinf(lon);
-			float loc = cosf(lon);
+			float los = sin(lon);
+			float loc = cos(lon);
 			
 			Vector p =
 			{
@@ -1162,7 +1145,7 @@ static float QFactor(float *accbuffer, int x, int y, unsigned width, float polar
 }
 
 
-static float GetQ(float *qbuffer, unsigned x, unsigned y, unsigned width, unsigned height, unsigned widthMask, unsigned heightMask)
+static float GetQ(float *qbuffer, int x, int y, unsigned width, unsigned height, unsigned widthMask, unsigned heightMask)
 {
 	// Correct Y wrapping mode, unoptimised.
 	//if (y < 0) { y = -y - 1; x += width / 2; }
@@ -1192,7 +1175,8 @@ static void SetMixConstants(OOPlanetTextureGeneratorInfo *info, float temperatur
 
 - (id) initWithCacheKey:(NSString *)cacheKey seed:(RANROTSeed)seed
 {
-	if ((self = [super init]))
+	// AllowCubeMap not used yet but might be in future
+	if ((self = [super initWithPath:[NSString stringWithFormat:@"OOPlanetNormalTexture@%p", self] options:kOOTextureAllowCubeMap]))
 	{
 		_cacheKey = [cacheKey copy];
 		_seed = seed;
@@ -1240,10 +1224,10 @@ static void SetMixConstants(OOPlanetTextureGeneratorInfo *info, float temperatur
 
 - (void) completeWithData:(void *)data_ width:(unsigned)width_ height:(unsigned)height_
 {
-	data = data_;
-	width = width_;
-	height = height_;
-	format = kOOTextureDataRGBA;
+	_data = data_;
+	_width = width_;
+	_height = height_;
+	_format = kOOTextureDataRGBA;
 	
 	// Enqueue so superclass can apply texture options and so forth.
 	[super enqueue];
@@ -1254,10 +1238,10 @@ static void SetMixConstants(OOPlanetTextureGeneratorInfo *info, float temperatur
 	
 	[[UNIVERSE gameView] dumpRGBAToRGBFileNamed:normalName
 							   andGrayFileNamed:specularName
-										  bytes:data
-										  width:width
-										 height:height
-									   rowBytes:width * 4];
+										  bytes:_data
+										  width:_width
+										 height:_height
+									   rowBytes:_width * 4];
 #endif
 }
 
@@ -1266,12 +1250,15 @@ static void SetMixConstants(OOPlanetTextureGeneratorInfo *info, float temperatur
 
 @implementation OOPlanetAtmosphereGenerator
 
-- (id) initWithCacheKey:(NSString *)cacheKey seed:(RANROTSeed)seed
+- (id) initWithCacheKey:(NSString *)cacheKey seed:(RANROTSeed)seed andParent:(OOPlanetTextureGenerator *)parent
 {
-	if ((self = [super init]))
+	OOLog(@"texture.planet.generate",@"Initialising atmosphere generator %@",cacheKey);
+	// AllowCubeMap not used yet but might be in future
+	if ((self = [super initWithPath:[NSString stringWithFormat:@"OOPlanetAtmoTexture@%p", self] options:kOOTextureAllowCubeMap]))
 	{
 		_cacheKey = [cacheKey copy];
 		_seed = seed;
+		_parent = [parent retain];
 	}
 	return self;
 }
@@ -1280,7 +1267,7 @@ static void SetMixConstants(OOPlanetTextureGeneratorInfo *info, float temperatur
 - (void) dealloc
 {
 	DESTROY(_cacheKey);
-	
+	DESTROY(_parent);
 	[super dealloc];
 }
 
@@ -1309,12 +1296,40 @@ static void SetMixConstants(OOPlanetTextureGeneratorInfo *info, float temperatur
 }
 
 
+/* Because of the hack to avoid making texture loaders need to know
+ * about multiple textures, when we call the atmosphere generator on a
+ * planet with a from-file texture, things can occasionally go wrong
+ * because this isn't queued until after it's ready, but might be
+ * requested from the queue before then. So, when getResult is called,
+ * it waits for its parent task to complete if it's not itself ready -
+ * by the time the parent task is complete, this texture loader will
+ * have been enqueued, and [super getResult] will then work
+ * properly. - CIM 2014-04-05 */
+- (BOOL) getResult:(OOPixMap *)result
+			format:(OOTextureDataFormat *)outFormat
+	 originalWidth:(uint32_t *)outWidth
+	originalHeight:(uint32_t *)outHeight
+{
+	if (!_ready)
+	{
+		[[OOAsyncWorkManager sharedAsyncWorkManager] waitForTaskToComplete:_parent];
+	}
+
+	return [super getResult:result
+					 format:outFormat
+			  originalWidth:outWidth
+			 originalHeight:outHeight];
+}
+
+
 - (void) completeWithData:(void *)data_ width:(unsigned)width_ height:(unsigned)height_
 {
-	data = data_;
-	width = width_;
-	height = height_;
-	format = kOOTextureDataRGBA;
+	OOLog(@"texture.planet.generate",@"Completing atmosphere generator");
+
+	_data = data_;
+	_width = width_;
+	_height = height_;
+	_format = kOOTextureDataRGBA;
 	
 	// Enqueue so superclass can apply texture options and so forth.
 	[super enqueue];
@@ -1325,10 +1340,10 @@ static void SetMixConstants(OOPlanetTextureGeneratorInfo *info, float temperatur
 	
 	[[UNIVERSE gameView] dumpRGBAToRGBFileNamed:rgbName
 							   andGrayFileNamed:alphaName
-										  bytes:data
-										  width:width
-										 height:height
-									   rowBytes:width * 4];
+										  bytes:_data
+										  width:_width
+										 height:_height
+									   rowBytes:_width * 4];
 #endif
 }
 
