@@ -4,10 +4,10 @@ NSFileManagerOOExtensions.h
 
 This extends NSFileManager and adds some methods to insulate the
 main oolite code from the gory details of creating/chdiring to the
-commander save directory.
+commander save directory, as well as handling OXZ inspection
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -36,13 +36,27 @@ MA 02110-1301, USA.
 
 @interface NSFileManager (OOExtensions)
 
-- (NSArray*) commanderContentsOfPath:(NSString*) savePath;
-- (NSString*) defaultCommanderPath;
+- (NSArray *) commanderContentsOfPath:(NSString*)savePath;
+- (NSString *) defaultCommanderPath;
+
+/*
+	Wrappers for methods that were deprecated in Leopard but whose replacements
+	still aren't in GNUstep.
+*/
+- (NSArray *) oo_directoryContentsAtPath:(NSString *)path;
+- (BOOL) oo_createDirectoryAtPath:(NSString *)path attributes:(NSDictionary *)attributes;
+- (NSDictionary *) oo_fileAttributesAtPath:(NSString *)path traverseLink:(BOOL)yorn;
+- (NSDictionary *) oo_fileSystemAttributesAtPath:(NSString *)path;
+- (BOOL) oo_removeItemAtPath:(NSString *)path;
+- (BOOL) oo_moveItemAtPath:(NSString *)src toPath:(NSString *)dest;
 
 #if OOLITE_SDL
 - (BOOL) chdirToSnapshotPath;
 #endif
 
+// this also checks inside OXZ files; always returns NO for directories
+- (BOOL) oo_oxzFileExistsAtPath:(NSString *)path;
+
 @end
 
 
