@@ -2,7 +2,7 @@
 
 OOCollectionExtractors.m
 
-Copyright (C) 2007-2011 Jens Ayton and contributors
+Copyright (C) 2007-2013 Jens Ayton and contributors
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -40,117 +40,117 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 
 @implementation NSArray (OOExtractor)
 
-- (char) oo_charAtIndex:(OOUInteger)index defaultValue:(char)value
+- (char) oo_charAtIndex:(NSUInteger)index defaultValue:(char)value
 {
 	return OOCharFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (short) oo_shortAtIndex:(OOUInteger)index defaultValue:(short)value
+- (short) oo_shortAtIndex:(NSUInteger)index defaultValue:(short)value
 {
 	return OOShortFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (int) oo_intAtIndex:(OOUInteger)index defaultValue:(int)value
+- (int) oo_intAtIndex:(NSUInteger)index defaultValue:(int)value
 {
 	return OOIntFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (long) oo_longAtIndex:(OOUInteger)index defaultValue:(long)value
+- (long) oo_longAtIndex:(NSUInteger)index defaultValue:(long)value
 {
 	return OOLongFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (long long) oo_longLongAtIndex:(OOUInteger)index defaultValue:(long long)value
+- (long long) oo_longLongAtIndex:(NSUInteger)index defaultValue:(long long)value
 {
 	return OOLongLongFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (OOInteger) oo_integerAtIndex:(OOUInteger)index defaultValue:(OOInteger)value
+- (NSInteger) oo_integerAtIndex:(NSUInteger)index defaultValue:(NSInteger)value
 {
 	return OOIntegerFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (unsigned char) oo_unsignedCharAtIndex:(OOUInteger)index defaultValue:(unsigned char)value
+- (unsigned char) oo_unsignedCharAtIndex:(NSUInteger)index defaultValue:(unsigned char)value
 {
 	return OOUnsignedCharFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (unsigned short) oo_unsignedShortAtIndex:(OOUInteger)index defaultValue:(unsigned short)value
+- (unsigned short) oo_unsignedShortAtIndex:(NSUInteger)index defaultValue:(unsigned short)value
 {
 	return OOUnsignedShortFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (unsigned int) oo_unsignedIntAtIndex:(OOUInteger)index defaultValue:(unsigned int)value
+- (unsigned int) oo_unsignedIntAtIndex:(NSUInteger)index defaultValue:(unsigned int)value
 {
 	return OOUnsignedIntFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (unsigned long) oo_unsignedLongAtIndex:(OOUInteger)index defaultValue:(unsigned long)value
+- (unsigned long) oo_unsignedLongAtIndex:(NSUInteger)index defaultValue:(unsigned long)value
 {
 	return OOUnsignedLongFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (unsigned long long) oo_unsignedLongLongAtIndex:(OOUInteger)index defaultValue:(unsigned long long)value
+- (unsigned long long) oo_unsignedLongLongAtIndex:(NSUInteger)index defaultValue:(unsigned long long)value
 {
 	return OOUnsignedLongLongFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (OOUInteger) oo_unsignedIntegerAtIndex:(OOUInteger)index defaultValue:(OOUInteger)value
+- (NSUInteger) oo_unsignedIntegerAtIndex:(NSUInteger)index defaultValue:(NSUInteger)value
 {
 	return OOUIntegerFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (BOOL) oo_boolAtIndex:(OOUInteger)index defaultValue:(BOOL)value
+- (BOOL) oo_boolAtIndex:(NSUInteger)index defaultValue:(BOOL)value
 {
 	return OOBooleanFromObject([self oo_objectAtIndex:index], value);
 }
 
 
 #ifndef OOCOLLECTIONEXTRACTORS_SIMPLE
-- (BOOL) oo_fuzzyBooleanAtIndex:(OOUInteger)index defaultValue:(float)value
+- (BOOL) oo_fuzzyBooleanAtIndex:(NSUInteger)index defaultValue:(float)value
 {
 	return OOFuzzyBooleanFromObject([self oo_objectAtIndex:index], value);
 }
 #endif
 
 
-- (float) oo_floatAtIndex:(OOUInteger)index defaultValue:(float)value
+- (float) oo_floatAtIndex:(NSUInteger)index defaultValue:(float)value
 {
 	return OOFloatFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (double) oo_doubleAtIndex:(OOUInteger)index defaultValue:(double)value
+- (double) oo_doubleAtIndex:(NSUInteger)index defaultValue:(double)value
 {
 	return OODoubleFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (float) oo_nonNegativeFloatAtIndex:(OOUInteger)index defaultValue:(float)value
+- (float) oo_nonNegativeFloatAtIndex:(NSUInteger)index defaultValue:(float)value
 {
 	return OONonNegativeFloatFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (double) oo_nonNegativeDoubleAtIndex:(OOUInteger)index defaultValue:(double)value
+- (double) oo_nonNegativeDoubleAtIndex:(NSUInteger)index defaultValue:(double)value
 {
 	return OONonNegativeDoubleFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (id) oo_objectAtIndex:(OOUInteger)index defaultValue:(id)value
+- (id) oo_objectAtIndex:(NSUInteger)index defaultValue:(id)value
 {
 	id					objVal = [self oo_objectAtIndex:index];
 	id					result;
@@ -162,7 +162,7 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
-- (id) oo_objectOfClass:(Class)class atIndex:(OOUInteger)index defaultValue:(id)value
+- (id) oo_objectOfClass:(Class)class atIndex:(NSUInteger)index defaultValue:(id)value
 {
 	id					objVal = [self oo_objectAtIndex:index];
 	NSString			*result;
@@ -174,211 +174,211 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
-- (NSString *) oo_stringAtIndex:(OOUInteger)index defaultValue:(NSString *)value
+- (NSString *) oo_stringAtIndex:(NSUInteger)index defaultValue:(NSString *)value
 {
 	return StringForObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (NSArray *) oo_arrayAtIndex:(OOUInteger)index defaultValue:(NSArray *)value
+- (NSArray *) oo_arrayAtIndex:(NSUInteger)index defaultValue:(NSArray *)value
 {
 	return [self oo_objectOfClass:[NSArray class] atIndex:index defaultValue:value];
 }
 
 
-- (NSSet *) oo_setAtIndex:(OOUInteger)index defaultValue:(NSSet *)value
+- (NSSet *) oo_setAtIndex:(NSUInteger)index defaultValue:(NSSet *)value
 {
 	return SetForObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (NSDictionary *) oo_dictionaryAtIndex:(OOUInteger)index defaultValue:(NSDictionary *)value
+- (NSDictionary *) oo_dictionaryAtIndex:(NSUInteger)index defaultValue:(NSDictionary *)value
 {
 	return [self oo_objectOfClass:[NSDictionary class] atIndex:index defaultValue:value];
 }
 
 
-- (NSData *) oo_dataAtIndex:(OOUInteger)index defaultValue:(NSData *)value
+- (NSData *) oo_dataAtIndex:(NSUInteger)index defaultValue:(NSData *)value
 {
 	return [self oo_objectOfClass:[NSData class] atIndex:index defaultValue:value];
 }
 
 
 #ifndef OOCOLLECTIONEXTRACTORS_SIMPLE
-- (Vector) oo_vectorAtIndex:(OOUInteger)index defaultValue:(Vector)value
+- (Vector) oo_vectorAtIndex:(NSUInteger)index defaultValue:(Vector)value
 {
 	return OOVectorFromObject([self oo_objectAtIndex:index], value);
 }
 
 
-- (Quaternion) oo_quaternionAtIndex:(OOUInteger)index defaultValue:(Quaternion)value
+- (Quaternion) oo_quaternionAtIndex:(NSUInteger)index defaultValue:(Quaternion)value
 {
 	return OOQuaternionFromObject([self oo_objectAtIndex:index], value);
 }
 #endif
 
 
-- (char) oo_charAtIndex:(OOUInteger)index
+- (char) oo_charAtIndex:(NSUInteger)index
 {
 	return [self oo_charAtIndex:index defaultValue:0];
 }
 
 
-- (short) oo_shortAtIndex:(OOUInteger)index
+- (short) oo_shortAtIndex:(NSUInteger)index
 {
 	return [self  oo_shortAtIndex:index defaultValue:0];
 }
 
 
-- (int) oo_intAtIndex:(OOUInteger)index
+- (int) oo_intAtIndex:(NSUInteger)index
 {
 	return [self oo_intAtIndex:index defaultValue:0];
 }
 
 
-- (long) oo_longAtIndex:(OOUInteger)index
+- (long) oo_longAtIndex:(NSUInteger)index
 {
 	return [self oo_longAtIndex:index defaultValue:0];
 }
 
 
-- (long long) oo_longLongAtIndex:(OOUInteger)index
+- (long long) oo_longLongAtIndex:(NSUInteger)index
 {
 	return [self oo_longLongAtIndex:index defaultValue:0];
 }
 
 
-- (OOInteger) oo_integerAtIndex:(OOUInteger)index
+- (NSInteger) oo_integerAtIndex:(NSUInteger)index
 {
 	return [self oo_integerAtIndex:index defaultValue:0];
 }
 
 
-- (unsigned char) oo_unsignedCharAtIndex:(OOUInteger)index
+- (unsigned char) oo_unsignedCharAtIndex:(NSUInteger)index
 {
 	return [self oo_unsignedCharAtIndex:index defaultValue:0];
 }
 
 
-- (unsigned short) oo_unsignedShortAtIndex:(OOUInteger)index
+- (unsigned short) oo_unsignedShortAtIndex:(NSUInteger)index
 {
 	return [self oo_unsignedShortAtIndex:index defaultValue:0];
 }
 
 
-- (unsigned int) oo_unsignedIntAtIndex:(OOUInteger)index
+- (unsigned int) oo_unsignedIntAtIndex:(NSUInteger)index
 {
 	return [self oo_unsignedIntAtIndex:index defaultValue:0];
 }
 
 
-- (unsigned long) oo_unsignedLongAtIndex:(OOUInteger)index
+- (unsigned long) oo_unsignedLongAtIndex:(NSUInteger)index
 {
 	return [self oo_unsignedLongAtIndex:index defaultValue:0];
 }
 
 
-- (unsigned long long) oo_unsignedLongLongAtIndex:(OOUInteger)index
+- (unsigned long long) oo_unsignedLongLongAtIndex:(NSUInteger)index
 {
 	return [self oo_unsignedLongLongAtIndex:index defaultValue:0];
 }
 
 
-- (OOUInteger) oo_unsignedIntegerAtIndex:(OOUInteger)index
+- (NSUInteger) oo_unsignedIntegerAtIndex:(NSUInteger)index
 {
 	return [self oo_unsignedIntegerAtIndex:index defaultValue:0];
 }
 
 
-- (BOOL) oo_boolAtIndex:(OOUInteger)index
+- (BOOL) oo_boolAtIndex:(NSUInteger)index
 {
 	return [self oo_boolAtIndex:index defaultValue:NO];
 }
 
 
 #ifndef OOCOLLECTIONEXTRACTORS_SIMPLE
-- (BOOL) oo_fuzzyBooleanAtIndex:(OOUInteger)index
+- (BOOL) oo_fuzzyBooleanAtIndex:(NSUInteger)index
 {
 	return [self oo_fuzzyBooleanAtIndex:index defaultValue:0.0f];
 }
 #endif
 
 
-- (float) oo_floatAtIndex:(OOUInteger)index
+- (float) oo_floatAtIndex:(NSUInteger)index
 {
 	return OOFloatFromObject([self oo_objectAtIndex:index], 0.0f);
 }
 
 
-- (double) oo_doubleAtIndex:(OOUInteger)index
+- (double) oo_doubleAtIndex:(NSUInteger)index
 {
 	return OODoubleFromObject([self oo_objectAtIndex:index], 0.0);
 }
 
 
-- (float) oo_nonNegativeFloatAtIndex:(OOUInteger)index
+- (float) oo_nonNegativeFloatAtIndex:(NSUInteger)index
 {
 	return OONonNegativeFloatFromObject([self oo_objectAtIndex:index], 0.0f);
 }
 
 
-- (double) oo_nonNegativeDoubleAtIndex:(OOUInteger)index
+- (double) oo_nonNegativeDoubleAtIndex:(NSUInteger)index
 {
 	return OONonNegativeDoubleFromObject([self oo_objectAtIndex:index], 0.0);
 }
 
 
-- (id) oo_objectAtIndex:(OOUInteger)index
+- (id) oo_objectAtIndex:(NSUInteger)index
 {
 	if (index < [self count])  return [self objectAtIndex:index];
 	else  return nil;
 }
 
 
-- (id) oo_objectOfClass:(Class)class atIndex:(OOUInteger)index
+- (id) oo_objectOfClass:(Class)class atIndex:(NSUInteger)index
 {
 	return [self oo_objectOfClass:class atIndex:index defaultValue:nil];
 }
 
 
-- (NSString *) oo_stringAtIndex:(OOUInteger)index
+- (NSString *) oo_stringAtIndex:(NSUInteger)index
 {
 	return [self oo_stringAtIndex:index defaultValue:nil];
 }
 
 
-- (NSArray *) oo_arrayAtIndex:(OOUInteger)index
+- (NSArray *) oo_arrayAtIndex:(NSUInteger)index
 {
 	return [self oo_arrayAtIndex:index defaultValue:nil];
 }
 
 
-- (NSSet *) oo_setAtIndex:(OOUInteger)index
+- (NSSet *) oo_setAtIndex:(NSUInteger)index
 {
 	return [self oo_setAtIndex:index defaultValue:nil];
 }
 
 
-- (NSDictionary *) oo_dictionaryAtIndex:(OOUInteger)index
+- (NSDictionary *) oo_dictionaryAtIndex:(NSUInteger)index
 {
 	return [self oo_dictionaryAtIndex:index defaultValue:nil];
 }
 
 
-- (NSData *) oo_dataAtIndex:(OOUInteger)index
+- (NSData *) oo_dataAtIndex:(NSUInteger)index
 {
 	return [self oo_dataAtIndex:index defaultValue:nil];
 }
 
 
 #ifndef OOCOLLECTIONEXTRACTORS_SIMPLE
-- (Vector) oo_vectorAtIndex:(OOUInteger)index
+- (Vector) oo_vectorAtIndex:(NSUInteger)index
 {
 	return [self oo_vectorAtIndex:index defaultValue:kZeroVector];
 }
 
 
-- (Quaternion) oo_quaternionAtIndex:(OOUInteger)index
+- (Quaternion) oo_quaternionAtIndex:(NSUInteger)index
 {
 	return [self oo_quaternionAtIndex:index defaultValue:kIdentityQuaternion];
 }
@@ -419,7 +419,7 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
-- (OOInteger) oo_integerForKey:(id)key defaultValue:(OOInteger)value
+- (NSInteger) oo_integerForKey:(id)key defaultValue:(NSInteger)value
 {
 	return OOIntegerFromObject([self objectForKey:key], value);
 }
@@ -455,7 +455,7 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
-- (OOUInteger) oo_unsignedIntegerForKey:(id)key defaultValue:(OOUInteger)value
+- (NSUInteger) oo_unsignedIntegerForKey:(id)key defaultValue:(NSUInteger)value
 {
 	return OOUIntegerFromObject([self objectForKey:key], value);
 }
@@ -547,6 +547,13 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
+- (NSMutableDictionary *) oo_mutableDictionaryForKey:(id)key defaultValue:(NSDictionary *)value
+{
+	return [self oo_objectOfClass:[NSMutableDictionary class] forKey:key defaultValue:value];
+}
+
+
+
 - (NSData *) oo_dataForKey:(id)key defaultValue:(NSData *)value
 {
 	return [self oo_objectOfClass:[NSData class] forKey:key defaultValue:value];
@@ -559,6 +566,11 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 	return OOVectorFromObject([self objectForKey:key], value);
 }
 
+- (HPVector) oo_hpvectorForKey:(id)key defaultValue:(HPVector)value
+{
+	return OOHPVectorFromObject([self objectForKey:key], value);
+}
+
 
 - (Quaternion) oo_quaternionForKey:(id)key defaultValue:(Quaternion)value
 {
@@ -597,7 +609,7 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
-- (OOInteger) oo_integerForKey:(id)key
+- (NSInteger) oo_integerForKey:(id)key
 {
 	return [self oo_integerForKey:key defaultValue:0];
 }
@@ -627,7 +639,7 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
-- (OOUInteger) oo_unsignedIntegerForKey:(id)key
+- (NSUInteger) oo_unsignedIntegerForKey:(id)key
 {
 	return [self oo_unsignedIntegerForKey:key defaultValue:0];
 }
@@ -707,6 +719,12 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
+- (NSMutableDictionary *) oo_mutableDictionaryForKey:(id)key
+{
+	return [self oo_mutableDictionaryForKey:key defaultValue:nil];
+}
+
+
 - (NSData *) oo_dataForKey:(id)key
 {
 	return [self oo_dataForKey:key defaultValue:nil];
@@ -719,6 +737,11 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 	return [self oo_vectorForKey:key defaultValue:kZeroVector];
 }
 
+- (HPVector) oo_hpvectorForKey:(id)key
+{
+	return [self oo_hpvectorForKey:key defaultValue:kZeroHPVector];
+}
+
 
 - (Quaternion) oo_quaternionForKey:(id)key
 {
@@ -761,6 +784,12 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
+- (NSInteger) oo_integerForKey:(id)key defaultValue:(NSInteger)value
+{
+	return OOIntegerFromObject([self objectForKey:key], value);
+}
+
+
 - (unsigned char) oo_unsignedCharForKey:(id)key defaultValue:(unsigned char)value
 {
 	return OOUnsignedCharFromObject([self objectForKey:key], value);
@@ -791,6 +820,12 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 }
 
 
+- (NSUInteger) oo_unsignedIntegerForKey:(id)key defaultValue:(NSUInteger)value
+{
+	return OOUIntegerFromObject([self objectForKey:key], value);
+}
+
+
 - (BOOL) oo_boolForKey:(id)key defaultValue:(BOOL)value
 {
 	return OOBooleanFromObject([self objectForKey:key], value);
@@ -1023,38 +1058,38 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 #endif
 
 
-- (void) oo_insertInteger:(long)value atIndex:(OOUInteger)index
+- (void) oo_insertInteger:(long)value atIndex:(NSUInteger)index
 {
 	[self insertObject:[NSNumber numberWithLong:value] atIndex:index];
 }
 
 
-- (void) oo_insertUnsignedInteger:(unsigned long)value atIndex:(OOUInteger)index
+- (void) oo_insertUnsignedInteger:(unsigned long)value atIndex:(NSUInteger)index
 {
 	[self insertObject:[NSNumber numberWithUnsignedLong:value] atIndex:index];
 }
 
 
-- (void) oo_insertFloat:(double)value atIndex:(OOUInteger)index
+- (void) oo_insertFloat:(double)value atIndex:(NSUInteger)index
 {
 	[self insertObject:[NSNumber numberWithDouble:value] atIndex:index];
 }
 
 
-- (void) oo_insertBool:(BOOL)value atIndex:(OOUInteger)index
+- (void) oo_insertBool:(BOOL)value atIndex:(NSUInteger)index
 {
 	[self insertObject:[NSNumber numberWithBool:value] atIndex:index];
 }
 
 
 #ifndef OOCOLLECTIONEXTRACTORS_SIMPLE
-- (void) oo_insertVector:(Vector)value atIndex:(OOUInteger)index
+- (void) oo_insertVector:(Vector)value atIndex:(NSUInteger)index
 {
 	[self insertObject:OOPropertyListFromVector(value) atIndex:index];
 }
 
 
-- (void) oo_insertQuaternion:(Quaternion)value atIndex:(OOUInteger)index
+- (void) oo_insertQuaternion:(Quaternion)value atIndex:(NSUInteger)index
 {
 	[self insertObject:OOPropertyListFromQuaternion(value) atIndex:index];
 }
@@ -1107,6 +1142,10 @@ static NSString *StringForObject(id object, NSString *defaultValue);
 	[self setObject:OOPropertyListFromVector(value) forKey:key];
 }
 
+- (void) oo_setHPVector:(HPVector)value forKey:(id)key
+{
+	[self setObject:OOPropertyListFromHPVector(value) forKey:key];
+}
 
 - (void) oo_setQuaternion:(Quaternion)value forKey:(id)key
 {
@@ -1347,7 +1386,7 @@ float OONonNegativeFloatFromObject(id object, float defaultValue)
 	else if ([object respondsToSelector:@selector(intValue)])  result = [object intValue];
 	else return defaultValue;	// Don't clamp default
 	
-	return OOMax_f(result, 0.0f);
+	return fmax(result, 0.0f);
 }
 
 
@@ -1360,7 +1399,7 @@ double OONonNegativeDoubleFromObject(id object, double defaultValue)
 	else if ([object respondsToSelector:@selector(intValue)])  result = [object intValue];
 	else return defaultValue;	// Don't clamp default
 	
-	return OOMax_d(result, 0.0f);
+	return fmax(result, 0.0);
 }
 
 
@@ -1399,6 +1438,40 @@ Vector OOVectorFromObject(id object, Vector defaultValue)
 	return result;
 }
 
+HPVector OOHPVectorFromObject(id object, HPVector defaultValue)
+{
+	HPVector				result = defaultValue;
+	NSDictionary		*dict = nil;
+	
+	if ([object isKindOfClass:[NSString class]])
+	{
+		// This will only write result if a valid vector is found, and will write an error message otherwise.
+		ScanHPVectorFromString(object, &result);
+	}
+	else if ([object isKindOfClass:[NSArray class]] && [object count] == 3)
+	{
+		result.x = [object oo_doubleAtIndex:0];
+		result.y = [object oo_doubleAtIndex:1];
+		result.z = [object oo_doubleAtIndex:2];
+	}
+	else if ([object isKindOfClass:[NSDictionary class]])
+	{
+		dict = object;
+		// Require at least one of the keys x, y, or z
+		if ([dict objectForKey:@"x"] != nil ||
+			[dict objectForKey:@"y"] != nil ||
+			[dict objectForKey:@"z"] != nil)
+		{
+			// Note: uses 0 for unknown components rather than components of defaultValue.
+			result.x = [dict oo_doubleForKey:@"x" defaultValue:0.0];
+			result.y = [dict oo_doubleForKey:@"y" defaultValue:0.0];
+			result.z = [dict oo_doubleForKey:@"z" defaultValue:0.0];
+		}
+	}
+	
+	return result;
+}
+
 
 Quaternion OOQuaternionFromObject(id object, Quaternion defaultValue)
 {
@@ -1447,6 +1520,15 @@ NSDictionary *OOPropertyListFromVector(Vector value)
 			nil];
 }
 
+NSDictionary *OOPropertyListFromHPVector(HPVector value)
+{
+	return [NSDictionary dictionaryWithObjectsAndKeys:
+			[NSNumber numberWithDouble:value.x], @"x",
+			[NSNumber numberWithDouble:value.y], @"y",
+			[NSNumber numberWithDouble:value.z], @"z",
+			nil];
+}
+
 
 NSDictionary *OOPropertyListFromQuaternion(Quaternion value)
 {
