@@ -3,7 +3,7 @@
 OOLogging.m
 
 
-Copyright (C) 2007-2011 Jens Ayton and contributors
+Copyright (C) 2007-2013 Jens Ayton and contributors
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -41,9 +41,6 @@ SOFTWARE.
 
 
 #define PER_THREAD_INDENTATION		1
-#ifndef APPNAME
-#define APPNAME						@"Oolite"
-#endif
 
 
 #if PER_THREAD_INDENTATION
@@ -65,7 +62,6 @@ SOFTWARE.
 #define OOLOG_NOT_INITED			1
 #define OOLOG_SETTING_SET			0
 #define OOLOG_SETTING_RETRIEVE		0
-#define OOLOG_METACLASS_LOOP		1
 #define OOLOG_UNDEFINED_METACLASS	1
 #define OOLOG_BAD_SETTING			1
 #define OOLOG_BAD_DEFAULT_SETTING	1
@@ -94,8 +90,8 @@ static THREAD_LOCAL OOLogIndentStackElement
 #endif
 static BOOL						sShowFunction = NO;
 static BOOL						sShowFileAndLine = NO;
-static BOOL						sShowTime = YES;
-static BOOL						sShowClass = YES;
+static BOOL						sShowTime = NO;		// This is defaulted to YES when loading settings, but after printing log header.
+static BOOL						sShowClass = NO;	// Ditto.
 static BOOL						sDefaultDisplay = YES;
 static BOOL						sOverrideInEffect = NO;
 static BOOL						sOverrideValue = NO;
@@ -127,8 +123,8 @@ static NSMapTable				*sFileNamesCache = NULL;
 
 
 // Given a boolean, return the appropriate value for the cache dictionary.
-static inline id CacheValue(BOOL inValue) __attribute__((pure));
-static inline id CacheValue(BOOL inValue)
+OOINLINE id CacheValue(BOOL inValue) PURE_FUNC;
+OOINLINE id CacheValue(BOOL inValue)
 {
 	return inValue ? kTrueToken : kFalseToken;
 }
@@ -137,7 +133,7 @@ static inline id CacheValue(BOOL inValue)
 /*	Inited()
 	Test wether OOLoggingInit() has been called.
 */
-static inline BOOL Inited(void)
+OOINLINE BOOL Inited(void)
 {
 	if (EXPECT(sInited)) return YES;
 	OOLogInternal(OOLOG_NOT_INITED, @"***** ERROR: OOLoggingInit() has not been called.");
@@ -159,7 +155,8 @@ BOOL OOLogWillDisplayMessagesInClass(NSString *inMessageClass)
 	value = [sDerivedSettingsCache objectForKey:inMessageClass];
 	if (EXPECT_NOT(value == nil))
 	{
-		NS_DURING
+		@try
+		{
 			// No cached value.
 			value = ResolveDisplaySetting(inMessageClass);
 			
@@ -168,10 +165,12 @@ BOOL OOLogWillDisplayMessagesInClass(NSString *inMessageClass)
 				if (EXPECT_NOT(sDerivedSettingsCache == nil)) sDerivedSettingsCache = [[NSMutableDictionary alloc] init];
 				[sDerivedSettingsCache setObject:value forKey:inMessageClass];
 			}
-		NS_HANDLER
+		}
+		@catch (id exception)
+		{
 			[sLock unlock];
-			[localException raise];
-		NS_ENDHANDLER
+			@throw exception;
+		}
 	}
 	[sLock unlock];
 	
@@ -194,9 +193,8 @@ void OOLogSetDisplayMessagesInClass(NSString *inClass, BOOL inFlag)
 		
 		[sExplicitSettings setObject:CacheValue(inFlag) forKey:inClass];
 		
-		// Clear cache and let it be rebuilt as needed. Cost of rebuilding cache is not sufficient to warrant complexity of a partial clear.
-		[sDerivedSettingsCache release];
-		sDerivedSettingsCache = nil;
+		// Clear cache and let it be rebuilt as needed.
+		DESTROY(sDerivedSettingsCache);
 	}
 	else
 	{
@@ -329,6 +327,11 @@ void OOLogPushIndent(void)
 		
 		elem->indent = [[threadDict objectForKey:kIndentLevelKey] intValue];
 		elem->link = [val pointerValue];
+		
+		/*
+			Clang static analyzer reports elem not released here. It is in fact
+			released in OOLogPopIndent().
+		*/
 		[threadDict setObject:[NSValue valueWithPointer:elem] forKey:kIndentStackKey];
 	}
 }
@@ -414,7 +417,8 @@ void OOLogWithFunctionFileAndLineAndArguments(NSString *inMessageClass, const ch
 #endif
 	
 	pool = [[NSAutoreleasePool alloc] init];
-	NS_DURING
+	@try
+	{
 		// Do argument substitution
 		formattedMessage = [[[NSString alloc] initWithFormat:inFormat arguments:inArguments] autorelease];
 		
@@ -424,11 +428,11 @@ void OOLogWithFunctionFileAndLineAndArguments(NSString *inMessageClass, const ch
 		{
 			if (sShowFunction)
 			{
-				formattedMessage = [NSString stringWithFormat:@"%s (%@:%u): %@", inFunction, OOLogAbbreviatedFileName(inFile), inLine, formattedMessage];
+				formattedMessage = [NSString stringWithFormat:@"%s (%@:%lu): %@", inFunction, OOLogAbbreviatedFileName(inFile), inLine, formattedMessage];
 			}
 			else
 			{
-				formattedMessage = [NSString stringWithFormat:@"%@:%u: %@", OOLogAbbreviatedFileName(inFile), inLine, formattedMessage];
+				formattedMessage = [NSString stringWithFormat:@"%@:%lu: %@", OOLogAbbreviatedFileName(inFile), inLine, formattedMessage];
 			}
 		}
 		else
@@ -478,9 +482,11 @@ void OOLogWithFunctionFileAndLineAndArguments(NSString *inMessageClass, const ch
 		}
 		
 		OOLogOutputHandlerPrint(formattedMessage);
-	NS_HANDLER
-		OOLogInternal(OOLOG_EXCEPTION_IN_LOG, @"***** Exception thrown during logging: %@ : %@", [localException name], [localException reason]);
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		OOLogInternal(OOLOG_EXCEPTION_IN_LOG, @"***** Exception thrown during logging: %@ : %@", [exception name], [exception reason]);
+	}
 	
 	[pool release];
 }
@@ -585,7 +591,7 @@ void OOLoggingInit(void)
 	pool = [[NSAutoreleasePool alloc] init];
 	
 	sLock = [[NSLock alloc] init];
-	[sLock ooSetName:@"OOLogging lock"];
+	[sLock setName:@"OOLogging lock"];
 	if (sLock == nil) exit(EXIT_FAILURE);
 	
 #ifndef OOLOG_NO_FILE_NAME
@@ -595,10 +601,10 @@ void OOLoggingInit(void)
 	sInited = YES;	// Must be before OOLogOutputHandlerInit().
 	OOLogOutputHandlerInit();
 	
-	LoadExplicitSettings();
-	
 	OOPrintLogHeader();
 	
+	LoadExplicitSettings();
+	
 	[pool release];
 }
 
@@ -617,6 +623,12 @@ void OOLoggingTerminate(void)
 }
 
 
+void OOLoggingReloadSettings(void)
+{
+	LoadExplicitSettings();
+}
+
+
 void OOLogInsertMarker(void)
 {
 	static unsigned		lastMarkerID = 0;
@@ -655,7 +667,8 @@ static void OOLogInternal_(const char *inFunction, NSString *inFormat, ...)
 	
 	pool = [[NSAutoreleasePool alloc] init];
 	
-	NS_DURING
+	@try
+	{
 		va_start(args, inFormat);
 		formattedMessage = [[[NSString alloc] initWithFormat:inFormat arguments:args] autorelease];
 		va_end(args);
@@ -663,57 +676,68 @@ static void OOLogInternal_(const char *inFunction, NSString *inFormat, ...)
 		formattedMessage = [NSString stringWithFormat:@"OOLogging internal - %s: %@", inFunction, formattedMessage];
 		
 		OOLogOutputHandlerPrint(formattedMessage);
-	NS_HANDLER
-		fprintf(stderr, "***** Exception in OOLogInternal_(): %s : %s", [[localException name] UTF8String], [[localException reason] UTF8String]);
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		fprintf(stderr, "***** Exception in OOLogInternal_(): %s : %s", [[exception name] UTF8String], [[exception reason] UTF8String]);
+	}
 	
 	[pool release];
 }
 
 
 /*	LoadExplicitSettings()
-	Read settings from logcontrol.plist, merge in settings from preferences.
+	Read settings from logcontrol.plists, merge in settings from preferences.
+	
+	Log settings are loaded from the following locations, from lowest to
+	highest priority:
+		* logcontrol.plist inside OXPs, but only in hierarchies not defined
+		  by the built-in plist.
+		* Built-in logcontrol.plist.
+		* Loose logcontrol.plist files in AddOns folders.
+		* Preferences (settable through the debug console).
+	
+	Because the settings are loaded very early (OOLoggingInit() time), before
+	the state of strict mode has been set, OXP settings are always loaded.
+	Since these generally shouldn't have any effect in strict mode, that's
+	fine.
 */
 static void LoadExplicitSettings(void)
 {
-	NSEnumerator		*rootEnum = nil;
-	NSString			*basePath = nil;
-	NSString			*configPath = nil;
-	NSDictionary		*dict = nil;
-	NSUserDefaults		*prefs = nil;
-	id					value = nil;
+	// Load display settings.
+	NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
+	sShowFunction = [prefs oo_boolForKey:@"logging-show-function" defaultValue:NO];
+	sShowFileAndLine = [prefs oo_boolForKey:@"logging-show-file-and-line" defaultValue:NO];
+	sShowTime = [prefs oo_boolForKey:@"logging-show-time" defaultValue:YES];
+	sShowClass = [prefs oo_boolForKey:@"logging-show-class" defaultValue:YES];
 	
-	if (sExplicitSettings != nil) return;
+	NSDictionary *oldSettings = sExplicitSettings;
 	
+	/*
+		HACK: we look up search paths while loading settings, which touches
+		the cache, which logs stuff. To avoid spam like dataCache.retrieve.success,
+		we first load the built-in logcontrol.plist only and use it while
+		loading the full settings.
+	*/
+	NSString *path = [[[ResourceManager builtInPath] stringByAppendingPathComponent:@"Config"]
+					  stringByAppendingPathComponent:@"logcontrol.plist"];
+	sExplicitSettings = [NSMutableDictionary dictionary];
+	LoadExplicitSettingsFromDictionary(OODictionaryFromFile(path));
+	
+	// Release previous sExplicitSettings.
+	[oldSettings release];
+	
+	// Load new settings.
+	NSDictionary *settings = [ResourceManager logControlDictionary];
+	
+	// Replace the old dictionary.
 	sExplicitSettings = [[NSMutableDictionary alloc] init];
 	
-	rootEnum = [[ResourceManager rootPaths] objectEnumerator];
-	while ((basePath = [rootEnum nextObject]))
-	{
-		configPath = [[basePath stringByAppendingPathComponent:@"Config"]
-								stringByAppendingPathComponent:@"logcontrol.plist"];
-		dict = OODictionaryFromFile(configPath);
-		if (dict == nil)
-		{
-			configPath = [basePath stringByAppendingPathComponent:@"logcontrol.plist"];
-			dict = OODictionaryFromFile(configPath);
-		}
-		if (dict != nil)
-		{
-			LoadExplicitSettingsFromDictionary(dict);
-		}
-	}
+	// Populate.
+	LoadExplicitSettingsFromDictionary(settings);
 	
-	// Get overrides from preferences
-	prefs = [NSUserDefaults standardUserDefaults];
-	dict = [prefs objectForKey:@"logging-enable"];
-	if ([dict isKindOfClass:[NSDictionary class]])
-	{
-		LoadExplicitSettingsFromDictionary(dict);
-	}
-	
-	// Get _default and _override value
-	value = [sExplicitSettings objectForKey:@"_default"];
+	// Get _default and _override values.
+	id value = [sExplicitSettings objectForKey:@"_default"];
 	if (value != nil && [value respondsToSelector:@selector(boolValue)])
 	{
 		if (value == kTrueToken) sDefaultDisplay = YES;
@@ -740,11 +764,8 @@ static void LoadExplicitSettings(void)
 		[sExplicitSettings removeObjectForKey:@"_override"];
 	}
 	
-	// Load display settings
-	sShowFunction = [prefs oo_boolForKey:@"logging-show-function" defaultValue:sShowFunction];
-	sShowFileAndLine = [prefs oo_boolForKey:@"logging-show-file-and-line" defaultValue:sShowFileAndLine];
-	sShowTime = [prefs oo_boolForKey:@"logging-show-time" defaultValue:sShowTime];
-	sShowClass = [prefs oo_boolForKey:@"logging-show-class" defaultValue:sShowClass];
+	// Invalidate cache.
+	DESTROY(sDerivedSettingsCache);
 	
 	OOLogInternal(OOLOG_SETTING_SET, @"Settings: %@", sExplicitSettings);
 }
@@ -755,13 +776,10 @@ static void LoadExplicitSettings(void)
 */
 static void LoadExplicitSettingsFromDictionary(NSDictionary *inDict)
 {
-	NSEnumerator		*keyEnum = nil;
-	id					key = nil;
-	id					value = nil;
-	
-	for (keyEnum = [inDict keyEnumerator]; (key = [keyEnum nextObject]); )
+	id key = nil;
+	foreachkey (key, inDict)
 	{
-		value = [inDict objectForKey:key];
+		id value = [inDict objectForKey:key];
 		
 		/*	Supported values:
 			"yes", "true" or "on" -> kTrueToken
