@@ -3,7 +3,7 @@
 OOOpenGLExtensionManager.m
 
 
-Copyright (C) 2007-2011 Jens Ayton and contributors
+Copyright (C) 2007-2013 Jens Ayton and contributors
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -61,6 +61,7 @@ PFNGLUSEPROGRAMOBJECTARBPROC			glUseProgramObjectARB			= (PFNGLUSEPROGRAMOBJECTA
 PFNGLGETUNIFORMLOCATIONARBPROC			glGetUniformLocationARB			= (PFNGLGETUNIFORMLOCATIONARBPROC)&OOBadOpenGLExtensionUsed;
 PFNGLUNIFORM1IARBPROC					glUniform1iARB					= (PFNGLUNIFORM1IARBPROC)&OOBadOpenGLExtensionUsed;
 PFNGLUNIFORM1FARBPROC					glUniform1fARB					= (PFNGLUNIFORM1FARBPROC)&OOBadOpenGLExtensionUsed;
+PFNGLUNIFORMMATRIX3FVARBPROC			glUniformMatrix3fvARB			= (PFNGLUNIFORMMATRIX3FVARBPROC)&OOBadOpenGLExtensionUsed;
 PFNGLUNIFORMMATRIX4FVARBPROC			glUniformMatrix4fvARB			= (PFNGLUNIFORMMATRIX4FVARBPROC)&OOBadOpenGLExtensionUsed;
 PFNGLUNIFORM4FVARBPROC					glUniform4fvARB					= (PFNGLUNIFORM4FVARBPROC)&OOBadOpenGLExtensionUsed;
 PFNGLGETOBJECTPARAMETERIVARBPROC		glGetObjectParameterivARB		= (PFNGLGETOBJECTPARAMETERIVARBPROC)&OOBadOpenGLExtensionUsed;
@@ -166,7 +167,7 @@ static NSArray *ArrayOfExtensions(NSString *extensionString)
 	{
 #if OOOPENGLEXTMGR_LOCK_SET_ACCESS
 		lock = [[NSLock alloc] init];
-		[lock ooSetName:@"OOOpenGLExtensionManager extension set lock"];
+		[lock setName:@"OOOpenGLExtensionManager extension set lock"];
 #endif
 		
 		[self reset];
@@ -218,7 +219,7 @@ static NSArray *ArrayOfExtensions(NSString *extensionString)
 	[ResourceManager paths];
 	
 	OOLog(@"rendering.opengl.version", @"OpenGL renderer version: %u.%u.%u (\"%s\"). Vendor: \"%@\". Renderer: \"%@\".", major, minor, release, versionString, vendor, renderer);
-	OOLog(@"rendering.opengl.extensions", @"OpenGL extensions (%u):\n%@", [extensions count], [[extensions allObjects] componentsJoinedByString:@", "]);
+	OOLog(@"rendering.opengl.extensions", @"OpenGL extensions (%lu):\n%@", [extensions count], [[extensions allObjects] componentsJoinedByString:@", "]);
 	
 	if (![self versionIsAtLeastMajor:kMinMajorVersion minor:kMinMinorVersion])
 	{
@@ -261,8 +262,8 @@ static NSArray *ArrayOfExtensions(NSString *extensionString)
 		maximumShaderSetting = SHADERS_NOT_SUPPORTED;
 	}
 	
-	GLint texImageUnitOverride = [gpuConfig oo_unsignedIntegerForKey:@"texture_image_units" defaultValue:textureImageUnitCount];
-	if (texImageUnitOverride < textureImageUnitCount)  textureImageUnitCount = texImageUnitOverride;
+	GLint texImageUnitOverride = [gpuConfig oo_intForKey:@"texture_image_units" defaultValue:textureImageUnitCount];
+	if (texImageUnitOverride < textureImageUnitCount)  textureImageUnitCount = MAX(texImageUnitOverride, 0);
 #endif
 	
 #if OO_USE_VBO
@@ -273,8 +274,8 @@ static NSArray *ArrayOfExtensions(NSString *extensionString)
 #endif
 #if OO_MULTITEXTURE
 	[self checkTextureCombinersSupported];
-	GLint texUnitOverride = [gpuConfig oo_unsignedIntegerForKey:@"texture_units" defaultValue:textureUnitCount];
-	if (texUnitOverride < textureUnitCount)  textureUnitCount = texUnitOverride;
+	GLint texUnitOverride = [gpuConfig oo_intForKey:@"texture_units" defaultValue:textureUnitCount];
+	if (texUnitOverride < textureUnitCount)  textureUnitCount = MAX(texUnitOverride, 0);
 #endif
 	
 	usePointSmoothing = [gpuConfig oo_boolForKey:@"smooth_points" defaultValue:YES];
@@ -298,7 +299,7 @@ static NSArray *ArrayOfExtensions(NSString *extensionString)
 }
 
 
-+ (id)sharedManager
++ (OOOpenGLExtensionManager *)sharedManager
 {
 	// NOTE: assumes single-threaded first access. See header.
 	if (sSingleton == nil)  sSingleton = [[self alloc] init];
@@ -333,27 +334,41 @@ static NSArray *ArrayOfExtensions(NSString *extensionString)
 }
 
 
-- (OOShaderSetting)defaultShaderSetting
+- (OOGraphicsDetail)defaultDetailLevel
 {
 #if OO_SHADERS
-	return defaultShaderSetting;
+	if (defaultShaderSetting < SHADERS_FULL)
+	{
+		return DETAIL_LEVEL_MINIMUM;
+	}
+	else
+	{
+		return DETAIL_LEVEL_MAXIMUM;
+	}
 #else
 	return SHADERS_NOT_SUPPORTED;
 #endif
 }
 
 
-- (OOShaderSetting)maximumShaderSetting
+- (OOGraphicsDetail)maximumDetailLevel
 {
 #if OO_SHADERS
-	return maximumShaderSetting;
+	if (maximumShaderSetting < SHADERS_FULL)
+	{
+		return DETAIL_LEVEL_MINIMUM;
+	}
+	else
+	{
+		return DETAIL_LEVEL_MAXIMUM;
+	}
 #else
-	return SHADERS_NOT_SUPPORTED;
+	return DETAIL_LEVEL_MINIMUM;
 #endif
 }
 
 
-- (OOUInteger)textureImageUnitCount
+- (GLint)textureImageUnitCount
 {
 #if OO_SHADERS
 	return textureImageUnitCount;
@@ -393,7 +408,7 @@ static NSArray *ArrayOfExtensions(NSString *extensionString)
 }
 
 
-- (OOUInteger)textureUnitCount
+- (GLint)textureUnitCount
 {
 #if OO_MULTITEXTURE
 	return textureUnitCount;
@@ -403,19 +418,19 @@ static NSArray *ArrayOfExtensions(NSString *extensionString)
 }
 
 
-- (OOUInteger)majorVersionNumber
+- (NSUInteger)majorVersionNumber
 {
 	return major;
 }
 
 
-- (OOUInteger)minorVersionNumber
+- (NSUInteger)minorVersionNumber
 {
 	return minor;
 }
 
 
-- (OOUInteger)releaseVersionNumber
+- (NSUInteger)releaseVersionNumber
 {
 	return release;
 }
@@ -492,7 +507,25 @@ static unsigned IntegerFromString(const GLubyte **ioString)
 - (void)checkShadersSupported
 {
 	shadersAvailable = NO;
-	
+
+	/* Some cards claim to support shaders but do so extremely
+	 * badly. These are listed in gpu-settings.plist where we know
+	 * about them; for those we don't being able to run with
+	 * -noshaders may help get the game up and running at a frame rate
+	 * where thegraphics settings can be changed.  - CIM */
+	NSArray 		*arguments = [[NSProcessInfo processInfo] arguments];
+	NSEnumerator 	*argEnum = nil;
+	NSString 		*arg = nil;
+	// scan for shader overrides: -noshaders || --noshaders
+	for (argEnum = [arguments objectEnumerator]; (arg = [argEnum nextObject]); )
+	{
+		if ([arg isEqual:@"-noshaders"] || [arg isEqual:@"--noshaders"])
+		{
+			OOLog(kOOLogOpenGLShaderSupport, @"Shaders will not be used (disabled on command line).");
+			return;
+		}
+	}	
+
 	NSString * const requiredExtension[] = 
 						{
 							@"GL_ARB_shading_language_100",
@@ -528,6 +561,7 @@ static unsigned IntegerFromString(const GLubyte **ioString)
 	glGetUniformLocationARB		=	(PFNGLGETUNIFORMLOCATIONARBPROC)wglGetProcAddress("glGetUniformLocationARB");
 	glUniform1iARB				=	(PFNGLUNIFORM1IARBPROC)wglGetProcAddress("glUniform1iARB");
 	glUniform1fARB				=	(PFNGLUNIFORM1FARBPROC)wglGetProcAddress("glUniform1fARB");
+	glUniformMatrix3fvARB		=	(PFNGLUNIFORMMATRIX3FVARBPROC)wglGetProcAddress("glUniformMatrix3fvARB");
 	glUniformMatrix4fvARB		=	(PFNGLUNIFORMMATRIX4FVARBPROC)wglGetProcAddress("glUniformMatrix4fvARB");
 	glUniform4fvARB				=	(PFNGLUNIFORM4FVARBPROC)wglGetProcAddress("glUniform4fvARB");
 	glUniform2fvARB				=	(PFNGLUNIFORM2FVARBPROC)wglGetProcAddress("glUniform2fvARB");
@@ -744,7 +778,7 @@ NSComparisonResult CompareGPUSettingsByPriority(id a, id b, void *context)
 }
 
 
-- (OOUInteger)retainCount
+- (NSUInteger)retainCount
 {
 	return UINT_MAX;
 }
