@@ -4,7 +4,7 @@ OOPolygonSprite.m
 Oolite
 
 
-Copyright (C) 2009-2011 Jens Ayton
+Copyright (C) 2009-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -98,7 +98,6 @@ static void SVGDumpBeginPrimitive(TessPolygonData *data);
 static void SVGDumpEndPrimitive(TessPolygonData *data);
 static void SVGDumpAppendTriangle(TessPolygonData *data, NSPoint v0, NSPoint v1, NSPoint v2);
 #else
-#define SVGDumpBegin(data) do {} while (0)
 #define SVGDumpEnd(data) do {} while (0)
 #define SVGDumpBeginGroup(data, name) do {} while (0)
 #define SVGDumpEndGroup(data) do {} while (0)
@@ -179,6 +178,7 @@ static void APIENTRY ErrorCallback(GLenum error, void *polygonData);
 	NSParameterAssert(vbo != NULL && data != NULL);
 	
 	OO_ENTER_OPENGL();
+	OOSetOpenGLState(OPENGL_STATE_OVERLAY);
 	
 #if OO_USE_VBO
 	BOOL useVBO = [[OOOpenGLExtensionManager sharedManager] vboSupported];
@@ -210,6 +210,9 @@ static void APIENTRY ErrorCallback(GLenum error, void *polygonData);
 #if OO_USE_VBO
 	if (useVBO)  OOGL(glBindBufferARB(GL_ARRAY_BUFFER, 0));
 #endif
+	
+	OOVerifyOpenGLState();
+	OOCheckOpenGLErrors(@"OOPolygonSprite after rendering %@", self);
 }
 
 
@@ -289,7 +292,7 @@ static void APIENTRY ErrorCallback(GLenum error, void *polygonData);
 	gluTessBeginPolygon(tesselator, &polygonData);
 	SVGDumpBeginGroup(&polygonData, @"Fill");
 	
-	OOUInteger contourCount = [dataArray count], contourIndex;
+	NSUInteger contourCount = [dataArray count], contourIndex;
 	for (contourIndex = 0; contourIndex < contourCount && polygonData.OK; contourIndex++)
 	{
 		NSArray *contour = [dataArray oo_arrayAtIndex:contourIndex];
@@ -414,7 +417,7 @@ END:
 
 static void SubmitVertices(GLUtesselator *tesselator, TessPolygonData *polygonData, NSArray *contour)
 {
-	OOUInteger vertexCount = [contour count], vertexIndex;
+	NSUInteger vertexCount = [contour count], vertexIndex;
 	if (vertexCount > 2)
 	{
 		gluTessBeginContour(tesselator);
@@ -448,23 +451,23 @@ static NSArray *DataArrayToPoints(TessPolygonData *data, NSArray *dataArray)
 	
 	SVGDumpBeginGroup(data, @"Base contours");
 	
-	OOUInteger polyIter, polyCount = [dataArray count];
+	NSUInteger polyIter, polyCount = [dataArray count];
 	NSArray *subArrays[polyCount];
 	
 	for (polyIter = 0; polyIter < polyCount; polyIter++)
 	{
 		NSArray *polyDef = [dataArray objectAtIndex:polyIter];
-		OOUInteger vertIter, vertCount = [polyDef count] / 2;
+		NSUInteger vertIter, vertCount = [polyDef count] / 2;
 		NSMutableArray *newPolyDef = [NSMutableArray arrayWithCapacity:vertCount];
-		OOCGFloat area = 0;
+		CGFloat area = 0;
 		
-		OOCGFloat oldX = [polyDef oo_doubleAtIndex:(vertCount -1) * 2];
-		OOCGFloat oldY = [polyDef oo_doubleAtIndex:(vertCount -1) * 2 + 1];
+		CGFloat oldX = [polyDef oo_doubleAtIndex:(vertCount -1) * 2];
+		CGFloat oldY = [polyDef oo_doubleAtIndex:(vertCount -1) * 2 + 1];
 		
 		for (vertIter = 0; vertIter < vertCount; vertIter++)
 		{
-			OOCGFloat x = [polyDef oo_doubleAtIndex:vertIter * 2];
-			OOCGFloat y = [polyDef oo_doubleAtIndex:vertIter * 2 + 1];
+			CGFloat x = [polyDef oo_doubleAtIndex:vertIter * 2];
+			CGFloat y = [polyDef oo_doubleAtIndex:vertIter * 2 + 1];
 			
 			// Skip bad or duplicate vertices.
 			if (x == oldX && y == oldY)  continue;
@@ -504,7 +507,7 @@ static NSArray *DataArrayToPoints(TessPolygonData *data, NSArray *dataArray)
 
 static NSArray *BuildOutlineContour(NSArray *dataArray, GLfloat width, BOOL inner)
 {
-	OOUInteger i, count = [dataArray count];
+	NSUInteger i, count = [dataArray count];
 	if (count < 2)  return dataArray;
 	
 	/*
@@ -559,16 +562,16 @@ static NSArray *BuildOutlineContour(NSArray *dataArray, GLfloat width, BOOL inne
 	
 	for (i = 0; i < count; i++)
 	{
-		NSPoint a = PtFastNormal(PtSub(current, prev));
-		NSPoint b = PtFastNormal(PtSub(next, current));
+		NSPoint a = PtNormal(PtSub(current, prev));
+		NSPoint b = PtNormal(PtSub(next, current));
 		
-		OOCGFloat dot = PtDot(a, b);
+		CGFloat dot = PtDot(a, b);
 		BOOL clockwise = PtCross(a, b) < 0.0f;
 		
 		if (-dot < kCosMitreLimit || !clockwise)
 		{
 			// Non-mitreing case.
-			NSPoint t = PtFastNormal(PtAdd(a, b));
+			NSPoint t = PtNormal(PtAdd(a, b));
 			NSPoint v = PtScale(PtRotACW(t), width / PtDot(t, a));
 			
 			if (!isnan(v.x) && !isnan(v.y))
@@ -809,7 +812,7 @@ static void SVGDumpEnd(TessPolygonData *data)
 	if (data->debugSVG == nil)  return;
 	
 	[data->debugSVG appendString:@"</svg>\n"];
-	[ResourceManager writeDiagnosticData:[data->debugSVG dataUsingEncoding:NSUTF8StringEncoding] toFileNamed:[data->name stringByAppendingPathExtension:@"svg"]];
+	[ResourceManager writeDiagnosticString:data->debugSVG toFileNamed:[NSString stringWithFormat:@"Polygon Sprites/%@.svg", data->name]];
 	DESTROY(data->debugSVG);
 }
 
@@ -836,7 +839,7 @@ static void SVGDumpAppendBaseContour(TessPolygonData *data, NSArray *points)
 	NSString *groupName = [NSString stringWithFormat:@"contour %u", data->svgID++];
 	[data->debugSVG appendFormat:@"\t\t<g id=\"%@\" stroke=\"#BBB\" fill=\"none\">\n\t\t<path stroke-width=\"0.05\" d=\"", groupName];
 	
-	OOUInteger i, count = [points count];
+	NSUInteger i, count = [points count];
 	for (i = 0; i < count; i++)
 	{
 		NSPoint p = [[points objectAtIndex:i] pointValue];
