@@ -3,7 +3,7 @@
 OOShipRegistry.m
 
 
-Copyright (C) 2008-2011 Jens Ayton and contributors
+Copyright (C) 2008-2013 Jens Ayton and contributors
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -37,38 +37,49 @@ SOFTWARE.
 #import "GameController.h"
 #import "OOLegacyScriptWhitelist.h"
 #import "OODeepCopy.h"
+#import "OOColor.h"
+#import "OOStringExpander.h"
+#import "OOShipLibraryDescriptions.h"
+#import "Universe.h"
+#import "OOJSScript.h"
 
+#import "OODebugStandards.h"
 
 #define PRELOAD 0
 
 
 static void DumpStringAddrs(NSDictionary *dict, NSString *context);
+static NSComparisonResult SortDemoShipsByName (id a, id b, void* context);
+static NSComparisonResult SortDemoCategoriesByName (id a, id b, void* context);
 
 
 static OOShipRegistry	*sSingleton = nil;
 
+
 static NSString * const	kShipRegistryCacheName = @"ship registry";
 static NSString * const	kShipDataCacheKey = @"ship data";
 static NSString * const	kPlayerShipsCacheKey = @"player ships";
-static NSString * const	kDemoShipsCacheKey = @"demo ships";
 static NSString * const	kRoleWeightsCacheKey = @"role weights";
 static NSString * const	kDefaultDemoShip = @"coriolis-station";
+static NSString * const	kVisualEffectRegistryCacheName = @"visual effect registry";
+static NSString * const	kVisualEffectDataCacheKey = @"visual effect data";
 
 
 @interface OOShipRegistry (OODataLoader)
 
 - (void) loadShipData;
+- (void) loadDemoShipConditions;
 - (void) loadDemoShips;
 - (void) loadCachedRoleProbabilitySets;
 - (void) buildRoleProbabilitySets;
 
-- (BOOL) applyLikeShips:(NSMutableDictionary *)ioData;
+- (BOOL) applyLikeShips:(NSMutableDictionary *)ioData withKey:(NSString *)likeKey;
 - (BOOL) loadAndMergeShipyard:(NSMutableDictionary *)ioData;
 - (BOOL) stripPrivateKeys:(NSMutableDictionary *)ioData;
 - (BOOL) makeShipEntriesMutable:(NSMutableDictionary *)ioData;
 - (BOOL) loadAndApplyShipDataOverrides:(NSMutableDictionary *)ioData;
 - (BOOL) canonicalizeAndTagSubentities:(NSMutableDictionary *)ioData;
-- (BOOL) removeUnusableEntries:(NSMutableDictionary *)ioData;
+- (BOOL) removeUnusableEntries:(NSMutableDictionary *)ioData shipMode:(BOOL)shipMode;
 - (BOOL) sanitizeConditions:(NSMutableDictionary *)ioData;
 
 #if PRELOAD
@@ -125,6 +136,9 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 {
 	if (sSingleton != nil)
 	{
+		/* CIM: 'release' doesn't work - the class definition
+		 * overrides it, so this leaks memory. Needs a proper reset
+		 * method for reloading the ship registry data instead */
 		[sSingleton release];
 		sSingleton = nil;
 		
@@ -142,6 +156,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 		
 		_shipData = [[cache objectForKey:kShipDataCacheKey inCache:kShipRegistryCacheName] retain];
 		_playerShips = [[cache objectForKey:kPlayerShipsCacheKey inCache:kShipRegistryCacheName] retain];
+		_effectData = [[cache objectForKey:kVisualEffectDataCacheKey inCache:kVisualEffectRegistryCacheName] retain];
 		if ([_shipData count] == 0)	// Don't accept nil or empty
 		{
 			[self loadShipData];
@@ -155,14 +170,11 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 			}
 		}
 		
-		_demoShips = [[cache objectForKey:kDemoShipsCacheKey inCache:kShipRegistryCacheName] retain];
+		[self loadDemoShipConditions];
+		[self loadDemoShips]; // testing only
 		if ([_demoShips count] == 0)
 		{
-			[self loadDemoShips];
-			if ([_demoShips count] == 0)
-			{
-				[NSException raise:@"OOShipRegistryLoadFailure" format:@"Could not load or synthesize any demo ships."];
-			}
+			[NSException raise:@"OOShipRegistryLoadFailure" format:@"Could not load or synthesize any demo ships."];
 		}
 		
 		[self loadCachedRoleProbabilitySets];
@@ -198,6 +210,12 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 }
 
 
+- (NSDictionary *) effectInfoForKey:(NSString *)key
+{
+	return [_effectData objectForKey:key];
+}
+
+
 - (NSDictionary *) shipyardInfoForKey:(NSString *)key
 {
 	return [[self shipInfoForKey:key] objectForKey:@"_oo_shipyard"];
@@ -213,7 +231,10 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 
 - (NSArray *) demoShipKeys
 {
-	return _demoShips;
+	// with condition scripts in use, can't cache this value
+	[self loadDemoShips];
+
+	return [[_demoShips copy] autorelease]; 
 }
 
 
@@ -227,6 +248,16 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 
 @implementation OOShipRegistry (OOConveniences)
 
+- (NSArray *) shipKeys
+{
+	return [_shipData allKeys];
+}
+
+- (NSArray *) shipRoles
+{
+	return [_probabilitySets allKeys];
+}
+
 - (NSArray *) shipKeysWithRole:(NSString *)role
 {
 	return [[self probabilitySetForRole:role] allObjects];
@@ -257,9 +288,6 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 {
 	NSMutableDictionary		*result = nil;
 	
-	OOLog(@"shipData.load.begin", @"Loading ship data.");
-	OOLogIndentIf(@"shipData.load.begin");
-	
 	[_shipData release];
 	_shipData = nil;
 	[_playerShips release];
@@ -276,47 +304,106 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	
 	// Make each entry mutable to simplify later stages. Also removes any entries that aren't dictionaries.
 	if (![self makeShipEntriesMutable:result])  return;
-	OOLog(@"shipData.load.done", @"Finished initial cleanup...");
+	OOLog(@"shipData.load.progress", @"Finished initial cleanup...");
 	
 	// Apply patches.
 	if (![self loadAndApplyShipDataOverrides:result])  return;
-	OOLog(@"shipData.load.done", @"Finished applying patches...");
+	OOLog(@"shipData.load.progress", @"Finished applying patches...");
 	
 	// Strip private keys (anything starting with _oo_).
 	if (![self stripPrivateKeys:result])  return;
-	OOLog(@"shipData.load.done", @"Finished stripping private keys...");
+	OOLog(@"shipData.load.progress", @"Finished stripping private keys...");
 	
 	// Resolve like_ship entries.
-	if (![self applyLikeShips:result])  return;
-	OOLog(@"shipData.load.done", @"Finished resolving like_ships...");
+	if (![self applyLikeShips:result withKey:@"like_ship"])  return;
+	OOLog(@"shipData.load.progress", @"Finished resolving like_ships...");
 	
 	// Clean up subentity declarations and tag subentities so they won't be pruned.
 	if (![self canonicalizeAndTagSubentities:result])  return;
-	OOLog(@"shipData.load.done", @"Finished cleaning up subentities...");
+	OOLog(@"shipData.load.progress", @"Finished cleaning up subentities...");
 	
 	// Clean out templates and invalid entries.
-	if (![self removeUnusableEntries:result])  return;
-	OOLog(@"shipData.load.done", @"Finished removing invalid entries...");
+	if (![self removeUnusableEntries:result shipMode:YES])  return;
+	OOLog(@"shipData.load.progress", @"Finished removing invalid entries...");
 	
 	// Add shipyard entries into shipdata entries.
 	if (![self loadAndMergeShipyard:result])  return;
-	OOLog(@"shipData.load.done", @"Finished adding shipyard entries...");
+	OOLog(@"shipData.load.progress", @"Finished adding shipyard entries...");
 	
 	// Sanitize conditions.
 	if (![self sanitizeConditions:result])  return;
-	OOLog(@"shipData.load.done", @"Finished validating data...");
+	OOLog(@"shipData.load.progress", @"Finished validating data...");
 	
 #if PRELOAD
 	// Preload and cache meshes.
 	if (![self preloadShipMeshes:result])  return;
-	OOLog(@"shipData.load.done", @"Finished loading meshes...");
+	OOLog(@"shipData.load.progress", @"Finished loading meshes...");
 #endif
 	
 	_shipData = OODeepCopy(result);
 	[[OOCacheManager sharedCache] setObject:_shipData forKey:kShipDataCacheKey inCache:kShipRegistryCacheName];
 	
-	OOLogOutdentIf(@"shipData.load.begin");
 	OOLog(@"shipData.load.done", @"Ship data loaded.");
+
+	[_effectData release];
+	_effectData = nil;
+
+	result = [[[ResourceManager dictionaryFromFilesNamed:@"effectdata.plist"
+												inFolder:@"Config"
+											   mergeMode:MERGE_BASIC
+												   cache:NO] mutableCopy] autorelease];
+	if (result == nil)  return;
+
+	// Make each entry mutable to simplify later stages. Also removes any entries that aren't dictionaries.
+	if (![self makeShipEntriesMutable:result])  return;
+	OOLog(@"effectData.load.progress", @"Finished initial cleanup...");
+
+	// Strip private keys (anything starting with _oo_).
+	if (![self stripPrivateKeys:result])  return;
+	OOLog(@"effectData.load.progress", @"Finished stripping private keys...");
+	
+	// Resolve like_effect entries.
+	if (![self applyLikeShips:result withKey:@"like_effect"])  return;
+	OOLog(@"effectData.load.progress", @"Finished resolving like_effects...");
+	
+	// Clean up subentity declarations and tag subentities so they won't be pruned.
+	if (![self canonicalizeAndTagSubentities:result])  return;
+	OOLog(@"effectData.load.progress", @"Finished cleaning up subentities...");
+	
+	// Clean out templates and invalid entries.
+	if (![self removeUnusableEntries:result shipMode:NO])  return;
+	OOLog(@"effectData.load.progress", @"Finished removing invalid entries...");
+	
+	_effectData = OODeepCopy(result);
+	[[OOCacheManager sharedCache] setObject:_effectData forKey:kVisualEffectDataCacheKey inCache:kVisualEffectRegistryCacheName];
+	
+	OOLog(@"effectData.load.done", @"Effect data loaded.");
+}
+
+
+- (void) loadDemoShipConditions
+{
+	NSMutableArray *conditionScripts = [[NSMutableArray alloc] init];
+	NSEnumerator			*enumerator = nil;
+	NSDictionary			*key = nil;
+	NSArray					*initialDemoShips = nil;
+
+	initialDemoShips = [ResourceManager arrayFromFilesNamed:@"shiplibrary.plist"
+												   inFolder:@"Config"
+												   andMerge:YES
+													  cache:NO];
+
+	for (enumerator = [initialDemoShips objectEnumerator]; (key = [enumerator nextObject]); )
+	{
+		NSString *conditions = [key oo_stringForKey:kOODemoShipConditions defaultValue:nil];
+		if (conditions != nil)
+		{
+			[conditionScripts addObject:conditions];
+		}
+	}
+
+	[[OOCacheManager sharedCache] setObject:conditionScripts forKey:@"demoship conditions" inCache:@"condition scripts"];
+	[conditionScripts release];
 }
 
 
@@ -329,14 +416,13 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 - (void) loadDemoShips
 {
 	NSEnumerator			*enumerator = nil;
-	NSString				*key = nil;
+	NSDictionary			*key = nil;
 	NSArray					*initialDemoShips = nil;
 	NSMutableArray			*demoShips = nil;
 	
-	[_demoShips release];
-	_demoShips = nil;
+	DESTROY(_demoShips);
 	
-	initialDemoShips = [ResourceManager arrayFromFilesNamed:@"demoships.plist"
+	initialDemoShips = [ResourceManager arrayFromFilesNamed:@"shiplibrary.plist"
 												   inFolder:@"Config"
 												   andMerge:YES
 													  cache:NO];
@@ -345,20 +431,103 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	// Note: iterate over initialDemoShips to avoid mutating the collection being enu,erated.
 	for (enumerator = [initialDemoShips objectEnumerator]; (key = [enumerator nextObject]); )
 	{
-		if (![key isKindOfClass:[NSString class]] || [self shipInfoForKey:key] == nil)
+		NSString *shipKey = [key oo_stringForKey:kOODemoShipKey];
+		if (![key isKindOfClass:[NSDictionary class]] || [self shipInfoForKey:shipKey] == nil)
 		{
 			[demoShips removeObject:key];
 		}
+		else 
+		{
+			NSString *conditions = [key oo_stringForKey:kOODemoShipConditions defaultValue:nil];
+			if (conditions != nil)
+			{
+				if ([PLAYER status] == STATUS_START_GAME)
+				{
+					// conditions always false here
+					[demoShips removeObject:key];
+				}
+				else
+				{
+					OOJSScript *condScript = [UNIVERSE getConditionScript:conditions];
+					if (condScript != nil) // should always be non-nil, but just in case
+					{
+						JSContext			*context = OOJSAcquireContext();
+						BOOL OK;
+						JSBool allow_use;
+						jsval result;
+						jsval args[] = { OOJSValueFromNativeObject(context, shipKey) };
+						
+						OK = [condScript callMethod:OOJSID("allowShowLibraryShip")
+										  inContext:context
+									  withArguments:args count:sizeof args / sizeof *args
+											 result:&result];
+
+						if (OK) OK = JS_ValueToBoolean(context, result, &allow_use);
+			
+						OOJSRelinquishContext(context);
+						if (OK && !allow_use)
+						{
+							/* if the script exists, the function exists, the function
+							 * returns a bool, and that bool is false, hide the
+							 * ship. Otherwise allow it as default */
+							[demoShips removeObject:key];
+						}
+					}
+				}
+			}
+		}
 	}
 	
 	if ([demoShips count] == 0)
 	{
-		if ([self shipInfoForKey:kDefaultDemoShip] != nil)  [demoShips addObject:kDefaultDemoShip];
-		else  [demoShips addObject:[[_shipData allKeys] objectAtIndex:0]];
+		NSString *shipKey = nil;
+		if ([self shipInfoForKey:kDefaultDemoShip] != nil) 
+		{
+			shipKey = kDefaultDemoShip;
+		}
+		else
+		{
+			shipKey = [[_shipData allKeys] objectAtIndex:0];
+		}
+		[demoShips addObject:[NSDictionary dictionaryWithObject:shipKey forKey:kOODemoShipKey]];
 	}
 	
-	_demoShips = [demoShips copy];
-	[[OOCacheManager sharedCache] setObject:_demoShips forKey:kDemoShipsCacheKey inCache:kShipRegistryCacheName];
+	// now separate out the demoships by class, and add some extra keys
+	NSMutableDictionary *demoList = [NSMutableDictionary dictionaryWithCapacity:8];
+	NSMutableArray *demoClass = nil;
+	foreach(key, demoShips)
+	{
+		NSString *class = [key oo_stringForKey:kOODemoShipClass defaultValue:@"ship"];
+		if ([OOShipLibraryCategoryPlural(class) length] == 0)
+		{
+			OOLog(@"shipdata.load.warning",@"Unexpected class '%@' in shiplibrary.plist for '%@'",class,[key oo_stringForKey:kOODemoShipKey]);
+			class = @"ship";
+		}
+		demoClass = [demoList objectForKey:class];
+		if (demoClass == nil)
+		{
+			[demoList setObject:[NSMutableArray array] forKey:class];
+			demoClass = [demoList objectForKey:class];
+		}
+		NSMutableDictionary *demoEntry = [NSMutableDictionary dictionaryWithDictionary:key];
+		// add "name" object to dictionary from ship definition
+		[demoEntry setObject:[[self shipInfoForKey:[demoEntry oo_stringForKey:@"ship"]] oo_stringForKey:kOODemoShipName] forKey:kOODemoShipName];
+		// set "class" object to standard ship if not otherwise set
+		if (![[demoEntry oo_stringForKey:kOODemoShipClass defaultValue:nil] isEqualToString:class])
+		{
+			[demoEntry setObject:class forKey:kOODemoShipClass];
+		}
+		[demoClass addObject:demoEntry];
+	}
+	// sort each ship list by name
+	NSString *demoClassName = nil;
+	foreach(demoClassName, demoList)
+	{
+		[[demoList objectForKey:demoClassName] sortUsingFunction:SortDemoShipsByName context:NULL];
+	}
+
+	// and then sort the ship list list by class name
+	_demoShips = [[[demoList allValues] sortedArrayUsingFunction:SortDemoCategoriesByName context:NULL] retain];
 }
 
 
@@ -434,7 +603,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	resolved (either their like_ships do not exist, or they form reference
 	cycles) so we stop looping and report it.
 */
-- (BOOL) applyLikeShips:(NSMutableDictionary *)ioData
+- (BOOL) applyLikeShips:(NSMutableDictionary *)ioData withKey:(NSString *)likeKey
 {
 	NSMutableSet			*remainingLikeShips = nil;
 	NSEnumerator			*enumerator = nil;
@@ -442,7 +611,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	NSString				*parentKey = nil;
 	NSDictionary			*shipEntry = nil;
 	NSDictionary			*parentEntry = nil;
-	unsigned				count, lastCount;
+	NSUInteger				count, lastCount;
 	NSMutableArray			*reportedBadShips = nil;
 	
 	// Build set of ships with like_ship references
@@ -450,7 +619,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	for (enumerator = [ioData keyEnumerator]; (key = [enumerator nextObject]); )
 	{
 		shipEntry = [ioData objectForKey:key];
-		if ([shipEntry oo_stringForKey:@"like_ship"] != nil)
+		if ([shipEntry oo_stringForKey:likeKey] != nil)
 		{
 			[remainingLikeShips addObject:key];
 		}
@@ -463,7 +632,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 		{
 			// Look up like_ship entry
 			shipEntry = [ioData objectForKey:key];
-			parentKey = [shipEntry objectForKey:@"like_ship"];
+			parentKey = [shipEntry objectForKey:likeKey];
 			if (![remainingLikeShips containsObject:parentKey])
 			{
 				// If parent is fully resolved, we can resolve this child.
@@ -497,7 +666,8 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 			if ([reportedBadShips count] != 0)
 			{
 				[reportedBadShips sortUsingSelector:@selector(caseInsensitiveCompare:)];
-				OOLogERR(@"shipData.merge.failed", @"one or more shipdata.plist entries have like_ship references that cannot be resolved: %@", [reportedBadShips componentsJoinedByString:@", "]);
+				OOLogERR(@"shipData.merge.failed", @"one or more shipdata.plist entries have %@ references that cannot be resolved: %@", likeKey, [reportedBadShips componentsJoinedByString:@", "]); // FIXME: distinguish shipdata and effectdata
+				OOStandardsError(@"Likely missing a dependency in a manifest.plist");
 			}
 			break;
 		}
@@ -667,7 +837,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	NSDictionary			*shipyardOverrides = nil;
 	NSDictionary			*shipyardEntry = nil;
 	NSDictionary			*shipyardOverridesEntry = nil;
-	NSMutableSet			*playerShips = nil;
+	NSMutableArray			*playerShips = nil;
 	
 	// Strip out any shipyard stuff in shipdata (there shouldn't be any).
 	for (shipKeyEnum = [ioData keyEnumerator]; (shipKey = [shipKeyEnum nextObject]); )
@@ -755,6 +925,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 				// If entry is broken, we need to kill this ship.
 				if (fatal)
 				{
+					OOStandardsError(@"Bad subentity definition found");
 					remove = YES;
 				}
 				else if (subentityDict != nil)
@@ -766,7 +937,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 					{
 						subentityKey = [subentityDict oo_stringForKey:@"subentity_key"];
 						subentityShipEntry = [ioData objectForKey:subentityKey];
-						if (subentityKey == nil)
+						if (subentityKey == nil || subentityShipEntry == nil)
 						{
 							// Oops, reference to non-existent subent.
 							if (badSubentities == nil)  badSubentities = [NSMutableSet set];
@@ -797,6 +968,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 				{
 					badSubentitiesList = [[[badSubentities allObjects] sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] componentsJoinedByString:@", "];
 					OOLogERR(@"shipData.load.error", @"the shipdata.plist entry \"%@\" has unresolved subentit%@ %@.", shipKey, ([badSubentities count] == 1) ? @"y" : @"ies", badSubentitiesList);
+					OOStandardsError(@"Bad subentity definition found");
 				}
 				remove = YES;
 			}
@@ -813,7 +985,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 }
 
 
-- (BOOL) removeUnusableEntries:(NSMutableDictionary *)ioData
+- (BOOL) removeUnusableEntries:(NSMutableDictionary *)ioData shipMode:(BOOL)shipMode
 {
 	NSEnumerator			*shipKeyEnum = nil;
 	NSString				*shipKey = nil;
@@ -828,22 +1000,25 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 		remove = NO;
 		
 		if ([shipEntry oo_boolForKey:@"is_template"] || [shipEntry oo_boolForKey:@"_oo_deferred_remove"])  remove = YES;
-		else if ([[shipEntry oo_stringForKey:@"roles"] length] == 0 && ![shipEntry oo_boolForKey:@"_oo_is_subentity"])
+		else if (shipMode && [[shipEntry oo_stringForKey:@"roles"] length] == 0 && ![shipEntry oo_boolForKey:@"_oo_is_subentity"] && ![shipEntry oo_boolForKey:@"_oo_is_effect"])
 		{
 			OOLogERR(@"shipData.load.error", @"the shipdata.plist entry \"%@\" specifies no %@.", shipKey, @"roles");
 			remove = YES;
+			OOStandardsError(@"Error in shipdata.plist");
 		}
 		else
 		{
 			modelName = [shipEntry oo_stringForKey:@"model"];
-			if ([modelName length] == 0)
+			if (shipMode && [modelName length] == 0)
 			{
 				OOLogERR(@"shipData.load.error", @"the shipdata.plist entry \"%@\" specifies no %@.", shipKey, @"model");
+				OOStandardsError(@"Error in shipdata.plist");
 				remove = YES;
 			}
-			else if ([ResourceManager pathForFileNamed:modelName inFolder:@"Models"] == nil)
+			else if ([modelName length] != 0 && [ResourceManager pathForFileNamed:modelName inFolder:@"Models"] == nil)
 			{
-				OOLogERR(@"shipData.load.error", @"the shipdata.plist entry \"%@\" specifies non-existent model \"%@\".", shipKey, modelName, @"model");
+				OOLogERR(@"shipData.load.error", @"the shipdata.plist entry \"%@\" specifies non-existent model \"%@\".", shipKey, modelName);
+				OOStandardsError(@"Error in shipdata.plist");
 				remove = YES;
 			}
 		}
@@ -856,6 +1031,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 
 /*	Transform conditions, determinant (if conditions array) and
 	shipyard.conditions from hasShipyard to sanitized form.
+  Also get list of condition_scripts
 */
 - (BOOL) sanitizeConditions:(NSMutableDictionary *)ioData
 {
@@ -866,11 +1042,24 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	NSArray					*conditions = nil;
 	NSArray					*hasShipyard = nil;
 	NSArray					*shipyardConditions = nil;
+	NSString        *condition_script = nil;
+	NSString        *shipyard_condition_script = nil;
 	
+	NSMutableArray *conditionScripts = [[NSMutableArray alloc] init];
+
 	for (shipKeyEnum = [[ioData allKeys] objectEnumerator]; (shipKey = [shipKeyEnum nextObject]); )
 	{
 		shipEntry = [ioData objectForKey:shipKey];
 		conditions = [shipEntry objectForKey:@"conditions"];
+		condition_script = [shipEntry oo_stringForKey:@"condition_script"];
+		if (condition_script != nil)
+		{
+			if (![conditionScripts containsObject:condition_script])
+			{
+				[conditionScripts addObject:condition_script];
+			}
+		}
+
 		hasShipyard = [shipEntry objectForKey:@"has_shipyard"];
 		if (![hasShipyard isKindOfClass:[NSArray class]])  hasShipyard = nil;	// May also be fuzzy boolean
 		if (hasShipyard == nil)
@@ -879,73 +1068,96 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 			if (![hasShipyard isKindOfClass:[NSArray class]])  hasShipyard = nil;	// May also be fuzzy boolean
 		}
 		shipyardConditions = [[shipEntry oo_dictionaryForKey:@"_oo_shipyard"] objectForKey:@"conditions"];
+		shipyard_condition_script = [[shipEntry oo_dictionaryForKey:@"_oo_shipyard"] oo_stringForKey:@"condition_script"];
+		if (shipyard_condition_script != nil)
+		{
+			if (![conditionScripts containsObject:shipyard_condition_script])
+			{
+				[conditionScripts addObject:shipyard_condition_script];
+			}
+		}
+
 		
 		if (conditions == nil && hasShipyard && shipyardConditions == nil)  continue;
 		
 		if (conditions != nil)
 		{
-			if ([conditions isKindOfClass:[NSArray class]])
+			OOStandardsDeprecated([NSString stringWithFormat:@"The 'conditions' key is deprecated in shipdata entry %@",shipKey]);
+			if (!OOEnforceStandards())
 			{
-				conditions = OOSanitizeLegacyScriptConditions(conditions, [NSString stringWithFormat:@"<shipdata.plist entry \"%@\">", shipKey]);
-			}
-			else
-			{
-				OOLogWARN(@"shipdata.load.warning", @"conditions for shipdata.plist entry \"%@\" are not an array, ignoring.", shipKey);
-				conditions = nil;
-			}
+				if ([conditions isKindOfClass:[NSArray class]])
+				{
+					conditions = OOSanitizeLegacyScriptConditions(conditions, [NSString stringWithFormat:@"<shipdata.plist entry \"%@\">", shipKey]);
+				}
+				else
+				{
+					OOLogWARN(@"shipdata.load.warning", @"conditions for shipdata.plist entry \"%@\" are not an array, ignoring.", shipKey);
+					conditions = nil;
+				}
 			
-			if (conditions != nil)
-			{
-				[shipEntry setObject:conditions forKey:@"conditions"];
-			}
-			else
-			{
-				[shipEntry removeObjectForKey:@"conditions"];
+				if (conditions != nil)
+				{
+					[shipEntry setObject:conditions forKey:@"conditions"];
+				}
+				else
+				{
+					[shipEntry removeObjectForKey:@"conditions"];
+				}
 			}
 		}
 		
 		if (hasShipyard != nil)
 		{
 			hasShipyard = OOSanitizeLegacyScriptConditions(hasShipyard, [NSString stringWithFormat:@"<shipdata.plist entry \"%@\" hasShipyard conditions>", shipKey]);
-			
-			if (hasShipyard != nil)
+			OOStandardsDeprecated([NSString stringWithFormat:@"Use of legacy script conditions in the 'has_shipyard' key is deprecated in shipyard entry %@",shipKey]);
+			if (!OOEnforceStandards())
 			{
-				[shipEntry setObject:hasShipyard forKey:@"has_shipyard"];
-			}
-			else
-			{
-				[shipEntry removeObjectForKey:@"hasShipyard"];
-				[shipEntry removeObjectForKey:@"has_shipyard"];
+				if (hasShipyard != nil)
+				{
+					[shipEntry setObject:hasShipyard forKey:@"has_shipyard"];
+				}
+				else
+				{
+					[shipEntry removeObjectForKey:@"hasShipyard"];
+					[shipEntry removeObjectForKey:@"has_shipyard"];
+				}
 			}
 		}
 		
 		if (shipyardConditions != nil)
 		{
-			mutableShipyard = [[[shipEntry oo_dictionaryForKey:@"_oo_shipyard"] mutableCopy] autorelease];
+			OOStandardsDeprecated([NSString stringWithFormat:@"The 'conditions' key is deprecated in shipyard entry %@",shipKey]);
+			if (!OOEnforceStandards())
+			{
+				mutableShipyard = [[[shipEntry oo_dictionaryForKey:@"_oo_shipyard"] mutableCopy] autorelease];
 			
-			if ([shipyardConditions isKindOfClass:[NSArray class]])
-			{
-				shipyardConditions = OOSanitizeLegacyScriptConditions(shipyardConditions, [NSString stringWithFormat:@"<shipyard.plist entry \"%@\">", shipKey]);
-			}
-			else
-			{
-				OOLogWARN(@"shipdata.load.warning", @"conditions for shipyard.plist entry \"%@\" are not an array, ignoring.", shipKey);
-				shipyardConditions = nil;
-			}
+				if ([shipyardConditions isKindOfClass:[NSArray class]])
+				{
+					shipyardConditions = OOSanitizeLegacyScriptConditions(shipyardConditions, [NSString stringWithFormat:@"<shipyard.plist entry \"%@\">", shipKey]);
+				}
+				else
+				{
+					OOLogWARN(@"shipdata.load.warning", @"conditions for shipyard.plist entry \"%@\" are not an array, ignoring.", shipKey);
+					shipyardConditions = nil;
+				}
 			
-			if (shipyardConditions != nil)
-			{
-				[mutableShipyard setObject:shipyardConditions forKey:@"conditions"];
-			}
-			else
-			{
-				[mutableShipyard removeObjectForKey:@"conditions"];
-			}
+				if (shipyardConditions != nil)
+				{
+					[mutableShipyard setObject:shipyardConditions forKey:@"conditions"];
+				}
+				else
+				{
+					[mutableShipyard removeObjectForKey:@"conditions"];
+				}
 			
-			[shipEntry setObject:mutableShipyard forKey:@"_oo_shipyard"];
+				[shipEntry setObject:mutableShipyard forKey:@"_oo_shipyard"];
+			}
 		}
 	}
-	
+
+	[[OOCacheManager sharedCache] setObject:conditionScripts forKey:@"ship conditions" inCache:@"condition scripts"];
+	[conditionScripts release];
+
 	return YES;
 }
 
@@ -960,7 +1172,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	NSString				*modelName = nil;
 	OOMesh					*mesh = nil;
 	NSAutoreleasePool		*pool = nil;
-	OOUInteger				i = 0, count;
+	NSUInteger				i = 0, count;
 	
 	count = [ioData count];
 	
@@ -1018,6 +1230,10 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	*/
 	
 	rolesAndWeights = OOParseRolesFromString(roles);
+  // add default [shipKey] role
+	NSMutableDictionary *mutable = [NSMutableDictionary dictionaryWithDictionary:rolesAndWeights];
+	[mutable setObject:[NSNumber numberWithFloat:1.0] forKey:[[[NSString alloc] initWithFormat:@"[%@]",shipKey] autorelease]];
+	rolesAndWeights = mutable;
 	
 	id thargonValue = [rolesAndWeights objectForKey:@"thargon"];
 	if (thargonValue != nil && [rolesAndWeights objectForKey:@"EQ_THARGON"] == nil)
@@ -1054,11 +1270,14 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	if ([declaration isKindOfClass:[NSString class]])
 	{
 		// Update old-style string-based declaration.
-		result = [self translateOldStyleSubentityDeclaration:declaration
-													 forShip:shipKey
-													shipData:shipData
-												  fatalError:outFatalError];
-		
+		OOStandardsDeprecated([NSString stringWithFormat:@"Old style sub-entity declarations are deprecated in %@",shipKey]);
+		if (!OOEnforceStandards())
+		{
+			result = [self translateOldStyleSubentityDeclaration:declaration
+														 forShip:shipKey
+														shipData:shipData
+													  fatalError:outFatalError];
+		}
 		if (result != nil)
 		{
 			// Ensure internal translation made sense, and clean up a bit.
@@ -1106,12 +1325,12 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	{
 		if (!isFlasher)
 		{
-			OOLogERR(@"shipData.load.error.badSubentity", @"the shipdata.plist entry \"%@\" has a broken subentity definition \"%@\" (should have 8 tokens, has %u).", shipKey, subentityKey, [tokens count]);
+			OOLogERR(@"shipData.load.error.badSubentity", @"the shipdata.plist entry \"%@\" has a broken subentity definition \"%@\" (should have 8 tokens, has %lu).", shipKey, subentityKey, [tokens count]);
 			*outFatalError = YES;
 		}
 		else
 		{
-			OOLogWARN(@"shipData.load.warning.badFlasher", @"the shipdata.plist entry \"%@\" has a broken flasher definition (should have 8 tokens, has %u). This flasher will be ignored.", shipKey, [tokens count]);
+			OOLogWARN(@"shipData.load.warning.badFlasher", @"the shipdata.plist entry \"%@\" has a broken flasher definition (should have 8 tokens, has %lu). This flasher will be ignored.", shipKey, [tokens count]);
 		}
 		return nil;
 	}
@@ -1251,19 +1470,21 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	Vector					position = kZeroVector;
 	NSArray					*colors = nil;
 	id						colorDesc = nil;
-	float					size, frequency, phase;
+	float					size, frequency, phase, brightfraction;
 	BOOL					initiallyOn;
 	
+#define kDefaultFlasherColor @"redColor"
+	
 	// "Validate" is really "clean up", since all values have defaults.
 	colors = [declaration oo_arrayForKey:@"colors"];
 	if ([colors count] == 0)
 	{
 		colorDesc = [declaration objectForKey:@"color"];
-		if (colorDesc == nil) colorDesc = @"redColor";
+		if (colorDesc == nil) colorDesc = kDefaultFlasherColor;
 		if ([colorDesc isKindOfClass:[NSArray class]])
 		{
 			// an easy made error is adding an array to "color" instead of "colors"
-			OOLogWARN(@"shipData.load.warning.flasher.badColor", @"Changing flasher for ship %@ from a color to a colors definition.", shipKey);
+			OOLogWARN(@"shipData.load.warning.flasher.badColor", @"changing flasher for ship %@ from a color to a colors definition.", shipKey);
 			colors = colorDesc;
 		}
 		else
@@ -1272,6 +1493,27 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 		}
 	}
 	
+	// Validate colours.
+	NSMutableArray *validColors = [NSMutableArray arrayWithCapacity:[colors count]];
+	foreach (colorDesc, colors)
+	{
+		OOColor *color = [OOColor colorWithDescription:colorDesc];
+		if (color != nil)
+		{
+			[validColors addObject:[color normalizedArray]];
+		}
+		else
+		{
+			OOLogWARN(@"shipdata.load.warning.flasher.badColor", @"skipping invalid colour specifier for flasher for ship %@.", shipKey);
+		}
+	}
+	// Ensure there's at least one.
+	if ([validColors count] == 0)
+	{
+		[validColors addObject:kDefaultFlasherColor];
+	}
+	colors = validColors;
+	
 	position = [declaration oo_vectorForKey:@"position"];
 	
 	size = [declaration oo_floatForKey:@"size" defaultValue:8.0];
@@ -1281,19 +1523,26 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 		OOLogWARN(@"shipData.load.warning.flasher.badSize", @"skipping flasher of invalid size %g for ship %@.", size, shipKey);
 		return nil;
 	}
+
+	brightfraction = [declaration oo_floatForKey:@"bright_fraction" defaultValue:0.5];
+	if (brightfraction < 0.0 || brightfraction > 1.0)
+	{
+		OOLogWARN(@"shipData.load.warning.flasher.badFraction", @"skipping flasher of invalid bright fraction %g for ship %@.", brightfraction, shipKey);
+		return nil;
+	}
 	
 	frequency = [declaration oo_floatForKey:@"frequency" defaultValue:2.0];
 	phase = [declaration oo_floatForKey:@"phase" defaultValue:0.0];
-	
 	initiallyOn = [declaration oo_boolForKey:@"initially_on" defaultValue:YES];
 	
-	result = [NSMutableDictionary dictionaryWithCapacity:7];
+	result = [NSMutableDictionary dictionaryWithCapacity:8];
 	[result setObject:@"flasher" forKey:@"type"];
 	[result setObject:colors forKey:@"colors"];
 	[result oo_setVector:position forKey:@"position"];
 	[result setObject:[NSNumber numberWithFloat:size] forKey:@"size"];
 	[result setObject:[NSNumber numberWithFloat:frequency] forKey:@"frequency"];
 	if (phase != 0)  [result setObject:[NSNumber numberWithFloat:phase] forKey:@"phase"];
+	[result setObject:[NSNumber numberWithFloat:brightfraction] forKey:@"bright_fraction"];
 	[result setObject:[NSNumber numberWithBool:initiallyOn] forKey:@"initially_on"];
 	
 	return [[result copy] autorelease];
@@ -1310,7 +1559,10 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	Quaternion				orientation = kIdentityQuaternion;
 	BOOL					isTurret;
 	BOOL					isDock = NO;
-	float					fireRate = 0.5f;
+	float					fireRate = -1.0f; // out of range constants
+	float					weaponRange = -1.0f;
+	float					weaponEnergy = -1.0f;
+	NSDictionary			*scriptInfo = nil;
 	
 	subentityKey = [declaration objectForKey:@"subentity_key"];
 	if (subentityKey == nil)
@@ -1323,12 +1575,26 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	isTurret = [[declaration oo_stringForKey:@"type"] isEqualToString:@"ball_turret"];
 	if (isTurret)
 	{
-		fireRate = [declaration oo_floatForKey:@"fire_rate" defaultValue:0.5f];
-		if (fireRate < 0.25f)
+		fireRate = [declaration oo_floatForKey:@"fire_rate" defaultValue:-1.0f];
+		if (fireRate < 0.25f && fireRate >= 0.0f)
 		{
 			OOLogWARN(@"shipData.load.warning.turret.badFireRate", @"ball turret fire rate of %g for subenitity of ship %@ is invalid, using 0.25.", fireRate, shipKey);
 			fireRate = 0.25f;
 		}
+		weaponRange = [declaration oo_floatForKey:@"weapon_range" defaultValue:-1.0f];
+		if (weaponRange > 7500.0f)
+		{
+			OOLogWARN(@"shipData.load.warning.turret.badWeaponRange", @"ball turret weapon range of %g for subenitity of ship %@ is too high, using 7500.", weaponRange, shipKey);
+			weaponRange = 7500.0f; // range of primary plasma canon.
+		}
+
+		weaponEnergy = [declaration oo_floatForKey:@"weapon_energy" defaultValue:-1.0f];
+		if (weaponEnergy > 100.0f)
+			
+		{
+			OOLogWARN(@"shipData.load.warning.turret.badWeaponEnergy", @"ball turret weapon energy of %g for subenitity of ship %@ is too high, using 100.", weaponEnergy, shipKey);
+			weaponEnergy = 100.0f;
+		}
 	}
 	else
 	{
@@ -1339,13 +1605,42 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 	orientation = [declaration oo_quaternionForKey:@"orientation"];
 	quaternion_normalize(&orientation);
 	
-	result = [NSMutableDictionary dictionaryWithCapacity:5];
+	scriptInfo = [declaration oo_dictionaryForKey:@"script_info"];
+	
+	result = [NSMutableDictionary dictionaryWithCapacity:10];
 	[result setObject:isTurret ? @"ball_turret" : @"standard" forKey:@"type"];
 	[result setObject:subentityKey forKey:@"subentity_key"];
 	[result oo_setVector:position forKey:@"position"];
 	[result oo_setQuaternion:orientation forKey:@"orientation"];
-	if (isDock)  [result oo_setBool:YES forKey:@"is_dock"];
-	if (isTurret)  [result oo_setFloat:fireRate forKey:@"fire_rate"];
+	if (isDock) 
+	{
+		[result oo_setBool:YES forKey:@"is_dock"];
+
+		NSString* docklabel = [declaration oo_stringForKey:@"dock_label" defaultValue:@"the docking bay"];
+		[result setObject:docklabel forKey:@"dock_label"];
+
+		BOOL dockable = [declaration oo_boolForKey:@"allow_docking" defaultValue:YES];
+		BOOL playerdockable = [declaration oo_boolForKey:@"disallowed_docking_collides" defaultValue:NO];
+		BOOL undockable = [declaration oo_boolForKey:@"allow_launching" defaultValue:YES];
+
+		[result oo_setBool:dockable forKey:@"allow_docking"];
+		[result oo_setBool:playerdockable forKey:@"disallowed_docking_collides"];
+		[result oo_setBool:undockable forKey:@"allow_launching"];
+
+	}
+
+	if (isTurret)
+	{
+		// default constants are defined and set in shipEntity
+		if (fireRate > 0) [result oo_setFloat:fireRate forKey:@"fire_rate"];
+		if (weaponRange >= 0) [result oo_setFloat:weaponRange forKey:@"weapon_range"];
+		if (weaponEnergy >= 0) [result oo_setFloat:weaponEnergy forKey:@"weapon_energy"];
+	}
+	
+	if (scriptInfo != nil)
+	{
+		[result setObject:scriptInfo forKey:@"script_info"];
+	}
 	
 	return [[result copy] autorelease];
 }
@@ -1365,6 +1660,13 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 		if ([[ScanTokensFromString(action) objectAtIndex:0] isEqualToString:@"initialiseTurret"])  return YES;
 	}
 	
+	if ([shipKey isEqualToString:@"ballturret"])
+	{
+		// compatibility for OXPs using old subentity declarations and the
+		// core turret entity
+		return YES;
+	}
+
 	return NO;
 }
 
@@ -1384,6 +1686,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 {
 	if (sSingleton == nil)
 	{
+		OOLog(@"shipData.load.begin", @"Loading ship data.");
 		sSingleton = [super allocWithZone:inZone];
 		return sSingleton;
 	}
@@ -1403,7 +1706,7 @@ static NSString * const	kDefaultDemoShip = @"coriolis-station";
 }
 
 
-- (OOUInteger) retainCount
+- (NSUInteger) retainCount
 {
 	return UINT_MAX;
 }
@@ -1497,3 +1800,15 @@ static void GatherStringAddrs(id object, NSMutableSet *strings, NSString *contex
 		GatherStringAddrsDict(object, strings, context);
 	}
 }
+
+
+static NSComparisonResult SortDemoShipsByName (id a, id b, void* context)
+{
+	return [[a oo_stringForKey:@"name"] compare:[b oo_stringForKey:@"name"]];
+}
+
+
+static NSComparisonResult SortDemoCategoriesByName (id a, id b, void* context)
+{
+	return [OOShipLibraryCategoryPlural([[a oo_dictionaryAtIndex:0] oo_stringForKey:@"class"]) compare:OOShipLibraryCategoryPlural([[b oo_dictionaryAtIndex:0] oo_stringForKey:@"class"])];
+}
