@@ -4,7 +4,7 @@ OOSpatialReference.m
 
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -24,16 +24,3 @@ MA 02110-1301, USA.
 */
 
 #import "OOSpatialReference.h"
-
-
-@implementation Entity (OOSpatialReference)
-
-- (OOMatrix) transformationMatrix
-{
-	OOMatrix		rotate = [self rotationMatrix];
-	OOMatrix		translate = OOMatrixForTranslation([self position]);
-	
-	return OOMatrixMultiply(translate, rotate);
-}
-
-@end
