@@ -3,7 +3,7 @@
 OOPListSchemaVerifier.m
 
 
-Copyright (C) 2007-2011 Jens Ayton
+Copyright (C) 2007-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -121,9 +121,9 @@ OOINLINE BackLinkChain BackLink(BackLinkChain *link, id element)
 	return result;
 }
 
-OOINLINE BackLinkChain BackLinkIndex(BackLinkChain *link, unsigned index)
+OOINLINE BackLinkChain BackLinkIndex(BackLinkChain *link, NSUInteger index)
 {
-	BackLinkChain result = { link, [NSNumber numberWithInt:index] };
+	BackLinkChain result = { link, [NSNumber numberWithInteger:index] };
 	return result;
 }
 
@@ -195,8 +195,8 @@ static BOOL IsFailureAlreadyReportedError(NSError *error);
 
 #define VERIFY_PROTO(T) static NSError *Verify_##T(OOPListSchemaVerifier *verifier, id value, NSDictionary *params, id rootPList, NSString *name, BackLinkChain keyPath, BOOL tentative, BOOL *outStop)
 VERIFY_PROTO(String);
-VERIFY_PROTO(Array) GCC_ATTR((noinline));		// Inlining suppressed to avoid "variable may be clobbered" warning when building for Mac OS X/x86.
-VERIFY_PROTO(Dictionary) GCC_ATTR((noinline));	// Ditto.
+VERIFY_PROTO(Array);
+VERIFY_PROTO(Dictionary);
 VERIFY_PROTO(Integer);
 VERIFY_PROTO(PositiveInteger);
 VERIFY_PROTO(Float);
@@ -332,7 +332,8 @@ VERIFY_PROTO(DelegatedType);
 	
 	if ([_delegate respondsToSelector:@selector(verifier:withPropertyList:named:testProperty:atPath:againstType:error:)])
 	{
-		NS_DURING
+		@try
+		{
 			result = [_delegate verifier:self
 						withPropertyList:rootPList
 								   named:name
@@ -340,11 +341,13 @@ VERIFY_PROTO(DelegatedType);
 								  atPath:KeyPathToArray(keyPath)
 							 againstType:typeKey
 								   error:&error];
-		NS_HANDLER
-			OOLog(@"plistVerifier.delegateException", @"Property list schema verifier: delegate threw exception (%@) in -verifier:withPropertyList:named:testProperty:atPath:againstType: for type \"%@\" at %@ in %@ -- treating as failure.", [localException name], typeKey,KeyPathToString(keyPath), name);
+		}
+		@catch (NSException *exception)
+		{
+			OOLog(@"plistVerifier.delegateException", @"Property list schema verifier: delegate threw exception (%@) in -verifier:withPropertyList:named:testProperty:atPath:againstType: for type \"%@\" at %@ in %@ -- treating as failure.", [exception name], typeKey,KeyPathToString(keyPath), name);
 			result = NO;
 			error = nil;
-		NS_ENDHANDLER
+		}
 		
 		if (outError != NULL)
 		{
@@ -380,21 +383,24 @@ VERIFY_PROTO(DelegatedType);
 	
 	if ([_delegate respondsToSelector:@selector(verifier:withPropertyList:named:failedForProperty:withError:expectedType:)])
 	{
-		NS_DURING
+		@try
+		{
 			result = [_delegate verifier:self
 						withPropertyList:rootPList
 								   named:name
 					   failedForProperty:subPList
 							   withError:error
 							expectedType:localSchema];
-		NS_HANDLER
-			OOLog(@"plistVerifier.delegateException", @"Property list schema verifier: delegate threw exception (%@) in -verifier:withPropertyList:named:failedForProperty:atPath:expectedType: at %@ in %@ -- stopping.", [localException name], [error plistKeyPathDescription], name);
+		}
+		@catch (NSException *exception)
+		{
+			OOLog(@"plistVerifier.delegateException", @"Property list schema verifier: delegate threw exception (%@) in -verifier:withPropertyList:named:failedForProperty:atPath:expectedType: at %@ in %@ -- stopping.", [exception name], [error plistKeyPathDescription], name);
 			result = NO;
-		NS_ENDHANDLER
+		}
 	}
 	else
 	{
-		OOLog(@"plistVerifier.failed", @"Verification of property list \"%@\" failed at %@: %@", name, [error plistKeyPathDescription], [error localizedDescription]);
+		OOLog(@"plistVerifier.failed", @"Verification of property list \"%@\" failed at %@: %@", name, [error plistKeyPathDescription], [error localizedFailureReason]);
 		result = NO;
 	}
 	return result;
@@ -421,7 +427,8 @@ VERIFY_PROTO(DelegatedType);
 	
 	DebugDumpPushIndent();
 	
-	NS_DURING
+	@try
+	{
 		DebugDumpIndent();
 		
 		resolvedSpecifier = [self resolveSchemaType:subSchema atPath:keyPath error:&error];
@@ -450,9 +457,11 @@ VERIFY_PROTO(DelegatedType);
 				// resolveSchemaType:... or StringToSchemaType() should have provided an error.
 				*outStop = YES;
 		}
-	NS_HANDLER
-		error = Error(kPListErrorInternal, (BackLinkChain *)&keyPath, @"Uncaught exception %@: %@ in plist verifier for \"%@\" at %@.", [localException name], [localException reason], name, KeyPathToString(keyPath));
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		error = Error(kPListErrorInternal, (BackLinkChain *)&keyPath, @"Uncaught exception %@: %@ in plist verifier for \"%@\" at %@.", [exception name], [exception reason], name, KeyPathToString(keyPath));
+	}
 	
 	DebugDumpPopIndent();
 	
@@ -761,7 +770,7 @@ static NSString *ArrayForErrorReport(NSArray *array)
 {
 	NSString				*result = nil;
 	NSString				*string = nil;
-	unsigned				i, count;
+	NSUInteger				i, count;
 	NSAutoreleasePool		*pool = nil;
 	
 	count = [array count];
@@ -830,8 +839,8 @@ static NSError *Verify_String(OOPListSchemaVerifier *verifier, id value, NSDicti
 {
 	NSString			*filteredString = nil;
 	id					testValue = nil;
-	unsigned			length;
-	unsigned			lengthConstraint;
+	NSUInteger			length;
+	NSUInteger			lengthConstraint;
 	NSError				*error = nil;
 	
 	REQUIRE_TYPE(NSString, @"string");
@@ -875,13 +884,13 @@ static NSError *Verify_String(OOPListSchemaVerifier *verifier, id value, NSDicti
 	
 	// Apply length bounds.
 	length = [filteredString length];
-	lengthConstraint = [params oo_unsignedIntForKey:@"minLength"];
+	lengthConstraint = [params oo_unsignedIntegerForKey:@"minLength"];
 	if (length < lengthConstraint)
 	{
 		return  Error(kPListErrorMinimumConstraintNotMet, &keyPath, @"String \"%@\" is too short (%u bytes, minimum is %u).", StringForErrorReport(filteredString), length, lengthConstraint);
 	}
 	
-	lengthConstraint = [params oo_unsignedIntForKey:@"maxLength" defaultValue:UINT_MAX];
+	lengthConstraint = [params oo_unsignedIntegerForKey:@"maxLength" defaultValue:NSUIntegerMax];
 	if (lengthConstraint < length)
 	{
 		return  Error(kPListErrorMaximumConstraintNotMet, &keyPath, @"String \"%@\" is too long (%u bytes, maximum is %u).", StringForErrorReport(filteredString), length, lengthConstraint);
@@ -896,9 +905,9 @@ static NSError *Verify_Array(OOPListSchemaVerifier *verifier, id value, NSDictio
 {
 	id						valueType = nil;
 	BOOL					OK = YES, stop = NO;
-	unsigned				i, count;
+	NSUInteger				i, count;
 	id						subProperty = nil;
-	unsigned				constraint;
+	NSUInteger				constraint;
 	
 	REQUIRE_TYPE(NSArray, @"array");
 	
@@ -906,13 +915,13 @@ static NSError *Verify_Array(OOPListSchemaVerifier *verifier, id value, NSDictio
 	
 	// Apply count bounds.
 	count = [value count];
-	constraint = [params oo_unsignedIntForKey:@"minCount" defaultValue:0];
+	constraint = [params oo_unsignedIntegerForKey:@"minCount" defaultValue:0];
 	if (count < constraint)
 	{
 		return  Error(kPListErrorMinimumConstraintNotMet, &keyPath, @"Array has too few members (%u, minimum is %u).", count, constraint);
 	}
 	
-	constraint = [params oo_unsignedIntForKey:@"maxCount" defaultValue:UINT_MAX];
+	constraint = [params oo_unsignedIntegerForKey:@"maxCount" defaultValue:NSUIntegerMax];
 	if (constraint < count)
 	{
 		return  Error(kPListErrorMaximumConstraintNotMet, &keyPath, @"Array has too many members (%u, maximum is %u).", count, constraint);
@@ -961,7 +970,7 @@ static NSError *Verify_Dictionary(OOPListSchemaVerifier *verifier, id value, NSD
 	BOOL					allowOthers;
 	NSMutableSet			*requiredKeys = nil;
 	NSArray					*requiredKeyList = nil;
-	unsigned				count, constraint;
+	NSUInteger				count, constraint;
 	
 	REQUIRE_TYPE(NSDictionary, @"dictionary");
 	
@@ -969,12 +978,12 @@ static NSError *Verify_Dictionary(OOPListSchemaVerifier *verifier, id value, NSD
 	
 	// Apply count bounds.
 	count = [value count];
-	constraint = [params oo_unsignedIntForKey:@"minCount" defaultValue:0];
+	constraint = [params oo_unsignedIntegerForKey:@"minCount" defaultValue:0];
 	if (count < constraint)
 	{
 		return  Error(kPListErrorMinimumConstraintNotMet, &keyPath, @"Dictionary has too few pairs (%u, minimum is %u).", count, constraint);
 	}
-	constraint = [params oo_unsignedIntForKey:@"maxCount" defaultValue:UINT_MAX];
+	constraint = [params oo_unsignedIntegerForKey:@"maxCount" defaultValue:NSUIntegerMax];
 	if (constraint < count)
 	{
 		return  Error(kPListErrorMaximumConstraintNotMet, &keyPath, @"Dictionary has too manu pairs (%u, maximum is %u).", count, constraint);
@@ -1448,7 +1457,7 @@ static NSError *ErrorWithDictionaryAndArguments(OOPListSchemaVerifierErrorCode e
 	message = [[NSString alloc] initWithFormat:format arguments:arguments];
 	
 	userInfo = [NSMutableDictionary dictionaryWithDictionary:dict];
-	[userInfo setObject:message forKey:NSLocalizedDescriptionKey];
+	[userInfo setObject:message forKey:NSLocalizedFailureReasonErrorKey];
 	if (keyPath != NULL)
 	{
 		[userInfo setObject:KeyPathToArray(*keyPath) forKey:kPListKeyPathErrorKey];
