@@ -3,7 +3,7 @@
 OOJSEngineTimeManagement.h
 
 
-Copyright (C) 2010-2011 Jens Ayton
+Copyright (C) 2010-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,8 +31,17 @@ SOFTWARE.
 #import "OOJSScript.h"
 #import "OOCollectionExtractors.h"
 #import "OOLoggingExtended.h"
+
+#if OOLITE_LINUX
+// Workaround for clang/glibc incompatibility.
+#define __block __glibc_block
+#endif
 #include <unistd.h>
 
+#if OOLITE_LINUX
+#undef __block
+#endif
+
 
 #if OO_DEBUG
 #define OOJS_DEBUG_LIMITER	1
@@ -75,6 +84,9 @@ void OOJSStartTimeLimiterWithTimeLimit_(OOTimeDelta limit, const char *file, uns
 void OOJSStartTimeLimiterWithTimeLimit(OOTimeDelta limit)
 #endif
 {
+#if OOJS_DEBUG_LIMITER
+	OOLog(@"script.javaScript.timeLimit.debug",@"Limiter starting: %u => %u",sLimiterStartDepth,sLimiterStartDepth+1);
+#endif
 	if (sLimiterStartDepth++ == 0)
 	{
 		if (limit <= 0.0)  limit = OOJS_TIME_LIMIT;
@@ -107,6 +119,11 @@ void OOJSStopTimeLimiter(void)
 	
 	sLastStoppedFile = file;
 	sLastStoppedLine = line;
+
+#if OOJS_DEBUG_LIMITER
+	OOLog(@"script.javaScript.timeLimit.debug",@"Limiter ending: %u <= %u",sLimiterStartDepth-1,sLimiterStartDepth);
+#endif
+
 #endif
 	
 	if (--sLimiterStartDepth == 0)  sLimiterTimeLimit = 0.0;
@@ -621,7 +638,7 @@ static void UpdateProfileForFrame(OOHighResTimeValue now, OOJSProfileStackFrame
 							   [self profilerOverhead] * 1000.0];
 	
 	NSArray *profileEntries = [self profileEntries];
-	OOUInteger i, count = [profileEntries count];
+	NSUInteger i, count = [profileEntries count];
 	if (count != 0)
 	{
 		[result appendString:@"\n                                                        NAME  T  COUNT    TOTAL     SELF  TOTAL%   SELF%  SELFMAX"];
@@ -834,8 +851,8 @@ static void UpdateProfileForFrame(OOHighResTimeValue now, OOJSProfileStackFrame
 	_hitCount++;
 	_totalTimeSum += totalTime;
 	_selfTimeSum += selfTime;
-	_totalTimeMax = OOMax_f(_totalTimeMax, totalTime);
-	_selfTimeMax = OOMax_f(_selfTimeMax, selfTime);
+	_totalTimeMax = fmax(_totalTimeMax, totalTime);
+	_selfTimeMax = fmax(_selfTimeMax, selfTime);
 }
 
 
@@ -880,7 +897,7 @@ static void UpdateProfileForFrame(OOHighResTimeValue now, OOJSProfileStackFrame
 }
 
 
-- (OOUInteger) hitCount
+- (NSUInteger) hitCount
 {
 	return _hitCount;
 }
@@ -976,7 +993,7 @@ static void UpdateProfileForFrame(OOHighResTimeValue now, OOJSProfileStackFrame
 {
 	return [NSDictionary dictionaryWithObjectsAndKeys:
 			_function, @"name",
-			[NSNumber numberWithUnsignedInt:[self hitCount]], @"hitCount",
+			[NSNumber numberWithUnsignedInteger:[self hitCount]], @"hitCount",
 			[NSNumber numberWithDouble:[self totalTimeSum]], @"totalTimeSum",
 			[NSNumber numberWithDouble:[self selfTimeSum]], @"selfTimeSum",
 			[NSNumber numberWithDouble:[self totalTimeAverage]], @"totalTimeAverage",
