@@ -3,7 +3,7 @@
 OOJSQuaternion.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -57,6 +57,7 @@ static JSBool QuaternionRotateX(JSContext *context, uintN argc, jsval *vp);
 static JSBool QuaternionRotateY(JSContext *context, uintN argc, jsval *vp);
 static JSBool QuaternionRotateZ(JSContext *context, uintN argc, jsval *vp);
 static JSBool QuaternionNormalize(JSContext *context, uintN argc, jsval *vp);
+static JSBool QuaternionConjugate(JSContext *context, uintN argc, jsval *vp);
 static JSBool QuaternionVectorForward(JSContext *context, uintN argc, jsval *vp);
 static JSBool QuaternionVectorUp(JSContext *context, uintN argc, jsval *vp);
 static JSBool QuaternionVectorRight(JSContext *context, uintN argc, jsval *vp);
@@ -112,6 +113,7 @@ static JSFunctionSpec sQuaternionMethods[] =
 	{ "dot",					QuaternionDot,				1, },
 	{ "multiply",				QuaternionMultiply,			1, },
 	{ "normalize",				QuaternionNormalize,		0, },
+	{ "conjugate",				QuaternionConjugate,		0, },
 	{ "rotate",					QuaternionRotate,			2, },
 	{ "rotateX",				QuaternionRotateX,			1, },
 	{ "rotateY",				QuaternionRotateY,			1, },
@@ -196,12 +198,12 @@ BOOL JSValueToQuaternion(JSContext *context, jsval value, Quaternion *outQuatern
 
 typedef struct
 {
-	OOUInteger			quatCount;
-	OOUInteger			entityCount;
-	OOUInteger			arrayCount;
-	OOUInteger			protoCount;
-	OOUInteger			nullCount;
-	OOUInteger			failCount;
+	NSUInteger			quatCount;
+	NSUInteger			entityCount;
+	NSUInteger			arrayCount;
+	NSUInteger			protoCount;
+	NSUInteger			nullCount;
+	NSUInteger			failCount;
 } QuaternionStatistics;
 static QuaternionStatistics sQuaternionConversionStats;
 
@@ -215,7 +217,7 @@ static QuaternionStatistics sQuaternionConversionStats;
 {
 	QuaternionStatistics *stats = &sQuaternionConversionStats;
 	
-	OOUInteger sum = stats->quatCount + stats->entityCount + stats->arrayCount + stats->protoCount;
+	NSUInteger sum = stats->quatCount + stats->entityCount + stats->arrayCount + stats->protoCount;
 	double convFac = 100.0 / sum;
 	
 	return [NSString stringWithFormat:
@@ -538,7 +540,7 @@ static JSBool QuaternionConstruct(JSContext *context, uintN argc, jsval *vp)
 {
 	OOJS_PROFILE_ENTER
 	
-	Quaternion				quaternion = kZeroQuaternion;
+	Quaternion				quaternion = kIdentityQuaternion;
 	Quaternion				*private = NULL;
 	JSObject				*this = NULL;
 	
@@ -650,7 +652,7 @@ static JSBool QuaternionRotate(JSContext *context, uintN argc, jsval *vp)
 	OOJS_PROFILE_ENTER
 	
 	Quaternion				thisq;
-	Vector					axis;
+	HPVector					axis;
 	double					angle;
 	uintN					consumed;
 	jsval					*argv = OOJS_ARGV;
@@ -662,7 +664,7 @@ static JSBool QuaternionRotate(JSContext *context, uintN argc, jsval *vp)
 	if (argc > 0)
 	{
 		if (EXPECT_NOT(!OOJSArgumentListGetNumber(context, @"Quaternion", @"rotate", argc, argv, &angle, NULL)))  return NO;
-		quaternion_rotate_about_axis(&thisq, axis, angle);
+		quaternion_rotate_about_axis(&thisq, HPVectorToVector(axis), angle);
 	}
 	// Else no angle specified, so don't rotate and pass value through unchanged.
 	
@@ -746,6 +748,23 @@ static JSBool QuaternionNormalize(JSContext *context, uintN argc, jsval *vp)
 }
 
 
+// conjugate() : Quaternion
+static JSBool QuaternionConjugate(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_PROFILE_ENTER
+	
+		Quaternion				quat, result;
+	
+	if (EXPECT_NOT(!GetThisQuaternion(context, OOJS_THIS, &quat, @"conjugate"))) return NO;
+	
+	result = quaternion_conjugate(quat);
+	
+	OOJS_RETURN_QUATERNION(result);
+	
+	OOJS_PROFILE_EXIT
+}
+
+
 // vectorForward() : Vector
 static JSBool QuaternionVectorForward(JSContext *context, uintN argc, jsval *vp)
 {
