@@ -4,7 +4,7 @@
  
  
  Oolite
- Copyright (C) 2004-2011 Giles C Williams and contributors
+ Copyright (C) 2004-2013 Giles C Williams and contributors
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 #import "OOJavaScriptEngine.h"
 
 #import "OOJSVector.h"
+#import "OOJSQuaternion.h"
 #import "OOJSEntity.h"
 #import "OOJSPlayer.h"
 #import "Universe.h"
@@ -39,7 +40,8 @@
 #import "OOConstToJSString.h"
 #import "OOEntityFilterPredicate.h"
 #import "OOFilteringEnumerator.h"
-
+#import "OOJSPopulatorDefinition.h"
+#import "OODebugStandards.h"
 
 static JSObject *sSystemPrototype;
 
@@ -69,10 +71,14 @@ static JSBool SystemShipsWithRole(JSContext *context, uintN argc, jsval *vp);
 static JSBool SystemEntitiesWithScanClass(JSContext *context, uintN argc, jsval *vp);
 static JSBool SystemFilteredEntities(JSContext *context, uintN argc, jsval *vp);
 
+static JSBool SystemLocationFromCode(JSContext *context, uintN argc, jsval *vp);
 static JSBool SystemAddShips(JSContext *context, uintN argc, jsval *vp);
 static JSBool SystemAddGroup(JSContext *context, uintN argc, jsval *vp);
 static JSBool SystemAddShipsToRoute(JSContext *context, uintN argc, jsval *vp);
 static JSBool SystemAddGroupToRoute(JSContext *context, uintN argc, jsval *vp);
+static JSBool SystemAddVisualEffect(JSContext *context, uintN argc, jsval *vp);
+static JSBool SystemSetPopulator(JSContext *context, uintN argc, jsval *vp);
+static JSBool SystemSetWaypoint(JSContext *context, uintN argc, jsval *vp);
 
 static JSBool SystemLegacyAddShips(JSContext *context, uintN argc, jsval *vp);
 static JSBool SystemLegacyAddSystemShips(JSContext *context, uintN argc, jsval *vp);
@@ -105,7 +111,11 @@ static JSClass sSystemClass =
 enum
 {
 	// Property IDs
+	kSystem_allDemoShips,				// demo ships, array of Ship, read-only
 	kSystem_allShips,				// ships in system, array of Ship, read-only
+	kSystem_allVisualEffects,				// VEs in system, array of VEs, read-only
+	kSystem_ambientLevel,			// ambient light level, float, read/write
+	kSystem_breakPattern, // witchspace break pattern shown
 	kSystem_description,			// description, string, read/write
 	kSystem_economy,				// economy ID, integer, read/write
 	kSystem_economyDescription,		// economy ID description, string, read-only
@@ -120,19 +130,28 @@ enum
 	kSystem_name,					// name, string, read/write
 	kSystem_planets,				// planets in system, array of Planet, read-only
 	kSystem_population,				// population, integer, read/write
+	kSystem_populatorSettings,				// populator settings, dictionary, read-only
 	kSystem_productivity,			// productivity, integer, read/write
 	kSystem_pseudoRandom100,		// constant-per-system pseudorandom number in [0..100), integer, read-only
 	kSystem_pseudoRandom256,		// constant-per-system pseudorandom number in [0..256), integer, read-only
 	kSystem_pseudoRandomNumber,		// constant-per-system pseudorandom number in [0..1), double, read-only
 	kSystem_sun,					// system's sun, Planet, read-only
+	kSystem_stations,     // list of dockable entities, read-only
 	kSystem_techLevel,				// tech level ID, integer, read/write
+	kSystem_waypoints,     // dictionary of current player waypoints, read-only
+	kSystem_wormholes,     // list of active entry wormholes, read-only
+
 };
 
 
 static JSPropertySpec sSystemProperties[] =
 {
 	// JS name					ID								flags
+	{ "allDemoShips",			kSystem_allDemoShips,			OOJS_PROP_READONLY_CB },
 	{ "allShips",				kSystem_allShips,				OOJS_PROP_READONLY_CB },
+	{ "allVisualEffects",	 kSystem_allVisualEffects,		OOJS_PROP_READONLY_CB },
+	{ "ambientLevel",			kSystem_ambientLevel,			OOJS_PROP_READWRITE_CB },
+	{ "breakPattern",			kSystem_breakPattern,			OOJS_PROP_READWRITE_CB },
 	{ "description",			kSystem_description,			OOJS_PROP_READWRITE_CB },
 	{ "economy",				kSystem_economy,				OOJS_PROP_READWRITE_CB },
 	{ "economyDescription",		kSystem_economyDescription,		OOJS_PROP_READONLY_CB },
@@ -147,12 +166,16 @@ static JSPropertySpec sSystemProperties[] =
 	{ "name",					kSystem_name,					OOJS_PROP_READWRITE_CB },
 	{ "planets",				kSystem_planets,				OOJS_PROP_READONLY_CB },
 	{ "population",				kSystem_population,				OOJS_PROP_READWRITE_CB },
+	{ "populatorSettings",				kSystem_populatorSettings,				OOJS_PROP_READONLY_CB },
 	{ "productivity",			kSystem_productivity,			OOJS_PROP_READWRITE_CB },
 	{ "pseudoRandom100",		kSystem_pseudoRandom100,		OOJS_PROP_READONLY_CB },
 	{ "pseudoRandom256",		kSystem_pseudoRandom256,		OOJS_PROP_READONLY_CB },
 	{ "pseudoRandomNumber",		kSystem_pseudoRandomNumber,		OOJS_PROP_READONLY_CB },
+	{ "stations",					kSystem_stations,					OOJS_PROP_READONLY_CB },
 	{ "sun",					kSystem_sun,					OOJS_PROP_READONLY_CB },
 	{ "techLevel",				kSystem_techLevel,				OOJS_PROP_READWRITE_CB },
+	{ "waypoints",					kSystem_waypoints,					OOJS_PROP_READONLY_CB },
+	{ "wormholes",					kSystem_wormholes,					OOJS_PROP_READONLY_CB },
 	{ 0 }
 };
 
@@ -167,13 +190,17 @@ static JSFunctionSpec sSystemMethods[] =
 	{ "addPlanet",						SystemAddPlanet,					1 },
 	{ "addShips",						SystemAddShips,						3 },
 	{ "addShipsToRoute",				SystemAddShipsToRoute,				2 },
+	{ "addVisualEffect",						SystemAddVisualEffect,						2 },
 	{ "countEntitiesWithScanClass",		SystemCountEntitiesWithScanClass,	1 },
 	{ "countShipsWithPrimaryRole",		SystemCountShipsWithPrimaryRole,	1 },
 	{ "countShipsWithRole",				SystemCountShipsWithRole,			1 },
 	{ "entitiesWithScanClass",			SystemEntitiesWithScanClass,		1 },
 	{ "filteredEntities",				SystemFilteredEntities,				2 },
+	{ "locationFromCode",				SystemLocationFromCode,				1 },
 	// scrambledPseudoRandomNumber is implemented in oolite-global-prefix.js
 	{ "sendAllShipsAway",				SystemSendAllShipsAway,				1 },
+	{ "setPopulator",					SystemSetPopulator,					2 },
+	{ "setWaypoint",					SystemSetWaypoint,					4 },
 	{ "shipsWithPrimaryRole",			SystemShipsWithPrimaryRole,			1 },
 	{ "shipsWithRole",					SystemShipsWithRole,				1 },
 	
@@ -249,12 +276,45 @@ static JSBool SystemGetProperty(JSContext *context, JSObject *this, jsid propID,
 			handled = YES;
 			break;
 			
+		case kSystem_stations:
+			result = [UNIVERSE stations];
+			handled = YES;
+			break;
+
+		case kSystem_waypoints:
+			result = [UNIVERSE currentWaypoints];
+			handled = YES;
+			break;
+
+		case kSystem_wormholes:
+			result = [UNIVERSE wormholes];
+			handled = YES;
+			break;
+
 		case kSystem_allShips:
 			OOJS_BEGIN_FULL_NATIVE(context)
 			result = [UNIVERSE findShipsMatchingPredicate:JSEntityIsJavaScriptSearchablePredicate parameter:NULL inRange:-1 ofEntity:nil];
 			OOJS_END_FULL_NATIVE
 			handled = YES;
 			break;
+
+		case kSystem_allDemoShips:
+			OOJS_BEGIN_FULL_NATIVE(context)
+			result = [UNIVERSE findShipsMatchingPredicate:JSEntityIsDemoShipPredicate parameter:NULL inRange:-1 ofEntity:nil];
+			OOJS_END_FULL_NATIVE
+			handled = YES;
+			break;
+
+
+		case kSystem_allVisualEffects:
+			OOJS_BEGIN_FULL_NATIVE(context)
+			result = [UNIVERSE findVisualEffectsMatchingPredicate:JSEntityIsJavaScriptSearchablePredicate parameter:NULL inRange:-1 ofEntity:nil];
+			OOJS_END_FULL_NATIVE
+			handled = YES;
+			break;
+			
+		case kSystem_ambientLevel:
+			return JS_NewNumberValue(context, [UNIVERSE ambientLightLevel], value);
 			
 		case kSystem_info:
 			*value = GetJSSystemInfoForSystem(context, [player currentGalaxyID], [player currentSystemID]);
@@ -270,60 +330,121 @@ static JSBool SystemGetProperty(JSContext *context, JSObject *this, jsid propID,
 		case kSystem_pseudoRandom256:
 			*value = INT_TO_JSVAL([player systemPseudoRandom256]);
 			return YES;
+
+		case kSystem_breakPattern:
+			*value = OOJSValueFromBOOL([UNIVERSE witchspaceBreakPattern]);
+			return YES;
+
+		case kSystem_populatorSettings:
+			*value = OOJSValueFromNativeObject(context, [UNIVERSE getPopulatorSettings]);
+			return YES;
 	}
 	
 	if (!handled)
 	{
 		// Handle cases which do require systemData.
-		systemData = [UNIVERSE generateSystemData:player->system_seed];
-		
-		switch (JSID_TO_INT(propID))
+		if (EXPECT (![UNIVERSE inInterstellarSpace]))
 		{
-			case kSystem_name:
-				result = [systemData objectForKey:KEY_NAME];
-				break;
-				
-			case kSystem_description:
-				result = [systemData objectForKey:KEY_DESCRIPTION];
-				break;
-				
-			case kSystem_inhabitantsDescription:
-				result = [systemData objectForKey:KEY_INHABITANTS];
-				break;
-				
-			case kSystem_government:
-				*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_GOVERNMENT]);
-				return YES;
-				
-			case kSystem_governmentDescription:
-				result = OODisplayStringFromGovernmentID([systemData oo_intForKey:KEY_GOVERNMENT]);
-				if (result == nil && [UNIVERSE inInterstellarSpace])  result = DESC(@"not-applicable");
-				break;
-				
-			case kSystem_economy:
-				*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_ECONOMY]);
-				return YES;
-				
-			case kSystem_economyDescription:
-				result = OODisplayStringFromEconomyID([systemData oo_intForKey:KEY_ECONOMY]);
-				if (result == nil && [UNIVERSE inInterstellarSpace])  result = DESC(@"not-applicable");
-				break;
+			systemData = [UNIVERSE currentSystemData];
 			
-			case kSystem_techLevel:
-				*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_TECHLEVEL]);
-				return YES;
+			switch (JSID_TO_INT(propID))
+			{
+				case kSystem_name:
+					result = [systemData objectForKey:KEY_NAME];
+					break;
+					
+				case kSystem_description:
+					result = [systemData objectForKey:KEY_DESCRIPTION];
+					break;
+					
+				case kSystem_inhabitantsDescription:
+					result = [systemData objectForKey:KEY_INHABITANTS];
+					break;
+					
+				case kSystem_government:
+					*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_GOVERNMENT]);
+					return YES;
+					
+				case kSystem_governmentDescription:
+					result = OODisplayStringFromGovernmentID([systemData oo_intForKey:KEY_GOVERNMENT]);
+					if (result == nil)  result = DESC(@"not-applicable");
+					break;
+					
+				case kSystem_economy:
+					*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_ECONOMY]);
+					return YES;
+					
+				case kSystem_economyDescription:
+					result = OODisplayStringFromEconomyID([systemData oo_intForKey:KEY_ECONOMY]);
+					if (result == nil)  result = DESC(@"not-applicable");
+					break;
 				
-			case kSystem_population:
-				*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_POPULATION]);
-				return YES;
+				case kSystem_techLevel:
+					*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_TECHLEVEL]);
+					return YES;
+					
+				case kSystem_population:
+					*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_POPULATION]);
+					return YES;
+					
+				case kSystem_productivity:
+					*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_PRODUCTIVITY]);
+					return YES;
+					
+				default:
+					OOJSReportBadPropertySelector(context, this, propID, sSystemProperties);
+					return NO;
+			}
+		}
+		else
+		{
+			// if in interstellar space, systemData values are null & void!
+			switch (JSID_TO_INT(propID))
+			{
+				case kSystem_name:
+					result = DESC(@"interstellar-space");
+					break;
+					
+				case kSystem_description:
+					result = @"";
+					break;
+					
+				case kSystem_inhabitantsDescription:
+					result = DESC(@"not-applicable");
+					break;
+					
+				case kSystem_government:
+					*value = INT_TO_JSVAL(-1);
+					return YES;
+					
+				case kSystem_governmentDescription:
+					result = DESC(@"not-applicable");
+					break;
+					
+				case kSystem_economy:
+					*value = INT_TO_JSVAL(-1);
+					return YES;
+					
+				case kSystem_economyDescription:
+					result = DESC(@"not-applicable");
+					break;
 				
-			case kSystem_productivity:
-				*value = INT_TO_JSVAL([systemData oo_intForKey:KEY_PRODUCTIVITY]);
-				return YES;
-				
-			default:
-				OOJSReportBadPropertySelector(context, this, propID, sSystemProperties);
-				return NO;
+				case kSystem_techLevel:
+					*value = INT_TO_JSVAL(-1);
+					return YES;
+					
+				case kSystem_population:
+					*value = INT_TO_JSVAL(0);
+					return YES;
+					
+				case kSystem_productivity:
+					*value = INT_TO_JSVAL(0);
+					return YES;
+					
+				default:
+					OOJSReportBadPropertySelector(context, this, propID, sSystemProperties);
+					return NO;
+			}
 		}
 	}
 	
@@ -344,14 +465,43 @@ static JSBool SystemSetProperty(JSContext *context, JSObject *this, jsid propID,
 	OOGalaxyID					galaxy;
 	OOSystemID					system;
 	NSString					*stringValue = nil;
+	NSString					*manifest = nil;
+	jsdouble					fValue;
 	int32						iValue;
+	JSBool            bValue;
 	
 	player = OOPlayerForScripting();
 	
 	galaxy = [player currentGalaxyID];
 	system = [player currentSystemID];
+
+	switch (JSID_TO_INT(propID))
+	{
+		case kSystem_ambientLevel:
+			if (JS_ValueToNumber(context, *value, &fValue))
+			{
+				[UNIVERSE setAmbientLightLevel:fValue];
+				[UNIVERSE setLighting];
+				return YES;
+			}
+			break;
+			
+		case kSystem_breakPattern:
+			if (JS_ValueToBoolean(context, *value, &bValue))
+			{
+				[UNIVERSE setWitchspaceBreakPattern:bValue];
+				return YES;
+			}
+
+			break;
+		default:
+		{}// do nothing yet
+	}
+
 	
-	if (system == -1)  return YES;	// Can't change anything in interstellar space.
+	if (system == -1)  return YES;	// Can't change anything else in interstellar space.
+
+	manifest = [[OOJSScript currentlyRunningScript] propertyNamed:kLocalManifestProperty];
 	
 	switch (JSID_TO_INT(propID))
 	{
@@ -359,7 +509,7 @@ static JSBool SystemSetProperty(JSContext *context, JSObject *this, jsid propID,
 			stringValue = OOStringFromJSValue(context, *value);
 			if (stringValue != nil)
 			{
-				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_NAME value:stringValue];
+				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_NAME value:stringValue fromManifest:manifest forLayer:OO_LAYER_OXP_DYNAMIC];
 				return YES;
 			}
 			break;
@@ -368,7 +518,7 @@ static JSBool SystemSetProperty(JSContext *context, JSObject *this, jsid propID,
 			stringValue = OOStringFromJSValue(context, *value);
 			if (stringValue != nil)
 			{
-				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_DESCRIPTION value:stringValue];
+				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_DESCRIPTION value:stringValue fromManifest:manifest forLayer:OO_LAYER_OXP_DYNAMIC];
 				return YES;
 			}
 			break;
@@ -377,7 +527,7 @@ static JSBool SystemSetProperty(JSContext *context, JSObject *this, jsid propID,
 			stringValue = OOStringFromJSValue(context, *value);
 			if (stringValue != nil)
 			{
-				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_INHABITANTS value:stringValue];
+				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_INHABITANTS value:stringValue fromManifest:manifest forLayer:OO_LAYER_OXP_DYNAMIC];
 				return YES;
 			}
 			break;
@@ -387,7 +537,7 @@ static JSBool SystemSetProperty(JSContext *context, JSObject *this, jsid propID,
 			{
 				if (iValue < 0)  iValue = 0;
 				if (7 < iValue)  iValue = 7;
-				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_GOVERNMENT value:[NSNumber numberWithInt:iValue]];
+				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_GOVERNMENT value:[NSNumber numberWithInt:iValue] fromManifest:manifest forLayer:OO_LAYER_OXP_DYNAMIC];
 				return YES;
 			}
 			break;
@@ -397,7 +547,7 @@ static JSBool SystemSetProperty(JSContext *context, JSObject *this, jsid propID,
 			{
 				if (iValue < 0)  iValue = 0;
 				if (7 < iValue)  iValue = 7;
-				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_ECONOMY value:[NSNumber numberWithInt:iValue]];
+				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_ECONOMY value:[NSNumber numberWithInt:iValue] fromManifest:manifest forLayer:OO_LAYER_OXP_DYNAMIC];
 				return YES;
 			}
 			break;
@@ -407,7 +557,7 @@ static JSBool SystemSetProperty(JSContext *context, JSObject *this, jsid propID,
 			{
 				if (iValue < 0)  iValue = 0;
 				if (15 < iValue)  iValue = 15;
-				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_TECHLEVEL value:[NSNumber numberWithInt:iValue]];
+				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_TECHLEVEL value:[NSNumber numberWithInt:iValue] fromManifest:manifest forLayer:OO_LAYER_OXP_DYNAMIC];
 				return YES;
 			}
 			break;
@@ -415,7 +565,7 @@ static JSBool SystemSetProperty(JSContext *context, JSObject *this, jsid propID,
 		case kSystem_population:
 			if (JS_ValueToInt32(context, *value, &iValue))
 			{
-				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_POPULATION value:[NSNumber numberWithInt:iValue]];
+				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_POPULATION value:[NSNumber numberWithInt:iValue] fromManifest:manifest forLayer:OO_LAYER_OXP_DYNAMIC];
 				return YES;
 			}
 			break;
@@ -423,7 +573,7 @@ static JSBool SystemSetProperty(JSContext *context, JSObject *this, jsid propID,
 		case kSystem_productivity:
 			if (JS_ValueToInt32(context, *value, &iValue))
 			{
-				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_PRODUCTIVITY value:[NSNumber numberWithInt:iValue]];
+				[UNIVERSE setSystemDataForGalaxy:galaxy planet:system key:KEY_PRODUCTIVITY value:[NSNumber numberWithInt:iValue] fromManifest:manifest forLayer:OO_LAYER_OXP_DYNAMIC];
 				return YES;
 			}
 			break;
@@ -756,6 +906,39 @@ static JSBool SystemFilteredEntities(JSContext *context, uintN argc, jsval *vp)
 }
 
 
+// locationFromCode(populator_named_region : String)
+static JSBool SystemLocationFromCode(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	NSString			*code = nil;
+	if (argc > 0)  
+	{
+		code = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	}
+	if (EXPECT_NOT(code == nil))
+	{
+		OOJSReportBadArguments(context, @"System", @"locationFromCode", argc, OOJS_ARGV, nil, @"location code");
+		return NO;
+	}
+	OOSunEntity *sun = [UNIVERSE sun];
+	OOPlanetEntity *planet = [UNIVERSE planet];
+	HPVector position = kZeroHPVector;
+	if (sun == nil || planet == nil)
+	{
+		position = [UNIVERSE locationByCode:@"WITCHPOINT" withSun:nil andPlanet:nil];
+	}
+	else
+	{
+		position = [UNIVERSE locationByCode:code withSun:sun andPlanet:planet];
+	}
+
+	OOJS_RETURN_HPVECTOR(position);
+
+	OOJS_NATIVE_EXIT
+}
+
+
 // addShips(role : String, count : Number [, position: Vector [, radius: Number]]) : Array
 static JSBool SystemAddShips(JSContext *context, uintN argc, jsval *vp)
 {
@@ -787,8 +970,9 @@ static JSBool SystemAddGroupToRoute(JSContext *context, uintN argc, jsval *vp)
 // legacy_addShips(role : String, count : Number)
 static JSBool SystemLegacyAddShips(JSContext *context, uintN argc, jsval *vp)
 {
+	OOStandardsDeprecated(@"system.legacy_addShips() is deprecated");
 	OOJS_NATIVE_ENTER(context)
-	
+		
 	NSString			*role = nil;
 	int32				count;
 	
@@ -815,6 +999,7 @@ static JSBool SystemLegacyAddShips(JSContext *context, uintN argc, jsval *vp)
 // legacy_addSystemShips(role : String, count : Number, location : Number)
 static JSBool SystemLegacyAddSystemShips(JSContext *context, uintN argc, jsval *vp)
 {
+	OOStandardsDeprecated(@"system.legacy_addSystemShips() is deprecated");
 	OOJS_NATIVE_ENTER(context)
 	
 	jsdouble			position;
@@ -845,10 +1030,11 @@ static JSBool SystemLegacyAddSystemShips(JSContext *context, uintN argc, jsval *
 // legacy_addShipsAt(role : String, count : Number, coordScheme : String, coords : vectorExpression)
 static JSBool SystemLegacyAddShipsAt(JSContext *context, uintN argc, jsval *vp)
 {
+	OOStandardsDeprecated(@"system.legacy_addShipsAt() is deprecated");
 	OOJS_NATIVE_ENTER(context)
 	
 	PlayerEntity		*player = OOPlayerForScripting();
-	Vector				where;
+	HPVector				where;
 	NSString			*role = nil;
 	int32				count;
 	NSString			*coordScheme = nil;
@@ -881,10 +1067,11 @@ static JSBool SystemLegacyAddShipsAt(JSContext *context, uintN argc, jsval *vp)
 // legacy_addShipsAtPrecisely(role : String, count : Number, coordScheme : String, coords : vectorExpression)
 static JSBool SystemLegacyAddShipsAtPrecisely(JSContext *context, uintN argc, jsval *vp)
 {
+	OOStandardsDeprecated(@"system.legacy_addShipsAtPrecisely() is deprecated");
 	OOJS_NATIVE_ENTER(context)
 	
 	PlayerEntity		*player = OOPlayerForScripting();
-	Vector				where;
+	HPVector				where;
 	NSString			*role = nil;
 	int32				count;
 	NSString			*coordScheme = nil;
@@ -917,10 +1104,11 @@ static JSBool SystemLegacyAddShipsAtPrecisely(JSContext *context, uintN argc, js
 // legacy_addShipsWithinRadius(role : String, count : Number, coordScheme : String, coords : vectorExpression, radius : Number)
 static JSBool SystemLegacyAddShipsWithinRadius(JSContext *context, uintN argc, jsval *vp)
 {
+	OOStandardsDeprecated(@"system.legacy_addShipsWithinRadius() is deprecated");
 	OOJS_NATIVE_ENTER(context)
 	
 	PlayerEntity		*player = OOPlayerForScripting();
-	Vector				where;
+	HPVector				where;
 	jsdouble			radius;
 	NSString			*role = nil;
 	int32				count;
@@ -956,6 +1144,7 @@ static JSBool SystemLegacyAddShipsWithinRadius(JSContext *context, uintN argc, j
 // legacy_spawnShip(key : string)
 static JSBool SystemLegacySpawnShip(JSContext *context, uintN argc, jsval *vp)
 {
+	OOStandardsDeprecated(@"system.legacy_spawnShip() is deprecated");
 	OOJS_NATIVE_ENTER(context)
 	
 	NSString			*key = nil;
@@ -964,7 +1153,7 @@ static JSBool SystemLegacySpawnShip(JSContext *context, uintN argc, jsval *vp)
 	if (argc > 0)  key = OOStringFromJSValue(context, OOJS_ARGV[0]);
 	if (key == nil)
 	{
-		OOJSReportBadArguments(context, @"System", @"legacy_addShipWithinRadius", MIN(argc, 1U), OOJS_ARGV, nil, @"string (ship key)");
+		OOJSReportBadArguments(context, @"System", @"legacy_spawnShip", MIN(argc, 1U), OOJS_ARGV, nil, @"string (ship key)");
 		return NO;
 	}
 	
@@ -987,13 +1176,16 @@ static JSBool SystemStaticSystemNameForID(JSContext *context, uintN argc, jsval
 	
 	int32				systemID;
 	
-	if (argc < 1 || !JS_ValueToInt32(context, OOJS_ARGV[0], &systemID) || systemID < 0 || kOOMaximumSystemID < systemID)
+	if (argc < 1 || !JS_ValueToInt32(context, OOJS_ARGV[0], &systemID) || systemID < -1 || kOOMaximumSystemID < systemID)	// -1 interstellar space!
 	{
 		OOJSReportBadArguments(context, @"System", @"systemNameForID", MIN(argc, 1U), OOJS_ARGV, nil, @"system ID");
 		return NO;
 	}
 	
-	OOJS_RETURN_OBJECT([UNIVERSE getSystemName:[UNIVERSE systemSeedForSystemNumber:systemID]]);
+	if (systemID == -1)
+		OOJS_RETURN_OBJECT(DESC(@"interstellar-space"));
+	else
+		OOJS_RETURN_OBJECT([UNIVERSE getSystemName:systemID]);
 	
 	OOJS_NATIVE_EXIT
 }
@@ -1015,7 +1207,9 @@ static JSBool SystemStaticSystemIDForName(JSContext *context, uintN argc, jsval
 	}
 	
 	OOJS_BEGIN_FULL_NATIVE(context)
-	result = [UNIVERSE systemIDForSystemSeed:[UNIVERSE systemSeedForSystemName:name]];
+
+	result = [UNIVERSE findSystemFromName:name];
+
 	OOJS_END_FULL_NATIVE
 	
 	OOJS_RETURN_INT(result);
@@ -1056,6 +1250,164 @@ static JSBool SystemStaticInfoForSystem(JSContext *context, uintN argc, jsval *v
 }
 
 
+static JSBool SystemAddVisualEffect(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	NSString			*key = nil;
+	HPVector         where;
+	
+	uintN				consumed = 0;
+
+	if (argc > 0)  key = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	if (key == nil)
+	{
+		OOJSReportBadArguments(context, @"System", @"addVisualEffect", MIN(argc, 1U), &OOJS_ARGV[0], nil, @"string (key)");
+		return NO;
+	}
+
+	if (!VectorFromArgumentListNoError(context, argc - 1, OOJS_ARGV + 1, &where, &consumed))
+	{
+		OOJSReportBadArguments(context, @"System", @"addVisualEffect", MIN(argc - 1, 1U), &OOJS_ARGV[1], nil, @"vector");
+		return NO;
+	}
+
+	OOVisualEffectEntity *result = nil;
+
+	OOJS_BEGIN_FULL_NATIVE(context)
+
+	result = [UNIVERSE addVisualEffectAt:where withKey:key];
+
+	OOJS_END_FULL_NATIVE
+	
+	OOJS_RETURN_OBJECT(result);
+
+	OOJS_NATIVE_EXIT
+}
+
+static JSBool SystemSetPopulator(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+
+	NSString *key;
+	NSMutableDictionary *settings;
+	JSObject			*params = NULL;
+
+	if (argc < 1) 
+	{
+		OOJSReportBadArguments(context, @"System", @"setPopulator", MIN(argc, 0U), &OOJS_ARGV[0], nil, @"string (key), object (settings)");
+		return NO;
+	}
+	key = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	if (key == nil)
+	{
+		OOJSReportBadArguments(context, @"System", @"setPopulator", MIN(argc, 0U), &OOJS_ARGV[0], nil, @"key, settings");
+		return NO;
+	}
+	if (argc < 2 || JSVAL_IS_NULL(OOJS_ARGV[1]))
+	{
+		// clearing
+		[UNIVERSE setPopulatorSetting:key to:nil];
+	}
+	else
+	{
+		// adding
+		if (!JS_ValueToObject(context, OOJS_ARGV[1], &params))
+		{
+			OOJSReportBadArguments(context, @"System", @"setPopulator", MIN(argc, 1U), OOJS_ARGV, NULL, @"key, settings");
+			return NO;
+		}
+		jsval				callback = JSVAL_NULL;
+		if (JS_GetProperty(context, params, "callback", &callback) == JS_FALSE || JSVAL_IS_VOID(callback))
+		{
+			OOJSReportBadArguments(context, @"System", @"setPopulator", MIN(argc, 1U), OOJS_ARGV, NULL, @"settings must have a 'callback' property.");
+			return NO;
+		}
+
+		OOJSPopulatorDefinition *populator = [[OOJSPopulatorDefinition alloc] init];
+		[populator setCallback:callback];
+
+		settings = OOJSNativeObjectFromJSObject(context, JSVAL_TO_OBJECT(OOJS_ARGV[1]));
+		[settings setObject:populator forKey:@"callbackObj"];
+
+		jsval				coords = JSVAL_NULL;
+		if (JS_GetProperty(context, params, "coordinates", &coords) != JS_FALSE && !JSVAL_IS_VOID(coords))
+		{
+			Vector coordinates = kZeroVector;
+			if (JSValueToVector(context, coords, &coordinates))
+			{
+				// convert vector in NS-storable form
+				[settings setObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:coordinates.x],[NSNumber numberWithFloat:coordinates.y],[NSNumber numberWithFloat:coordinates.z],nil] forKey:@"coordinates"];
+			}
+		}
+
+		[populator release];
+
+		[UNIVERSE setPopulatorSetting:key to:settings];
+	}	
+
+	OOJS_RETURN_VOID;
+
+	OOJS_NATIVE_EXIT
+}
+
+
+static JSBool SystemSetWaypoint(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+
+	NSString *key;
+	NSMutableDictionary *settings;
+	HPVector position;
+	Quaternion orientation;
+
+	if (argc < 1) 
+	{
+		OOJSReportBadArguments(context, @"System", @"setWaypoint", MIN(argc, 0U), &OOJS_ARGV[0], nil, @"key, position, orientation, definition");
+		return NO;
+	}
+	key = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	if (key == nil)
+	{
+		OOJSReportBadArguments(context, @"System", @"setWaypoint", MIN(argc, 0U), &OOJS_ARGV[0], nil, @"key, position, orientation, definition");
+		return NO;
+	}
+	if (argc < 4 || JSVAL_IS_NULL(OOJS_ARGV[3]))
+	{
+		// clearing
+		[UNIVERSE defineWaypoint:nil forKey:key];
+	}
+	else
+	{
+		// adding
+		if (!JSValueToHPVector(context, OOJS_ARGV[1], &position))
+		{
+			OOJSReportBadArguments(context, @"System", @"setWaypoint", MIN(argc, 2U), OOJS_ARGV, NULL, @"key, position, orientation, definition");
+			return NO;
+		}
+		if (!JSValueToQuaternion(context, OOJS_ARGV[2], &orientation))
+		{
+			OOJSReportBadArguments(context, @"System", @"setWaypoint", MIN(argc, 3U), OOJS_ARGV, NULL, @"key, position, orientation, definition");
+			return NO;
+		}
+		if (!JSVAL_IS_OBJECT(OOJS_ARGV[3]) || JSVAL_IS_NULL(OOJS_ARGV[3]))
+		{
+			OOJSReportBadArguments(context, @"System", @"setWaypoint", MIN(argc, 4U), OOJS_ARGV, NULL, @"key, position, orientation, definition");
+			return NO;
+		}
+		
+		settings = [[OOJSNativeObjectFromJSObject(context, JSVAL_TO_OBJECT(OOJS_ARGV[3])) mutableCopy] autorelease];
+		[settings setObject:[NSArray arrayWithObjects:[NSNumber numberWithDouble:position.x],[NSNumber numberWithDouble:position.y],[NSNumber numberWithDouble:position.z],nil] forKey:@"position"];
+		[settings setObject:[NSArray arrayWithObjects:[NSNumber numberWithDouble:orientation.w],[NSNumber numberWithDouble:orientation.x],[NSNumber numberWithDouble:orientation.y],[NSNumber numberWithDouble:orientation.z],nil] forKey:@"orientation"];
+
+		[UNIVERSE defineWaypoint:settings forKey:key];
+	}	
+
+	OOJS_RETURN_VOID;
+
+	OOJS_NATIVE_EXIT
+}
+
 // *** Helper functions ***
 
 // Shared implementation of addShips() and addGroup().
@@ -1066,7 +1418,7 @@ static JSBool SystemAddShipsOrGroup(JSContext *context, uintN argc, jsval *vp, B
 	NSString			*role = nil;
 	int32				count = 0;
 	uintN				consumed = 0;
-	Vector				where;
+	HPVector				where;
 	double				radius = NSNotFound;	// a negative value means 
 	id					result = nil;
 	
@@ -1203,7 +1555,7 @@ static BOOL GetRelativeToAndRange(JSContext *context, NSString *methodName, uint
 	// Get optional argument relativeTo : Entity
 	if (*ioArgc != 0)
 	{
-		if (EXPECT_NOT(!JSValueToEntity(context, **ioArgv, outRelativeTo)))
+		if (EXPECT_NOT(JSVAL_IS_NULL(**ioArgv) || !JSValueToEntity(context, **ioArgv, outRelativeTo)))
 		{
 			OOJSReportBadArguments(context, @"System", methodName, 1, *ioArgv, nil, @"entity");
 			return NO;
@@ -1248,7 +1600,7 @@ static NSArray *FindJSVisibleEntities(EntityFilterPredicate predicate, void *par
 	{
 		[result sortUsingFunction:CompareEntitiesByDistance context:relativeTo];
 	}
-	if (result == nil)  result = [NSArray array];
+	if (result == nil)  return [NSArray array];
 	return result;
 	
 	OOJS_PROFILE_EXIT
@@ -1279,8 +1631,8 @@ static NSComparisonResult CompareEntitiesByDistance(id a, id b, void *relativeTo
 	*r = (id)relativeTo;
 	float				d1, d2;
 	
-	d1 = distance2(ea->position, r->position);
-	d2 = distance2(eb->position, r->position);
+	d1 = HPdistance2(ea->position, r->position);
+	d2 = HPdistance2(eb->position, r->position);
 	
 	if (d1 < d2)  return NSOrderedAscending;
 	else if (d1 > d2)  return NSOrderedDescending;
