@@ -3,7 +3,7 @@
 OOJSSystemInfo.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,6 +29,8 @@ MA 02110-1301, USA.
 #import "OOJSVector.h"
 #import "OOIsNumberLiteral.h"
 #import "OOConstToString.h"
+#import "OOSystemDescriptionManager.h"
+#import "OOJSScript.h"
 
 
 static JSObject *sSystemInfoPrototype;
@@ -45,6 +47,10 @@ static JSBool SystemInfoEnumerate(JSContext *context, JSObject *this, JSIterateO
 
 static JSBool SystemInfoDistanceToSystem(JSContext *context, uintN argc, jsval *vp);
 static JSBool SystemInfoRouteToSystem(JSContext *context, uintN argc, jsval *vp);
+static JSBool SystemInfoSamplePrice(JSContext *context, uintN argc, jsval *vp);
+static JSBool SystemInfoSetPropertyMethod(JSContext *context, uintN argc, jsval *vp);
+
+static JSBool SystemInfoStaticSetInterstellarProperty(JSContext *context, uintN argc, jsval *vp);
 static JSBool SystemInfoStaticFilteredSystems(JSContext *context, uintN argc, jsval *vp);
 
 
@@ -89,9 +95,11 @@ static JSPropertySpec sSystemInfoProperties[] =
 static JSFunctionSpec sSystemInfoMethods[] =
 {
 	// JS name					Function					min args
-	{ "toString",				OOJSObjectWrapperToString,	0 },
-	{ "distanceToSystem",		SystemInfoDistanceToSystem,	1 },
-	{ "routeToSystem",			SystemInfoRouteToSystem,	1 },
+	{ "toString",				OOJSObjectWrapperToString,		0 },
+	{ "distanceToSystem",		SystemInfoDistanceToSystem,		1 },
+	{ "routeToSystem",			SystemInfoRouteToSystem,		1 },
+	{ "samplePrice",			SystemInfoSamplePrice,			1 },
+	{ "setProperty",			SystemInfoSetPropertyMethod,	3 },
 	{ 0 }
 };
 
@@ -100,6 +108,7 @@ static JSFunctionSpec sSystemInfoStaticMethods[] =
 {
 	// JS name					Function					min args
 	{ "filteredSystems",		SystemInfoStaticFilteredSystems, 2 },
+	{ "setInterstellarProperty",SystemInfoStaticSetInterstellarProperty,	4 },
 	{ 0 }
 };
 
@@ -107,6 +116,7 @@ static JSFunctionSpec sSystemInfoStaticMethods[] =
 // Helper class wrapped by JS SystemInfo objects
 @interface OOSystemInfo: NSObject
 {
+@private
 	OOGalaxyID				_galaxy;
 	OOSystemID				_system;
 	NSString				*_planetKey;
@@ -121,7 +131,7 @@ static JSFunctionSpec sSystemInfoStaticMethods[] =
 
 - (OOGalaxyID) galaxy;
 - (OOSystemID) system;
-- (Random_Seed) systemSeed;
+//- (Random_Seed) systemSeed;
 
 @end
 
@@ -192,9 +202,9 @@ DEFINE_JS_OBJECT_GETTER(JSSystemInfoGetSystemInfo, &sSystemInfoClass, sSystemInf
 }
 
 
-- (OOUInteger) hash
+- (NSUInteger) hash
 {
-	OOUInteger hash = _galaxy;
+	NSUInteger hash = _galaxy;
 	hash <<= 16;
 	hash |= (uint16_t)_system;
 	return hash;
@@ -213,7 +223,9 @@ DEFINE_JS_OBJECT_GETTER(JSSystemInfoGetSystemInfo, &sSystemInfoClass, sSystemInf
 
 - (void) setValue:(id)value forKey:(NSString *)key
 {
-	[UNIVERSE setSystemDataForGalaxy:_galaxy planet:_system key:key value:value];
+	NSString *manifest = [[OOJSScript currentlyRunningScript] propertyNamed:kLocalManifestProperty];
+
+	[UNIVERSE setSystemDataForGalaxy:_galaxy planet:_system key:key value:value  fromManifest:manifest forLayer:OO_LAYER_OXP_DYNAMIC];
 }
 
 
@@ -239,11 +251,11 @@ DEFINE_JS_OBJECT_GETTER(JSSystemInfoGetSystemInfo, &sSystemInfoClass, sSystemInf
 }
 
 
-- (Random_Seed) systemSeed
+/*- (Random_Seed) systemSeed
 {
 	NSAssert([PLAYER currentGalaxyID] == _galaxy, @"Attempt to use -[OOSystemInfo systemSeed] from a different galaxy.");
 	return [UNIVERSE systemSeedForSystemNumber:_system];
-}
+	}*/
 
 
 - (NSPoint) coordinates
@@ -252,7 +264,7 @@ DEFINE_JS_OBJECT_GETTER(JSSystemInfoGetSystemInfo, &sSystemInfoClass, sSystemInf
 	{
 		return [PLAYER galaxy_coordinates];
 	}
-	return [UNIVERSE coordinatesForSystem:[self systemSeed]];
+	return [UNIVERSE coordinatesForSystem:_system];
 }
 
 
@@ -349,7 +361,9 @@ static JSBool SystemInfoEnumerate(JSContext *context, JSObject *this, JSIterateO
 			enumerator = [[keys objectEnumerator] retain];
 			*state = PRIVATE_TO_JSVAL(enumerator);
 			
-			if (idp != NULL)  *idp = INT_TO_JSID([keys count]);
+			NSUInteger count = [keys count];
+			assert(count <= INT32_MAX);
+			if (idp != NULL)  *idp = INT_TO_JSID((int32_t)count);
 			return YES;
 		}
 		
@@ -458,15 +472,16 @@ static JSBool SystemInfoGetProperty(JSContext *context, JSObject *this, jsid pro
 	{
 		NSString *key = OOStringFromJSString(context, JSID_TO_STRING(propID));
 		
-		if (!sameGalaxy || savedInterstellarInfo)
+		OOSystemDescriptionManager *systemManager = [UNIVERSE systemManager];
+		id propValue = nil;
+		// interstellar space needs more work at this stage
+		if ([info system] != -1)
 		{
-			OOJSReportError(context, @"Cannot read systemInfo values for %@.", savedInterstellarInfo ?  @"invalid interstellar space reference" : @"other galaxies");
-			*value = JSVAL_VOID;
-			return NO;
+			propValue = [systemManager getProperty:key forSystem:[info system] inGalaxy:[info galaxy]];
+		} else {
+			propValue = [info valueForKey:key];
 		}
 		
-		id propValue = [info valueForKey:key];
-		
 		if (propValue != nil)
 		{
 			if ([propValue isKindOfClass:[NSNumber class]] || OOIsNumberLiteral(propValue, YES))
@@ -585,6 +600,89 @@ static JSBool SystemInfoRouteToSystem(JSContext *context, uintN argc, jsval *vp)
 }
 
 
+// samplePrice(commodity)
+static JSBool SystemInfoSamplePrice(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	OOSystemInfo			*thisInfo = nil;
+	
+	if (!JSSystemInfoGetSystemInfo(context, OOJS_THIS, &thisInfo))  return NO;
+	OOCommodityType commodity = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	if (EXPECT_NOT(![[UNIVERSE commodities] goodDefined:commodity]))
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"samplePrice", MIN(argc, 1U), OOJS_ARGV, NULL, @"Unrecognised commodity type");
+		return NO;
+	}
+
+	BOOL sameGalaxy = ([thisInfo galaxy] == [PLAYER galaxyNumber]);
+	if (!sameGalaxy)
+	{
+		OOJSReportErrorForCaller(context, @"SystemInfo", @"routeToSystem", @"Cannot calculate sample price for destinations in other galaxies.");
+		return NO;
+	}
+
+	OOCreditsQuantity price = [[UNIVERSE commodities] samplePriceForCommodity:commodity inEconomy:[[thisInfo valueForKey:@"economy"] intValue] withScript:[thisInfo valueForKey:@"commodity_script"] inSystem:[thisInfo system]];
+	
+	OOJS_RETURN_INT(price);
+	
+	OOJS_NATIVE_EXIT
+}
+
+
+static JSBool SystemInfoSetPropertyMethod(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	OOSystemInfo			*thisInfo = nil;
+	
+	if (!JSSystemInfoGetSystemInfo(context, OOJS_THIS, &thisInfo))  return NO;
+
+	NSString *property = nil;
+	id value = nil;
+	NSString *manifest = nil;
+
+	int32 iValue;
+
+	if (argc < 3)
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"setProperty(layer, property, value [,manifest])");
+		return NO;
+	}
+	if (!JS_ValueToInt32(context, OOJS_ARGV[0], &iValue))
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"setProperty(layer, property, value [,manifest])");
+		return NO;
+	}
+	if (iValue < 0 || iValue >= OO_SYSTEM_LAYERS)
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"layer must be 0, 1, 2 or 3");
+		return NO;
+	}
+	OOSystemLayer layer = (OOSystemLayer)iValue;
+
+	property = OOStringFromJSValue(context, OOJS_ARGV[1]);
+	if (!JSVAL_IS_NULL(OOJS_ARGV[2]))
+	{
+		value = OOJSNativeObjectFromJSValue(context, OOJS_ARGV[2]);
+	}
+	if (argc >= 4)
+	{
+		manifest = OOStringFromJSValue(context, OOJS_ARGV[3]);
+	}
+	else
+	{
+		manifest = [[OOJSScript currentlyRunningScript] propertyNamed:kLocalManifestProperty];
+	}
+
+	[UNIVERSE setSystemDataForGalaxy:[thisInfo galaxy] planet:[thisInfo system] key:property value:value fromManifest:manifest forLayer:layer];
+
+	OOJS_RETURN_VOID;
+	
+	OOJS_NATIVE_EXIT
+}
+
+
 // filteredSystems(this : Object, predicate : Function) : Array
 static JSBool SystemInfoStaticFilteredSystems(JSContext *context, uintN argc, jsval *vp)
 {
@@ -658,3 +756,101 @@ static JSBool SystemInfoStaticFilteredSystems(JSContext *context, uintN argc, js
 	
 	OOJS_NATIVE_EXIT
 }
+
+
+static JSBool SystemInfoStaticSetInterstellarProperty(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	NSString *property = nil;
+	id value = nil;
+	NSString *manifest = nil;
+
+	int32 iValue;
+	OOGalaxyID g;
+	OOSystemID s1,s2;
+
+	if (argc < 6)
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setInterstellarProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"setProperty(galaxy, fromsystem, tosystem, layer, property, value [,manifest])");
+		return NO;
+	}
+	if (!JS_ValueToInt32(context, OOJS_ARGV[0], &iValue))
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setInterstellarProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"setProperty(galaxy, fromsystem, tosystem, layer, property, value [,manifest])");
+		return NO;
+	}
+	if (iValue < 0 || iValue > kOOMaximumGalaxyID)
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setInterstellarProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"galaxy out of range");
+		return NO;
+	}
+	else
+	{
+		g = (OOGalaxyID)iValue;
+	}
+
+	if (!JS_ValueToInt32(context, OOJS_ARGV[1], &iValue))
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setInterstellarProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"setProperty(galaxy, fromsystem, tosystem, layer, property, value [,manifest])");
+		return NO;
+	}
+	if (iValue < 0 || iValue > kOOMaximumSystemID)
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setInterstellarProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"fromsystem out of range");
+		return NO;
+	}
+	else
+	{
+		s1 = (OOSystemID)iValue;
+	}
+
+	if (!JS_ValueToInt32(context, OOJS_ARGV[2], &iValue))
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setInterstellarProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"setProperty(galaxy, fromsystem, tosystem, layer, property, value [,manifest])");
+		return NO;
+	}
+	if (iValue < 0 || iValue > kOOMaximumSystemID)
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setInterstellarProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"tosystem out of range");
+		return NO;
+	}
+	else
+	{
+		s2 = (OOSystemID)iValue;
+	}
+
+	if (!JS_ValueToInt32(context, OOJS_ARGV[3], &iValue))
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setInterstellarProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"setProperty(galaxy, fromsystem, tosystem, layer, property, value [,manifest])");
+		return NO;
+	}
+	if (iValue < 0 || iValue >= OO_SYSTEM_LAYERS)
+	{
+		OOJSReportBadArguments(context, @"SystemInfo", @"setInterstellarProperty", MIN(argc, 3U), OOJS_ARGV, NULL, @"layer must be 0, 1, 2 or 3");
+		return NO;
+	}
+	OOSystemLayer layer = (OOSystemLayer)iValue;
+
+	property = OOStringFromJSValue(context, OOJS_ARGV[4]);
+	if (!JSVAL_IS_NULL(OOJS_ARGV[5]))
+	{
+		value = OOJSNativeObjectFromJSValue(context, OOJS_ARGV[5]);
+	}
+	if (argc >= 7)
+	{
+		manifest = OOStringFromJSValue(context, OOJS_ARGV[6]);
+	}
+	else
+	{
+		manifest = [[OOJSScript currentlyRunningScript] propertyNamed:kLocalManifestProperty];
+	}
+
+	NSString *key = [NSString stringWithFormat:@"interstellar: %u %u %u",g,s1,s2];
+	
+	[[UNIVERSE systemManager] setProperty:property forSystemKey:key andLayer:layer toValue:value fromManifest:manifest];
+
+	OOJS_RETURN_VOID;
+	
+	OOJS_NATIVE_EXIT
+}
