@@ -3,7 +3,7 @@
 OOJavaScriptEngine.m
 
 JavaScript support for Oolite
-Copyright (C) 2007-2011 David Taylor and Jens Ayton.
+Copyright (C) 2007-2013 David Taylor and Jens Ayton.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -36,6 +36,8 @@ MA 02110-1301, USA.
 #import "ResourceManager.h"
 #import "NSNumberOOExtensions.h"
 #import "OOConstToJSString.h"
+#import "OOVisualEffectEntity.h"
+#import "OOWaypointEntity.h"
 
 #import "OOJSGlobal.h"
 #import "OOJSMissionVariables.h"
@@ -45,6 +47,12 @@ MA 02110-1301, USA.
 #import "OOJSEntity.h"
 #import "OOJSShip.h"
 #import "OOJSStation.h"
+#import "OOJSDock.h"
+#import "OOJSVisualEffect.h"
+#import "OOJSExhaustPlume.h"
+#import "OOJSFlasher.h"
+#import "OOJSWormhole.h"
+#import "OOJSWaypoint.h"
 #import "OOJSPlayer.h"
 #import "OOJSPlayerShip.h"
 #import "OOJSManifest.h"
@@ -194,7 +202,7 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 		if (activeScript == nil)  activeScript = @"<unidentified script>";
 		OOLog(messageClass, @"%@ JavaScript %@ (%@): %@", highlight, severity, activeScript, messageText);
 		
-		if (!showLocation && sErrorHandlerStackSkip == 0 && report->filename != NULL)
+		if (showLocation && sErrorHandlerStackSkip == 0 && report->filename != NULL)
 		{
 			// Second line: where error occured, and line if provided. (The line is only provided for compile-time errors, not run-time errors.)
 			if ([lineBuf length] != 0)
@@ -233,7 +241,7 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 
 @implementation OOJavaScriptEngine
 
-+ (OOJavaScriptEngine *)sharedEngine
++ (OOJavaScriptEngine *) sharedEngine
 {
 	if (sSharedEngine == nil)  sSharedEngine = [[self alloc] init];
 	
@@ -270,7 +278,7 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 	assert(sizeof(jschar) == sizeof(unichar));
 	
 	// initialize the JS run time, and return result in runtime.
-	_runtime = JS_NewRuntime(8L * 1024L * 1024L);
+	_runtime = JS_NewRuntime(32L * 1024L * 1024L);
 	
 	// if runtime creation failed, end the program here.
 	if (_runtime == NULL)
@@ -344,6 +352,12 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 	InitOOJSEntity(gOOJSMainThreadContext, _globalObject);
 	InitOOJSShip(gOOJSMainThreadContext, _globalObject);
 	InitOOJSStation(gOOJSMainThreadContext, _globalObject);
+	InitOOJSDock(gOOJSMainThreadContext, _globalObject);
+	InitOOJSVisualEffect(gOOJSMainThreadContext, _globalObject);
+	InitOOJSExhaustPlume(gOOJSMainThreadContext, _globalObject);
+	InitOOJSFlasher(gOOJSMainThreadContext, _globalObject);
+	InitOOJSWormhole(gOOJSMainThreadContext, _globalObject);
+	InitOOJSWaypoint(gOOJSMainThreadContext, _globalObject);
 	InitOOJSPlayer(gOOJSMainThreadContext, _globalObject);
 	InitOOJSPlayerShip(gOOJSMainThreadContext, _globalObject);
 	InitOOJSManifest(gOOJSMainThreadContext, _globalObject);
@@ -380,8 +394,6 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 		JSContext *context = OOJSAcquireContext();
 		JS_ClearScope(gOOJSMainThreadContext, _globalObject);
 		
-		OOJSFrameCallbacksRemoveAll();
-		
 		_globalObject = NULL;
 		_objectClass = NULL;
 		_stringClass = NULL;
@@ -402,11 +414,39 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 }
 
 
-- (void) reset
+- (BOOL) reset
 {
-	NSAssert(gOOJSMainThreadContext != NULL, @"Can't reset JavaScript engine at this point.");
+	NSAssert(gOOJSMainThreadContext != NULL, @"JavaScript engine not active. Can't reset.");
+	
+	OOJSFrameCallbacksRemoveAll();
+	
+# if 0
+	// deferred JS reset - test harness.
+	static int counter = 3;		// loading a savegame with different strict mode calls js reset twice
+	if (counter-- == 0) {
+	counter = 3;
+	OOLog(@"script.javascript.init.error", @"JavaScript processes still pending. Can't reset JavaScript engine.");
+		return NO;
+	}
+	else
+	{
+		OOLog(@"script.javascript.init", @"JavaScript reset successful.");
+	}
+#endif
+		
 #if JS_THREADSAFE
-	NSAssert(!JS_IsInRequest(gOOJSMainThreadContext), @"Can't reset JavaScript engine while running JavaScript.");
+	//NSAssert(!JS_IsInRequest(gOOJSMainThreadContext), @"JavaScript processes still pending. Can't reset JavaScript engine.");
+	
+	if (JS_IsInRequest(gOOJSMainThreadContext))
+	{
+		// some threads are still pending, this should mean timers are still being removed.
+		OOLog(@"script.javascript.init.error", @"JavaScript processes still pending. Can't reset JavaScript engine.");
+		return NO;
+	}
+	else
+	{
+		OOLog(@"script.javascript.init", @"JavaScript reset successful.");
+	}
 #endif
 	
 	JSContext *context = OOJSAcquireContext();
@@ -420,7 +460,8 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 	[[NSNotificationCenter defaultCenter] postNotificationName:kOOJavaScriptEngineDidResetNotification object:self];
 	OOJSRelinquishContext(context);
 	
-	[self garbageCollectionOpportunity];
+	[self garbageCollectionOpportunity:YES];
+	return YES;
 }
 
 
@@ -428,6 +469,8 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 {
 	sSharedEngine = nil;
 	
+	OOJSFrameCallbacksRemoveAll();
+	
 	[self destroyMainThreadContext];
 	JS_DestroyRuntime(_runtime);
 	
@@ -481,14 +524,17 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 }
 
 
-- (void) garbageCollectionOpportunity
+- (void) garbageCollectionOpportunity:(BOOL)force
 {
 	JSContext *context = OOJSAcquireContext();
-#ifndef NDEBUG
-	JS_GC(context);
-#else
-	JS_MaybeGC(context);
-#endif
+	if (force)
+	{
+		JS_GC(context);
+	}
+	else
+	{
+		JS_MaybeGC(context);
+	}
 	OOJSRelinquishContext(context);
 }
 
@@ -690,7 +736,8 @@ void OOJSDumpStack(JSContext *context)
 {
 	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 	
-	NS_DURING
+	@try
+	{
 		JSStackFrame	*frame = NULL;
 		unsigned		idx = 0;
 		unsigned		skip = sErrorHandlerStackSkip;
@@ -804,19 +851,21 @@ void OOJSDumpStack(JSContext *context)
 				JS_PutPropertyDescArray(context, &properties);
 			}
 		}
-	NS_HANDLER
-		OOLog(kOOLogException, @"Exception during JavaScript stack trace: %@:%@", [localException name], [localException reason]);
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		OOLog(kOOLogException, @"Exception during JavaScript stack trace: %@:%@", [exception name], [exception reason]);
+	}
 	
 	[pool release];
 }
 
 
 static const char *sConsoleScriptName;	// Lifetime is lifetime of script object, which is forever.
-static OOUInteger sConsoleEvalLineNo;
+static NSUInteger sConsoleEvalLineNo;
 
 
-static void GetLocationNameAndLine(JSContext *context, JSStackFrame *stackFrame, const char **name, OOUInteger *line)
+static void GetLocationNameAndLine(JSContext *context, JSStackFrame *stackFrame, const char **name, NSUInteger *line)
 {
 	NSCParameterAssert(context != NULL && stackFrame != NULL && name != NULL && line != NULL);
 	
@@ -845,9 +894,9 @@ NSString *OOJSDescribeLocation(JSContext *context, JSStackFrame *stackFrame)
 	NSCParameterAssert(context != NULL && stackFrame != NULL);
 	
 	const char	*fileName;
-	OOUInteger	lineNo;
+	NSUInteger	lineNo;
 	GetLocationNameAndLine(context, stackFrame, &fileName, &lineNo);
-	if (fileName == NULL)  return NO;
+	if (fileName == NULL)  return nil;
 	
 	// If this stops working, we probably need to switch to strcmp().
 	if (fileName == sConsoleScriptName && lineNo >= sConsoleEvalLineNo)  return @"<console input>";
@@ -860,7 +909,7 @@ NSString *OOJSDescribeLocation(JSContext *context, JSStackFrame *stackFrame)
 	NSString	*shortFileName = [fileNameObj lastPathComponent];
 	if (![[shortFileName lowercaseString] isEqualToString:@"script.js"])  fileNameObj = shortFileName;
 	
-	return [NSString stringWithFormat:@"%@:%u", fileNameObj, lineNo];
+	return [NSString stringWithFormat:@"%@:%lu", fileNameObj, lineNo];
 }
 
 
@@ -911,13 +960,13 @@ jsid OOJSIDFromString(NSString *string)
 	[string getCharacters:buffer];
 	
 	JSString *jsString = JS_InternUCStringN(context, buffer, length);
-	if (EXPECT_NOT(jsString == NULL))  return JSID_VOID;
 	
 	if (EXPECT_NOT(buffer != stackBuf))  free(buffer);
 	
 	OOJSRelinquishContext(context);
 	
-	return INTERNED_STRING_TO_JSID(jsString);
+	if (EXPECT(jsString != NULL))  return INTERNED_STRING_TO_JSID(jsString);
+	else  return JSID_VOID;
 }
 
 
@@ -961,15 +1010,18 @@ void OOJSReportErrorForCaller(JSContext *context, NSString *scriptClass, NSStrin
 	va_list					args;
 	NSString				*msg = nil;
 	
-	NS_DURING
+	@try
+	{
 		va_start(args, format);
 		msg = [[NSString alloc] initWithFormat:format arguments:args];
 		va_end(args);
 		
 		OOJSReportError(context, @"%@%@", CallerPrefix(scriptClass, function), msg);
-	NS_HANDLER
+	}
+	@catch (id exception)
+	{
 		// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
 	[msg release];
 }
 
@@ -980,12 +1032,15 @@ void OOJSReportErrorWithArguments(JSContext *context, NSString *format, va_list
 	
 	NSCParameterAssert(JS_IsInRequest(context));
 	
-	NS_DURING
+	@try
+	{
 		msg = [[NSString alloc] initWithFormat:format arguments:args];
 		JS_ReportError(context, "%s", [msg UTF8String]);
-	NS_HANDLER
+	}
+	@catch (id exception)
+	{
 		// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
 	[msg release];
 }
 
@@ -1027,15 +1082,18 @@ void OOJSReportWarningForCaller(JSContext *context, NSString *scriptClass, NSStr
 	va_list					args;
 	NSString				*msg = nil;
 	
-	NS_DURING
+	@try
+	{
 		va_start(args, format);
 		msg = [[NSString alloc] initWithFormat:format arguments:args];
 		va_end(args);
 		
 		OOJSReportWarning(context, @"%@%@", CallerPrefix(scriptClass, function), msg);
-	NS_HANDLER
-	// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
+	@catch (id exception)
+	{
+		// Squash any secondary errors during error handling.
+	}
 	[msg release];
 }
 
@@ -1044,12 +1102,15 @@ void OOJSReportWarningWithArguments(JSContext *context, NSString *format, va_lis
 {
 	NSString				*msg = nil;
 	
-	NS_DURING
+	@try
+	{
 		msg = [[NSString alloc] initWithFormat:format arguments:args];
 		JS_ReportWarning(context, "%s", [msg UTF8String]);
-	NS_HANDLER
-	// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
+	@catch (id exception)
+	{
+		// Squash any secondary errors during error handling.
+	}
 	[msg release];
 }
 
@@ -1075,15 +1136,18 @@ void OOJSReportBadPropertyValue(JSContext *context, JSObject *thisObj, jsid prop
 
 void OOJSReportBadArguments(JSContext *context, NSString *scriptClass, NSString *function, uintN argc, jsval *argv, NSString *message, NSString *expectedArgsDescription)
 {
-	NS_DURING
+	@try
+	{
 		if (message == nil)  message = @"Invalid arguments";
 		message = [NSString stringWithFormat:@"%@ %@", message, [NSString stringWithJavaScriptParameters:argv count:argc inContext:context]];
 		if (expectedArgsDescription != nil)  message = [NSString stringWithFormat:@"%@ -- expected %@", message, expectedArgsDescription];
 		
 		OOJSReportErrorForCaller(context, scriptClass, function, @"%@.", message);
-	NS_HANDLER
-	// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
+	@catch (id exception)
+	{
+		// Squash any secondary errors during error handling.
+	}
 }
 
 
@@ -1137,20 +1201,25 @@ static JSObject *JSArrayFromNSArray(JSContext *context, NSArray *array)
 	OOJS_PROFILE_ENTER
 	
 	JSObject				*result = NULL;
-	unsigned				i;
-	unsigned				count;
-	jsval					value;
 	
 	if (array == nil)  return NULL;
 	
-	NS_DURING
+	@try
+	{
+		NSUInteger fullCount = [array count];
+		if (EXPECT_NOT(fullCount > INT32_MAX))
+		{
+			return NULL;
+		}
+		
+		uint32_t i, count = (int32_t)fullCount;
+		
 		result = JS_NewArrayObject(context, 0, NULL);
 		if (result != NULL)
 		{
-			count = [array count];
 			for (i = 0; i != count; ++i)
 			{
-				value = [[array objectAtIndex:i] oo_jsValueInContext:context];
+				jsval value = [[array objectAtIndex:i] oo_jsValueInContext:context];
 				BOOL OK = JS_SetElement(context, result, i, &value);
 				
 				if (EXPECT_NOT(!OK))
@@ -1160,9 +1229,11 @@ static JSObject *JSArrayFromNSArray(JSContext *context, NSArray *array)
 				}
 			}
 		}
-	NS_HANDLER
+	}
+	@catch (id ex)
+	{
 		result = NULL;
-	NS_ENDHANDLER
+	}
 	
 	return (JSObject *)result;
 	
@@ -1217,7 +1288,8 @@ static JSObject *JSObjectFromNSDictionary(JSContext *context, NSDictionary *dict
 	
 	if (dictionary == nil)  return NULL;
 	
-	NS_DURING
+	@try
+	{
 		result = JS_NewObject(context, NULL, NULL, NULL);	// create object of class Object
 		if (result != NULL)
 		{
@@ -1225,7 +1297,23 @@ static JSObject *JSObjectFromNSDictionary(JSContext *context, NSDictionary *dict
 			{
 				if ([key isKindOfClass:[NSString class]] && [key length] != 0)
 				{
+#ifndef __GNUC__
 					value = [[dictionary objectForKey:key] oo_jsValueInContext:context];
+#else
+#if __GNUC__ > 4 || __GNUC_MINOR__ > 6
+					value = [[dictionary objectForKey:key] oo_jsValueInContext:context];
+#else
+					// GCC before 4.7 seems to have problems with this
+					// bit if the object is a weakref, causing crashes
+					// in docking code.
+					id tmp = [dictionary objectForKey:key];
+					if ([tmp respondsToSelector:@selector(weakRefUnderlyingObject)])
+					{
+						tmp = [tmp weakRefUnderlyingObject];
+					}
+					value = [tmp oo_jsValueInContext:context];
+#endif
+#endif
 					if (!JSVAL_IS_VOID(value))
 					{
 						OK = JS_SetPropertyById(context, result, OOJSIDFromString(key), &value);
@@ -1249,9 +1337,11 @@ static JSObject *JSObjectFromNSDictionary(JSContext *context, NSDictionary *dict
 				if (EXPECT_NOT(!OK))  break;
 			}
 		}
-	NS_HANDLER
+	}
+	@catch (id exception)
+	{
 		OK = NO;
-	NS_ENDHANDLER
+	}
 	
 	if (EXPECT_NOT(!OK))
 	{
@@ -1484,7 +1574,7 @@ NSString *OOStringFromJSString(JSContext *context, JSString *string)
 	{
 		return nil;
 	}
-
+	
 	OOJS_PROFILE_EXIT
 }
 
@@ -1649,19 +1739,7 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 
 @implementation NSString (OOJavaScriptExtensions)
 
-+ (id) stringOrNilWithJavaScriptValue:(jsval)value inContext:(JSContext *)context
-{
-	return OOStringFromJSValue(context, value);
-}
-
-
-+ (id) stringWithJavaScriptValue:(jsval)value inContext:(JSContext *)context
-{
-	return OOStringFromJSValueEvenIfNull(context, value);
-}
-
-
-+ (id) stringWithJavaScriptParameters:(jsval *)params count:(uintN)count inContext:(JSContext *)context
++ (NSString *) stringWithJavaScriptParameters:(jsval *)params count:(uintN)count inContext:(JSContext *)context
 {
 	OOJS_PROFILE_ENTER
 	
@@ -1713,7 +1791,7 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 }
 
 
-+ (id) concatenationOfStringsFromJavaScriptValues:(jsval *)values count:(size_t)count separator:(NSString *)separator inContext:(JSContext *)context
++ (NSString *) concatenationOfStringsFromJavaScriptValues:(jsval *)values count:(size_t)count separator:(NSString *)separator inContext:(JSContext *)context
 {
 	OOJS_PROFILE_ENTER
 	
@@ -1726,7 +1804,7 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 	
 	for (i = 0; i != count; ++i)
 	{
-		element = [NSString stringWithJavaScriptValue:values[i] inContext:context];
+		element = OOStringFromJSValueEvenIfNull(context, values[i]);
 		if (result == nil)  result = [[element mutableCopy] autorelease];
 		else
 		{
@@ -1746,12 +1824,12 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 	OOJS_PROFILE_ENTER
 	
 	NSMutableString			*result = nil;
-	unsigned				i, length;
+	NSUInteger				i, length;
 	unichar					c;
 	NSAutoreleasePool		*pool = nil;
 	
 	length = [self length];
-	result = [NSMutableString stringWithCapacity:[self length]];
+	result = [NSMutableString stringWithCapacity:length];
 	
 	// Not hugely efficient.
 	pool = [[NSAutoreleasePool alloc] init];
@@ -1849,38 +1927,24 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 	BOOL					isFloat = NO;
 	long long				longLongValue;
 	
-#if 0
-	/*	Under GNUstep, it is not possible to distinguish between boolean
-		NSNumbers and integer NSNumbers - there is no such distinction.
-		It is better to convert booleans to integers than vice versa.
-	*/
-	if ([self oo_isBoolean])
+	isFloat = [self oo_isFloatingPointNumber];
+	if (!isFloat)
 	{
-		if ([self boolValue])  result = JSVAL_TRUE;
-		else  result = JSVAL_FALSE;
+		longLongValue = [self longLongValue];
+		if (longLongValue < (long long)JSVAL_INT_MIN || (long long)JSVAL_INT_MAX < longLongValue)
+		{
+			// values outside JSVAL_INT range are returned as doubles.
+			isFloat = YES;
+		}
+	}
+	
+	if (isFloat)
+	{
+		if (!JS_NewNumberValue(context, [self doubleValue], &result)) result = JSVAL_VOID;
 	}
 	else
-#endif
 	{
-		isFloat = [self oo_isFloatingPointNumber];
-		if (!isFloat)
-		{
-			longLongValue = [self longLongValue];
-			if (longLongValue < (long long)JSVAL_INT_MIN || (long long)JSVAL_INT_MAX < longLongValue)
-			{
-				// values outside JSVAL_INT range are returned as doubles.
-				isFloat = YES;
-			}
-		}
-		
-		if (isFloat)
-		{
-			if (!JS_NewNumberValue(context, [self doubleValue], &result)) result = JSVAL_VOID;
-		}
-		else
-		{
-			result = INT_TO_JSVAL(longLongValue);
-		}
+		result = INT_TO_JSVAL((int32_t)longLongValue);
 	}
 	
 	return result;
@@ -2054,6 +2118,11 @@ BOOL JSEntityIsJavaScriptSearchablePredicate(Entity *entity, void *parameter)
 }
 
 
+BOOL JSEntityIsDemoShipPredicate(Entity *entity, void *parameter)
+{
+	return ([entity isVisibleToScripts] && [entity isShip] && [entity status] == STATUS_COCKPIT_DISPLAY && ![entity isSubEntity]);
+}
+
 static NSMapTable *sRegisteredSubClasses;
 
 void OOJSRegisterSubclass(JSClass *subclass, JSClass *superclass)
@@ -2190,6 +2259,11 @@ NSDictionary *OOJSDictionaryFromJSObject(JSContext *context, JSObject *object)
 		}
 		else if (JSID_IS_INT(thisID))
 		{
+			/* this causes problems with native functions which expect string keys
+			 * e.g. in mission.runScreen with the 'choices' parameter
+			 * should this instead be making the objKey a string?
+			 * is there anything that relies on the current behaviour?
+			 * - CIM 15/2/13 */
 			objKey = [NSNumber numberWithInt:JSID_TO_INT(thisID)];
 		}
 		else
@@ -2229,7 +2303,7 @@ NSDictionary *OOJSDictionaryFromStringTable(JSContext *context, jsval tableValue
 	id							objKey = nil;
 	id							objValue = nil;
 	
-	if (EXPECT_NOT(!JS_ValueToObject(context, tableValue, &tableObject)))
+	if (EXPECT_NOT(JSVAL_IS_NULL(tableValue) || !JS_ValueToObject(context, tableValue, &tableObject)))
 	{
 		return nil;
 	}
@@ -2259,8 +2333,7 @@ NSDictionary *OOJSDictionaryFromStringTable(JSContext *context, jsval tableValue
 		
 		if (objKey != nil && !JSVAL_IS_VOID(value))
 		{
-			// Note: we want nulls and undefines included, so not OOStringFromJSValue().
-			objValue = [NSString stringWithJavaScriptValue:value inContext:context];
+			objValue = OOStringFromJSValueEvenIfNull(context, value);
 			
 			if (objValue != nil)
 			{
@@ -2427,7 +2500,7 @@ static id JSArrayConverter(JSContext *context, JSObject *array)
 
 static id JSStringConverter(JSContext *context, JSObject *object)
 {
-	return [NSString stringOrNilWithJavaScriptValue:OBJECT_TO_JSVAL(object) inContext:context];
+	return OOStringFromJSValue(context, OBJECT_TO_JSVAL(object));
 }
 
 
