@@ -5,7 +5,7 @@ Universe.h
 Manages a lot of stuff that isn't managed somewhere else.
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -33,15 +33,19 @@ MA 02110-1301, USA.
 #import "OOTypes.h"
 #import "OOSound.h"
 #import "OOJSPropID.h"
-
+#import "OOStellarBody.h"
+#import "OOEntityWithDrawable.h"
+#import "OOCommodities.h"
+#import "OOSystemDescriptionManager.h"
 
 #if OOLITE_ESPEAK
 #include <espeak/speak_lib.h>
 #endif
 
 @class	GameController, CollisionRegion, MyOpenGLView, GuiDisplayGen,
-		Entity, ShipEntity, StationEntity, OOPlanetEntity, OOSunEntity,
-		PlayerEntity, OORoleSet;
+	Entity, ShipEntity, StationEntity, OOPlanetEntity, OOSunEntity,
+	OOVisualEffectEntity, PlayerEntity, OORoleSet, WormholeEntity, 
+	DockEntity, OOJSScript, OOWaypointEntity, OOSystemDescriptionManager;
 
 
 typedef BOOL (*EntityFilterPredicate)(Entity *entity, void *parameter);
@@ -79,6 +83,7 @@ enum
 	EQUIPMENT_EXTRA_INFO_INDEX				= 5
 };
 
+
 #define SHADERS_MIN SHADERS_OFF
 
 
@@ -94,8 +99,11 @@ enum
 
 #define KEY_TECHLEVEL						@"techlevel"
 #define KEY_ECONOMY							@"economy"
+#define KEY_ECONOMY_DESC					@"economy_description"
 #define KEY_GOVERNMENT						@"government"
+#define KEY_GOVERNMENT_DESC					@"government_description"
 #define KEY_POPULATION						@"population"
+#define KEY_POPULATION_DESC					@"population_description"
 #define KEY_PRODUCTIVITY					@"productivity"
 #define KEY_RADIUS							@"radius"
 #define KEY_NAME							@"name"
@@ -103,6 +111,8 @@ enum
 #define KEY_INHABITANTS						@"inhabitants"
 #define KEY_DESCRIPTION						@"description"
 #define KEY_SHORT_DESCRIPTION				@"short_description"
+#define KEY_PLANETNAME						@"planet_name"
+#define KEY_SUNNAME							@"sun_name"
 
 #define KEY_CHANCE							@"chance"
 #define KEY_PRICE							@"price"
@@ -115,24 +125,26 @@ enum
 #define KEY_EQUIPMENT_STARBOARD_WEAPON		@"starboard_weapon_type"
 #define KEY_EQUIPMENT_EXTRAS				@"extras"
 #define KEY_WEAPON_FACINGS					@"weapon_facings"
+#define KEY_RENOVATION_MULTIPLIER					@"renovation_multiplier"
 
 #define SHIPYARD_KEY_ID						@"id"
 #define SHIPYARD_KEY_SHIPDATA_KEY			@"shipdata_key"
 #define SHIPYARD_KEY_SHIP					@"ship"
 #define SHIPYARD_KEY_PRICE					@"price"
 #define SHIPYARD_KEY_PERSONALITY			@"personality"
-#define SHIPYARD_KEY_DESCRIPTION			@"description"
+// default passenger berth required space
+#define PASSENGER_BERTH_SPACE				5
 
 #define PLANETINFO_UNIVERSAL_KEY			@"universal"
-
-// Derived constants (MIN_ENTITY_UID, MAX_ENTITY_UID) are defined in OOTypes.h
-#define	UNIVERSE_MAX_ENTITIES				2048
+#define PLANETINFO_INTERSTELLAR_KEY			@"interstellar space"
 
 #define OOLITE_EXCEPTION_LOOPING			@"OoliteLoopingException"
 #define OOLITE_EXCEPTION_DATA_NOT_FOUND		@"OoliteDataNotFoundException"
 #define OOLITE_EXCEPTION_FATAL				@"OoliteFatalException"
 
-#define BILLBOARD_DEPTH						50000.0
+// the distance the sky backdrop is from the camera
+// though it appears at infinity
+#define BILLBOARD_DEPTH						75000.0
 
 #define TIME_ACCELERATION_FACTOR_MIN		0.0625f
 #define TIME_ACCELERATION_FACTOR_DEFAULT	1.0f
@@ -140,6 +152,12 @@ enum
 
 #define DEMO_LIGHT_POSITION 5000.0f, 25000.0f, -10000.0f
 
+#define MIN_DISTANCE_TO_BUOY			750.0f // don't add ships within this distance
+#define MIN_DISTANCE_TO_BUOY2			(MIN_DISTANCE_TO_BUOY * MIN_DISTANCE_TO_BUOY)
+
+// if this is changed, also change oolite-populator.js
+// once this number has been in a stable release, cannot easily be changed
+#define SYSTEM_REPOPULATION_INTERVAL 20.0f;
 
 #ifndef OO_LOCALIZATION_TOOLS
 #define OO_LOCALIZATION_TOOLS	1
@@ -150,16 +168,11 @@ enum
 #endif
 
 
-typedef OOUInteger	OOTechLevelID;		// 0..14, 99 is special. NSNotFound is used, so OOUInteger required.
-typedef uint8_t		OOGovernmentID;		// 0..7
-typedef uint8_t		OOEconomyID;		// 0..7
-
-
 @interface Universe: OOWeakRefObject
 {
 @public
 	// use a sorted list for drawing and other activities
-	Entity					*sortedEntities[UNIVERSE_MAX_ENTITIES];
+	Entity					*sortedEntities[UNIVERSE_MAX_ENTITIES + 1];	// One extra for padding; see -doRemoveEntity:.
 	unsigned				n_entities;
 	
 	int						cursor_row;
@@ -170,7 +183,7 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	GLfloat					stars_ambient[4];
 	
 @private
-	OOUInteger				_sessionID;
+	NSUInteger				_sessionID;
 	
 	// colors
 	GLfloat					sun_diffuse[4];
@@ -191,7 +204,8 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	
 	OOWeakReference			*_firstBeacon,
 							*_lastBeacon;
-	
+	NSMutableDictionary		*waypoints;
+
 	GLfloat					skyClearColor[4];
 	
 	NSString				*currentMessage;
@@ -204,13 +218,14 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	
 	BOOL					displayGUI;
 	BOOL					wasDisplayGUI;
-	BOOL					displayCursor;
 	
 	BOOL					autoSaveNow;
 	BOOL					autoSave;
 	BOOL					wireframeGraphics;
-	BOOL					reducedDetail;
-	OOShaderSetting			shaderEffectsLevel;
+	OOGraphicsDetail		detailLevel;
+// Above entry replaces these two
+//	BOOL					reducedDetail;
+//	OOShaderSetting			shaderEffectsLevel;
 	
 	BOOL					displayFPS;		
 			
@@ -221,32 +236,38 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	OOTimeAbsolute			demo_start_time;
 	GLfloat					demo_start_z;
 	int						demo_stage;
-	int						demo_ship_index;
+	NSUInteger				demo_ship_index;
+	NSUInteger				demo_ship_subindex;
 	NSArray					*demo_ships;
 	
 	GLfloat					main_light_position[4];
 	
 	BOOL					dumpCollisionInfo;
 	
-	NSDictionary			*commodityLists;		// holds data on commodities for various types of station, loaded at initialisation
-	NSArray					*commodityData;			// holds data on commodities extracted from commodityLists
-	
-	NSDictionary			*illegalGoods;			// holds the legal penalty for illicit commodities, loaded at initialisation
+	OOCommodities			*commodities;
+	OOCommodityMarket		*commodityMarket;
+
+
 	NSDictionary			*_descriptions;			// holds descriptive text for lots of stuff, loaded at initialisation
 	NSDictionary			*customSounds;			// holds descriptive audio for lots of stuff, loaded at initialisation
 	NSDictionary			*characters;			// holds descriptons of characters
-	NSDictionary			*planetInfo;			// holds overrides for individual planets, keyed by "g# p#" where g# is the galaxy number 0..7 and p# the planet number 0..255
+	NSArray					*_scenarios;			// game start scenarios
+	NSDictionary			*globalSettings;		// miscellaneous global game settings
+	OOSystemDescriptionManager	*systemManager; // planetinfo data manager
 	NSDictionary			*missiontext;			// holds descriptive text for missions, loaded at initialisation
 	NSArray					*equipmentData;			// holds data on available equipment, loaded at initialisation
-	NSSet					*pirateVictimRoles;		// Roles listed in pirateVictimRoles.plist.
+//	NSSet					*pirateVictimRoles;		// Roles listed in pirateVictimRoles.plist.
+	NSDictionary			*roleCategories;		// Categories for roles from role-categories.plist, extending the old pirate-victim-roles.plist
 	NSDictionary			*autoAIMap;				// Default AIs for roles from autoAImap.plist.
 	NSDictionary			*screenBackgrounds;		// holds filenames for various screens backgrounds, loaded at initialisation
+	NSDictionary			*explosionSettings;		// explosion settings from explosions.plist
+
+	NSDictionary      *cargoPods; // template cargo pods
+
+	OOGalaxyID				galaxyID;
+	OOSystemID				systemID;
+	OOSystemID				targetSystemID;
 	
-	Random_Seed				galaxy_seed;
-	Random_Seed				system_seed;
-	Random_Seed				target_system_seed;
-	
-	Random_Seed				systems[256];			// hold pregenerated universe info
 	NSString				*system_names[256];		// hold pregenerated universe info
 	BOOL					system_found[256];		// holds matches for input strings
 	
@@ -258,10 +279,18 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	OOPlanetEntity			*cachedPlanet;
 	OOSunEntity				*cachedSun;
 	NSMutableArray			*allPlanets;
+	NSMutableSet			*allStations;
 	
+	float					ambientLightLevel;
+	
+	NSMutableDictionary		*populatorSettings;
+	OOTimeDelta		next_repopulation;
+	NSString		*system_repopulator;
+	BOOL			deterministic_population;
+
 	NSArray					*closeSystems;
 	
-	BOOL					strict;
+	NSString				*useAddOns;
 	
 	BOOL					no_update;
 	
@@ -269,10 +298,6 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	double					timeAccelerationFactor;
 #endif
 	
-	NSMutableDictionary		*localPlanetInfoOverrides;
-	
-	NSException				*exception;
-	
 	NSMutableArray			*activeWormholes;
 	
 	NSMutableArray			*characterPool;
@@ -287,7 +312,7 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	
 #if OOLITE_SPEECH_SYNTH
 #if OOLITE_MAC_OS_X
-	NSSpeechSynthesizer		*speechSynthesizer;		// use this from OS X 10.3 onwards
+	NSSpeechSynthesizer		*speechSynthesizer;
 #elif OOLITE_ESPEAK
 	const espeak_VOICE		**espeak_voices;
 	unsigned int			espeak_voice_count;
@@ -300,39 +325,61 @@ typedef uint8_t		OOEconomyID;		// 0..7
 #endif
 	BOOL					doProcedurallyTexturedPlanets;
 	
+	GLfloat					frustum[6][4];
+	
+	NSMutableDictionary		*conditionScripts;
+	
 	BOOL					_pauseMessage;
 	BOOL					_autoCommLog;
 	BOOL					_permanentCommLog;
+	BOOL					_permanentMessageLog;
+	BOOL					_witchspaceBreakPattern;
+	BOOL					_dockingClearanceProtocolActive;
+	BOOL					_doingStartUp;
 }
 
 - (id)initWithGameView:(MyOpenGLView *)gameView;
 
 // SessionID: a value that's incremented when the game is reset.
-- (OOUInteger) sessionID;
+- (NSUInteger) sessionID;
 
 - (BOOL) doProcedurallyTexturedPlanets;
 - (void) setDoProcedurallyTexturedPlanets:(BOOL) value;
 
-- (BOOL) strict;
-- (void) setStrict:(BOOL) value;
-- (void) setStrict:(BOOL)value fromSaveGame: (BOOL)saveGame;
+- (NSString *) useAddOns;
+- (BOOL) setUseAddOns:(NSString *)newUse fromSaveGame: (BOOL)saveGame;
+- (BOOL) setUseAddOns:(NSString *) newUse fromSaveGame:(BOOL) saveGame forceReinit:(BOOL)force;
 
-- (void) reinitAndShowDemo:(BOOL)showDemo;
+- (void) setUpSettings;
 
-- (int) entityCount;
+- (BOOL) reinitAndShowDemo:(BOOL)showDemo;
+
+- (BOOL) doingStartUp;	// True during initial game startup (not reset).
+
+- (NSUInteger) entityCount;
 #ifndef NDEBUG
 - (void) debugDumpEntities;
 - (NSArray *) entityList;
 #endif
 
 - (void) pauseGame;
-- (BOOL) isGamePaused;
 
+- (void) carryPlayerOn:(StationEntity*)carrier inWormhole:(WormholeEntity*)wormhole;
 - (void) setUpUniverseFromStation;
 - (void) setUpUniverseFromWitchspace;
 - (void) setUpUniverseFromMisjump;
 - (void) setUpWitchspace;
+- (void) setUpWitchspaceBetweenSystem:(OOSystemID)s1 andSystem:(OOSystemID)s2;
 - (void) setUpSpace;
+- (void) populateNormalSpace;
+- (void) clearSystemPopulator;
+- (BOOL) deterministicPopulation;
+- (void) populateSystemFromDictionariesWithSun:(OOSunEntity *)sun andPlanet:(OOPlanetEntity *)planet;
+- (NSDictionary *) getPopulatorSettings;
+- (void) setPopulatorSetting:(NSString *)key to:(NSDictionary *)setting;
+- (HPVector) locationByCode:(NSString *)code withSun:(OOSunEntity *)sun andPlanet:(OOPlanetEntity *)planet;
+- (void) setAmbientLightLevel:(float)newValue;
+- (float) ambientLightLevel;
 - (void) setLighting;
 - (void) forceLightSwitch;
 - (void) setMainLightPosition: (Vector) sunPos;
@@ -340,38 +387,53 @@ typedef uint8_t		OOEconomyID;		// 0..7
 
 - (void) makeSunSkimmer:(ShipEntity *) ship andSetAI:(BOOL)setAI;
 - (void) addShipWithRole:(NSString *) desc nearRouteOneAt:(double) route_fraction;
-- (Vector) coordinatesForPosition:(Vector) pos withCoordinateSystem:(NSString *) system returningScalar:(GLfloat*) my_scalar;
-- (NSString *) expressPosition:(Vector) pos inCoordinateSystem:(NSString *) system;
-- (Vector) legacyPositionFrom:(Vector) pos asCoordinateSystem:(NSString *) system;
-- (Vector) coordinatesFromCoordinateSystemString:(NSString *) system_x_y_z;
-- (BOOL) addShipWithRole:(NSString *) desc nearPosition:(Vector) pos withCoordinateSystem:(NSString *) system;
-- (BOOL) addShips:(int) howMany withRole:(NSString *) desc atPosition:(Vector) pos withCoordinateSystem:(NSString *) system;
-- (BOOL) addShips:(int) howMany withRole:(NSString *) desc nearPosition:(Vector) pos withCoordinateSystem:(NSString *) system;
-- (BOOL) addShips:(int) howMany withRole:(NSString *) desc nearPosition:(Vector) pos withCoordinateSystem:(NSString *) system withinRadius:(GLfloat) radius;
+- (HPVector) coordinatesForPosition:(HPVector) pos withCoordinateSystem:(NSString *) system returningScalar:(GLfloat*) my_scalar;
+- (NSString *) expressPosition:(HPVector) pos inCoordinateSystem:(NSString *) system;
+- (HPVector) legacyPositionFrom:(HPVector) pos asCoordinateSystem:(NSString *) system;
+- (HPVector) coordinatesFromCoordinateSystemString:(NSString *) system_x_y_z;
+- (BOOL) addShipWithRole:(NSString *) desc nearPosition:(HPVector) pos withCoordinateSystem:(NSString *) system;
+- (BOOL) addShips:(int) howMany withRole:(NSString *) desc atPosition:(HPVector) pos withCoordinateSystem:(NSString *) system;
+- (BOOL) addShips:(int) howMany withRole:(NSString *) desc nearPosition:(HPVector) pos withCoordinateSystem:(NSString *) system;
+- (BOOL) addShips:(int) howMany withRole:(NSString *) desc nearPosition:(HPVector) pos withCoordinateSystem:(NSString *) system withinRadius:(GLfloat) radius;
 - (BOOL) addShips:(int) howMany withRole:(NSString *) desc intoBoundingBox:(BoundingBox) bbox;
 - (BOOL) spawnShip:(NSString *) shipdesc;
 - (void) witchspaceShipWithPrimaryRole:(NSString *)role;
 - (ShipEntity *) spawnShipWithRole:(NSString *) desc near:(Entity *) entity;
 
-- (ShipEntity *) addShipAt:(Vector)pos withRole:(NSString *)role withinRadius:(GLfloat)radius;
-- (NSArray *) addShipsAt:(Vector)pos withRole:(NSString *)role quantity:(unsigned)count withinRadius:(GLfloat)radius asGroup:(BOOL)isGroup;
+- (OOVisualEffectEntity *) addVisualEffectAt:(HPVector)pos withKey:(NSString *)key;
+- (ShipEntity *) addShipAt:(HPVector)pos withRole:(NSString *)role withinRadius:(GLfloat)radius;
+- (NSArray *) addShipsAt:(HPVector)pos withRole:(NSString *)role quantity:(unsigned)count withinRadius:(GLfloat)radius asGroup:(BOOL)isGroup;
 - (NSArray *) addShipsToRoute:(NSString *)route withRole:(NSString *)role quantity:(unsigned)count routeFraction:(double)routeFraction asGroup:(BOOL)isGroup;
 
 - (BOOL) roleIsPirateVictim:(NSString *)role;
+- (BOOL) role:(NSString *)role isInCategory:(NSString *)category;
 
+- (void) forceWitchspaceEntries;
 - (void) addWitchspaceJumpEffectForShip:(ShipEntity *)ship;
+- (GLfloat) safeWitchspaceExitDistance;
+
+- (void) setUpBreakPattern:(HPVector)pos orientation:(Quaternion)q forDocking:(BOOL)forDocking;
+- (BOOL) witchspaceBreakPattern;
+- (void) setWitchspaceBreakPattern:(BOOL)newValue;
+
+- (BOOL) dockingClearanceProtocolActive;
+- (void) setDockingClearanceProtocolActive:(BOOL)newValue;
 
-- (void) setUpBreakPattern:(Vector)pos orientation:(Quaternion)q forDocking:(BOOL)forDocking;
 - (void) handleGameOver;
 
 - (void) setupIntroFirstGo:(BOOL)justCobra;
 - (void) selectIntro2Previous;
 - (void) selectIntro2Next;
+- (void) selectIntro2PreviousCategory;
+- (void) selectIntro2NextCategory;
 
 - (StationEntity *) station;
 - (OOPlanetEntity *) planet;
 - (OOSunEntity *) sun;
 - (NSArray *) planets;	// Note: does not include sun.
+- (NSArray *) stations; // includes main station
+- (NSArray *) wormholes; 
+- (StationEntity *) stationWithRole:(NSString *)role andPosition:(HPVector)position;
 
 // Turn main station into just another station, for blowUpStation.
 - (void) unMagicMainStation;
@@ -379,9 +441,13 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (StationEntity *) stationFriendlyTo:(ShipEntity *) ship;
 
 - (void) resetBeacons;
-- (ShipEntity *) firstBeacon;
-- (ShipEntity *) lastBeacon;
-- (void) setNextBeacon:(ShipEntity *) beaconShip;
+- (Entity <OOBeaconEntity> *) firstBeacon;
+- (Entity <OOBeaconEntity> *) lastBeacon;
+- (void) setNextBeacon:(Entity <OOBeaconEntity> *) beaconShip;
+- (void) clearBeacon:(Entity <OOBeaconEntity> *) beaconShip;
+
+- (NSDictionary *) currentWaypoints;
+- (void) defineWaypoint:(NSDictionary *)definition forKey:(NSString *)key;
 
 - (GLfloat *) skyClearColor;
 // Note: the alpha value is also air resistance!
@@ -391,9 +457,14 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (BOOL) breakPatternHide;
 
 - (NSString *) randomShipKeyForRoleRespectingConditions:(NSString *)role;
-- (ShipEntity *) newShipWithRole:(NSString *)role;		// Selects ship using role weights, applies auto_ai, respects conditions
-- (ShipEntity *) newShipWithName:(NSString *)shipKey;	// Does not apply auto_ai or respect conditions
-- (ShipEntity *) newShipWithName:(NSString *)shipKey usePlayerProxy:(BOOL)usePlayerProxy;	// If usePlayerProxy, non-carriers are instantiated as ProxyPlayerEntity.
+- (ShipEntity *) newShipWithRole:(NSString *)role OO_RETURNS_RETAINED;		// Selects ship using role weights, applies auto_ai, respects conditions
+- (ShipEntity *) newShipWithName:(NSString *)shipKey OO_RETURNS_RETAINED;	// Does not apply auto_ai or respect conditions
+- (ShipEntity *) newSubentityWithName:(NSString *)shipKey andScaleFactor:(float)scale OO_RETURNS_RETAINED;	// Does not apply auto_ai or respect conditions
+- (OOVisualEffectEntity *) newVisualEffectWithName:(NSString *)effectKey OO_RETURNS_RETAINED;
+- (DockEntity *) newDockWithName:(NSString *)shipKey andScaleFactor:(float)scale OO_RETURNS_RETAINED;	// Does not apply auto_ai or respect conditions
+- (ShipEntity *) newShipWithName:(NSString *)shipKey usePlayerProxy:(BOOL)usePlayerProxy OO_RETURNS_RETAINED;	// If usePlayerProxy, non-carriers are instantiated as ProxyPlayerEntity.
+- (ShipEntity *) newShipWithName:(NSString *)shipKey usePlayerProxy:(BOOL)usePlayerProxy isSubentity:(BOOL)isSubentity OO_RETURNS_RETAINED;
+- (ShipEntity *) newShipWithName:(NSString *)shipKey usePlayerProxy:(BOOL)usePlayerProxy isSubentity:(BOOL)isSubentity andScaleFactor:(float)scale OO_RETURNS_RETAINED;
 
 - (Class) shipClassForShipDictionary:(NSDictionary *)dict;
 
@@ -403,31 +474,37 @@ typedef uint8_t		OOEconomyID;		// 0..7
 
 - (OOCreditsQuantity) getEquipmentPriceForKey:(NSString *) eq_key;
 
-- (int) legalStatusOfManifest:(NSArray *)manifest;
+- (OOCommodities *) commodities;
 
-- (NSArray *) getContainersOfGoods:(OOCargoQuantity)how_many scarce:(BOOL)scarce;
-- (NSArray *) getContainersOfDrugs:(OOCargoQuantity) how_many;
-- (NSArray *) getContainersOfCommodity:(NSString*) commodity_name :(OOCargoQuantity) how_many;
+- (ShipEntity *) reifyCargoPod:(ShipEntity *)cargoObj;
+- (ShipEntity *) cargoPodFromTemplate:(ShipEntity *)cargoObj;
+- (NSArray *) getContainersOfGoods:(OOCargoQuantity)how_many scarce:(BOOL)scarce legal:(BOOL)legal;
+- (NSArray *) getContainersOfCommodity:(OOCommodityType) commodity_name :(OOCargoQuantity) how_many;
 - (void) fillCargopodWithRandomCargo:(ShipEntity *)cargopod;
 
-- (OOCargoType) getRandomCommodity;
-- (OOCargoQuantity) getRandomAmountOfCommodity:(OOCargoType) co_type;
+- (NSString *) getRandomCommodity;
+- (OOCargoQuantity) getRandomAmountOfCommodity:(OOCommodityType) co_type;
 
-- (NSArray *) commodityDataForType:(OOCargoType)type;
-- (OOCargoType) commodityForName:(NSString *) co_name;
-- (NSString *) symbolicNameForCommodity:(OOCargoType) co_type;
-- (NSString *) displayNameForCommodity:(OOCargoType) co_type;
-- (OOMassUnit) unitsForCommodity:(OOCargoType) co_type;
-- (NSString *) describeCommodity:(OOCargoType) co_type amount:(OOCargoQuantity) co_amount;
+- (NSDictionary *) commodityDataForType:(OOCommodityType)type;
+- (NSString *) displayNameForCommodity:(OOCommodityType)co_type;
+- (NSString *) describeCommodity:(OOCommodityType)co_type amount:(OOCargoQuantity) co_amount;
 
 - (void) setGameView:(MyOpenGLView *)view;
 - (MyOpenGLView *) gameView;
 - (GameController *) gameController;
 - (NSDictionary *) gameSettings;
 
+- (void) useGUILightSource:(BOOL)GUILight;
+
 - (void) drawUniverse;
+
+- (void) defineFrustum;
+- (BOOL) viewFrustumIntersectsSphereAt:(Vector)position withRadius:(GLfloat)radius;
+
 - (void) drawMessage;
 
+- (void) drawWatermarkString:(NSString *)watermarkString;
+
 // Used to draw subentities. Should be getting this from camera.
 - (OOMatrix) viewMatrix;
 
@@ -441,14 +518,17 @@ typedef uint8_t		OOEconomyID;		// 0..7
 
 - (ShipEntity *) makeDemoShipWithRole:(NSString *)role spinning:(BOOL)spinning;
 
-- (BOOL) isVectorClearFromEntity:(Entity *) e1 toDistance:(double)dist fromPoint:(Vector) p2;
-- (Entity*) hazardOnRouteFromEntity:(Entity *) e1 toDistance:(double)dist fromPoint:(Vector) p2;
-- (Vector) getSafeVectorFromEntity:(Entity *) e1 toDistance:(double)dist fromPoint:(Vector) p2;
+- (BOOL) isVectorClearFromEntity:(Entity *) e1 toDistance:(double)dist fromPoint:(HPVector) p2;
+- (Entity*) hazardOnRouteFromEntity:(Entity *) e1 toDistance:(double)dist fromPoint:(HPVector) p2;
+- (HPVector) getSafeVectorFromEntity:(Entity *) e1 toDistance:(double)dist fromPoint:(HPVector) p2;
 
-- (ShipEntity *) getFirstShipHitByLaserFromShip:(ShipEntity *)srcEntity inView:(OOViewID)viewdir offset:(Vector)offset rangeFound:(GLfloat*)range_ptr;
-- (Entity *) getFirstEntityTargetedByPlayer;
+- (ShipEntity *) addWreckageFrom:(ShipEntity *)ship withRole:(NSString *)wreckRole at:(HPVector)rpos scale:(GLfloat)scale lifetime:(GLfloat)lifetime;
+- (void) addLaserHitEffectsAt:(HPVector)pos against:(ShipEntity *)target damage:(float)damage color:(OOColor *)color;
+- (ShipEntity *) firstShipHitByLaserFromShip:(ShipEntity *)srcEntity inDirection:(OOWeaponFacing)direction offset:(Vector)offset gettingRangeFound:(GLfloat*)range_ptr;
+- (Entity *) firstEntityTargetedByPlayer;
+- (Entity *) firstEntityTargetedByPlayerPrecisely;
 
-- (NSArray *) getEntitiesWithinRange:(double)range ofEntity:(Entity *)entity;
+- (NSArray *) entitiesWithinRange:(double)range ofEntity:(Entity *)entity;
 - (unsigned) countShipsWithRole:(NSString *)role inRange:(double)range ofEntity:(Entity *)entity;
 - (unsigned) countShipsWithRole:(NSString *)role;
 - (unsigned) countShipsWithPrimaryRole:(NSString *)role inRange:(double)range ofEntity:(Entity *)entity;
@@ -476,6 +556,10 @@ typedef uint8_t		OOEconomyID;		// 0..7
 									  parameter:(void *)parameter
 										inRange:(double)range
 									   ofEntity:(Entity *)entity;
+- (NSMutableArray *) findVisualEffectsMatchingPredicate:(EntityFilterPredicate)predicate
+									  parameter:(void *)parameter
+										inRange:(double)range
+									   ofEntity:(Entity *)entity;
 - (id) nearestEntityMatchingPredicate:(EntityFilterPredicate)predicate
 							parameter:(void *)parameter
 					 relativeToEntity:(Entity *)entity;
@@ -491,8 +575,9 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (NSString*) collisionDescription;
 - (void) dumpCollisions;
 
-- (void) setViewDirection:(OOViewID) vd;
 - (OOViewID) viewDirection;
+- (void) setViewDirection:(OOViewID)vd;
+- (void) enterGUIViewModeWithMouseInteraction:(BOOL)mouseInteraction;	// Use instead of setViewDirection:VIEW_GUI_DISPLAY
 
 - (NSString *) soundNameForCustomSoundKey:(NSString *)key;
 - (NSDictionary *) screenTextureDescriptorForKey:(NSString *)key;
@@ -519,99 +604,87 @@ typedef uint8_t		OOEconomyID;		// 0..7
 
 ///////////////////////////////////////
 
-- (void) setGalaxySeed:(Random_Seed) gal_seed;
-- (void) setGalaxySeed:(Random_Seed) gal_seed andReinit:(BOOL) forced;
+- (void) setGalaxyTo:(OOGalaxyID) g;
+- (void) setGalaxyTo:(OOGalaxyID) g andReinit:(BOOL) forced;
 
-- (void) setSystemTo:(Random_Seed) s_seed;
+- (void) setSystemTo:(OOSystemID) s;
 
-- (Random_Seed) systemSeed;
-- (Random_Seed) systemSeedForSystemNumber:(OOSystemID) n;
-- (Random_Seed) systemSeedForSystemName:(NSString *)sysname;
-- (OOSystemID) systemIDForSystemSeed:(Random_Seed)seed;
 - (OOSystemID) currentSystemID;
 
 - (NSDictionary *) descriptions;
 - (NSDictionary *) characters;
 - (NSDictionary *) missiontext;
+- (NSArray *) scenarios;
+- (NSDictionary *) explosionSetting:(NSString *)explosion;
+
+- (OOSystemDescriptionManager *) systemManager;
 
 - (NSString *)descriptionForKey:(NSString *)key;	// String, or random item from array
 - (NSString *)descriptionForArrayKey:(NSString *)key index:(unsigned)index;	// Indexed item from array
 - (BOOL) descriptionBooleanForKey:(NSString *)key;	// Boolean from descriptions.plist, for configuration.
 
-- (NSString *) keyForPlanetOverridesForSystemSeed:(Random_Seed) s_seed inGalaxySeed:(Random_Seed) g_seed;
-- (NSString *) keyForInterstellarOverridesForSystemSeeds:(Random_Seed) s_seed1 :(Random_Seed) s_seed2 inGalaxySeed:(Random_Seed) g_seed;
-- (NSDictionary *) generateSystemData:(Random_Seed) system_seed;
-- (NSDictionary *) generateSystemData:(Random_Seed) s_seed useCache:(BOOL) useCache;
+- (NSString *) keyForPlanetOverridesForSystem:(OOSystemID) s inGalaxy:(OOGalaxyID) g;
+- (NSString *) keyForInterstellarOverridesForSystems:(OOSystemID) s1 :(OOSystemID) s2 inGalaxy:(OOGalaxyID) g;
+- (NSDictionary *) generateSystemData:(OOSystemID) s;
+- (NSDictionary *) generateSystemData:(OOSystemID) s useCache:(BOOL) useCache;
 - (NSDictionary *) currentSystemData;	// Same as generateSystemData:systemSeed unless in interstellar space.
+
 - (BOOL) inInterstellarSpace;
 
-- (void)setObject:(id)object forKey:(NSString *)key forPlanetKey:(NSString *)planetKey;
-
-- (void) setSystemDataKey:(NSString*) key value:(NSObject*) object;
-- (void) setSystemDataForGalaxy:(OOGalaxyID) gnum planet:(OOSystemID) pnum key:(NSString *)key value:(id)object;
+- (void) setSystemDataKey:(NSString*) key value:(NSObject*) object fromManifest:(NSString *)manifest;
+- (void) setSystemDataForGalaxy:(OOGalaxyID) gnum planet:(OOSystemID) pnum key:(NSString *)key value:(id)object fromManifest:(NSString *)manifest forLayer:(OOSystemLayer)layer;
 - (id) systemDataForGalaxy:(OOGalaxyID) gnum planet:(OOSystemID) pnum key:(NSString *)key;
 - (NSArray *) systemDataKeysForGalaxy:(OOGalaxyID)gnum planet:(OOSystemID)pnum;
-- (NSString *) getSystemName:(Random_Seed) s_seed;
-- (OOGovernmentID) getSystemGovernment:(Random_Seed)s_seed;
-- (NSString *) getSystemInhabitants:(Random_Seed) s_seed;
-- (NSString *) getSystemInhabitants:(Random_Seed) s_seed plural:(BOOL)plural;
-- (NSString *) generateSystemName:(Random_Seed) system_seed;
-- (NSString *) generatePhoneticSystemName:(Random_Seed) s_seed;
-- (NSString *) generateSystemInhabitants:(Random_Seed) s_seed plural:(BOOL)plural;
-- (NSPoint) coordinatesForSystem:(Random_Seed)s_seed;
-- (Random_Seed) findSystemAtCoords:(NSPoint) coords withGalaxySeed:(Random_Seed) gal_seed;
-- (Random_Seed) findSystemFromName:(NSString *) sysName;
+- (NSString *) getSystemName:(OOSystemID) sys;
+- (OOGovernmentID) getSystemGovernment:(OOSystemID) sys;
+- (NSString *) getSystemInhabitants:(OOSystemID) sys;
+- (NSString *) getSystemInhabitants:(OOSystemID) sys plural:(BOOL)plural;
+
+- (NSPoint) coordinatesForSystem:(OOSystemID)s;
+- (OOSystemID) findSystemAtCoords:(NSPoint) coords withGalaxy:(OOGalaxyID) gal;
+- (OOSystemID) findSystemFromName:(NSString *) sysName;
 
 /**
  * Finds systems within range.  If range is greater than 7.0LY then only look within 7.0LY.
  */
 - (NSMutableArray *) nearbyDestinationsWithinRange:(double) range;
 
-- (Random_Seed) findNeighbouringSystemToCoords:(NSPoint) coords withGalaxySeed:(Random_Seed) gal_seed;
-- (Random_Seed) findConnectedSystemAtCoords:(NSPoint) coords withGalaxySeed:(Random_Seed) gal_seed;
-- (int) findSystemNumberAtCoords:(NSPoint) coords withGalaxySeed:(Random_Seed) gal_seed;
+- (OOSystemID) findNeighbouringSystemToCoords:(NSPoint) coords withGalaxy:(OOGalaxyID) gal;
+- (OOSystemID) findConnectedSystemAtCoords:(NSPoint) coords withGalaxy:(OOGalaxyID) gal;
+- (OOSystemID) findSystemNumberAtCoords:(NSPoint) coords withGalaxy:(OOGalaxyID) gal;
 - (NSPoint) findSystemCoordinatesWithPrefix:(NSString *) p_fix;
 - (NSPoint) findSystemCoordinatesWithPrefix:(NSString *) p_fix exactMatch:(BOOL) exactMatch;
 - (BOOL*) systemsFound;
 - (NSString*) systemNameIndex:(OOSystemID) index;
 - (NSDictionary *) routeFromSystem:(OOSystemID) start toSystem:(OOSystemID) goal optimizedBy:(OORouteType) optimizeBy;
 - (NSArray *) neighboursToSystem:(OOSystemID) system_number;
-- (NSArray *) neighboursToRandomSeed:(Random_Seed) seed;
 
-- (NSMutableDictionary *) localPlanetInfoOverrides;
-- (void) setLocalPlanetInfoOverrides:(NSDictionary*) dict;
+- (void) preloadPlanetTexturesForSystem:(OOSystemID)system;
+- (void) preloadSounds;
 
-- (void) preloadPlanetTexturesForSystem:(Random_Seed)seed;
-
-- (NSDictionary *) planetInfo;
+- (NSDictionary *) globalSettings;
 
 - (NSArray *) equipmentData;
-- (NSDictionary *) commodityLists;
-- (NSArray *) commodityData;
-
-- (BOOL) generateEconomicDataWithEconomy:(OOEconomyID) economy andRandomFactor:(int) random_factor;
-- (NSArray *) commodityDataForEconomy:(OOEconomyID) economy andStation:(StationEntity *)some_station andRandomFactor:(int) random_factor;
+- (OOCommodityMarket *) commodityMarket;
 
 - (NSString *) timeDescription:(OOTimeDelta) interval;
 - (NSString *) shortTimeDescription:(OOTimeDelta) interval;
 
-- (Random_Seed) marketSeed;
-- (NSArray *) passengersForLocalSystemAtTime:(OOTimeAbsolute) current_time;
-- (NSArray *) contractsForLocalSystemAtTime:(OOTimeAbsolute) current_time;
-- (NSArray *) shipsForSaleForSystem:(Random_Seed) s_seed withTL:(OOTechLevelID) specialTL atTime:(OOTimeAbsolute) current_time;
+- (void) loadStationMarkets:(NSArray *)marketData;
+- (NSArray *) getStationMarkets;
+
+- (NSArray *) shipsForSaleForSystem:(OOSystemID) s withTL:(OOTechLevelID) specialTL atTime:(OOTimeAbsolute) current_time;
 
 /* Calculate base cost, before depreciation */
 - (OOCreditsQuantity) tradeInValueForCommanderDictionary:(NSDictionary*) cmdr_dict;
 
 - (NSString*) brochureDescriptionWithDictionary:(NSDictionary*) dict standardEquipment:(NSArray*) extras optionalEquipment:(NSArray*) options;
 
-- (Vector) getWitchspaceExitPosition;
-- (Vector) randomizeFromSeedAndGetWitchspaceExitPosition;
-- (Vector) getWitchspaceExitPositionResettingRandomSeed:(BOOL)resetSeed;
+- (HPVector) getWitchspaceExitPosition;
 - (Quaternion) getWitchspaceExitRotation;
 
-- (Vector) getSunSkimStartPositionForShip:(ShipEntity*) ship;
-- (Vector) getSunSkimEndPositionForShip:(ShipEntity*) ship;
+- (HPVector) getSunSkimStartPositionForShip:(ShipEntity*) ship;
+- (HPVector) getSunSkimEndPositionForShip:(ShipEntity*) ship;
 
 - (NSArray*) listBeaconsWithCode:(NSString*) code;
 
@@ -627,9 +700,6 @@ typedef uint8_t		OOEconomyID;		// 0..7
 
 - (void) resetCommsLogColor;
 
-- (void) setDisplayCursor:(BOOL) value;
-- (BOOL) displayCursor;
-
 - (void) setDisplayText:(BOOL) value;
 - (BOOL) displayGUI;
 
@@ -642,16 +712,12 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (void) setWireframeGraphics:(BOOL) value;
 - (BOOL) wireframeGraphics;
 
-- (void) setReducedDetail:(BOOL) value;
-- (void) setReducedDetail:(BOOL) value transiently:(BOOL)transiently;
 - (BOOL) reducedDetail;
-
-- (void) setShaderEffectsLevel:(OOShaderSetting)value;
-- (void) setShaderEffectsLevel:(OOShaderSetting)value transiently:(BOOL)transiently;
-- (OOShaderSetting) shaderEffectsLevel;
+- (void) setDetailLevel:(OOGraphicsDetail)value;
+- (OOGraphicsDetail) detailLevel;
 - (BOOL) useShaders;
 
-- (void) handleOoliteException:(NSException*) ooliteException;
+- (void) handleOoliteException:(NSException *)ooliteException;
 
 - (GLfloat)airResistanceFactor;
 
@@ -687,10 +753,16 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (BOOL) permanentCommLog;
 - (void) setPermanentCommLog:(BOOL)value;
 - (void) setAutoCommLog:(BOOL)value;
+- (BOOL) permanentMessageLog;
+- (void) setPermanentMessageLog:(BOOL)value;
 
 - (BOOL) blockJSPlayerShipProps;
 - (void) setBlockJSPlayerShipProps:(BOOL)value;
 
+- (void) loadConditionScripts;
+- (void) addConditionScripts:(NSEnumerator *)scripts;
+- (OOJSScript *) getConditionScript:(NSString *)scriptname;
+
 @end
 
 
@@ -698,20 +770,26 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	The purpose of this is that it makes UNIVERSE essentially a read-only
 	global with zero overhead.
 */
-OOINLINE Universe *GetUniverse(void) INLINE_CONST_FUNC;
-OOINLINE Universe *GetUniverse(void)
+OOINLINE Universe *OOGetUniverse(void) INLINE_CONST_FUNC;
+OOINLINE Universe *OOGetUniverse(void)
 {
 	extern Universe *gSharedUniverse;
 	return gSharedUniverse;
 }
-#define UNIVERSE GetUniverse()
+#define UNIVERSE OOGetUniverse()
 
 
 // Only for use with string literals, and only for looking up strings.
-NSString *DESC_(NSString *key);
-NSString *DESC_PLURAL_(NSString *key, int count);
-#define DESC(key)	(DESC_(key ""))
-#define DESC_PLURAL(key,count)	(DESC_PLURAL_(key, count))
+// DESC() is deprecated in favour of OOExpandKey() except in known performance-
+// critical contexts.
+#define DESC(key)	(OOLookUpDescriptionPRIV(key ""))
+#define DESC_PLURAL(key,count)	(OOLookUpPluralDescriptionPRIV(key "", count))
+
+// Not for direct use.
+NSComparisonResult populatorPrioritySort(id a, id b, void *context);
+NSComparisonResult equipmentSort(id a, id b, void *context);
+NSString *OOLookUpDescriptionPRIV(NSString *key);
+NSString *OOLookUpPluralDescriptionPRIV(NSString *key, NSInteger count);
 
 
 @interface OOSound (OOCustomSounds)
